﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using POS.Framework;

namespace POS.CashPlugin
{
    partial class CashPlugin
    {

        internal class PaymentButton : Button
        {

            private bool bool_paid;
            private string str_code;

            public bool Paid
            {
                get
                {
                    return bool_paid;
                }
                set
                {
                    bool_paid = value;
                }
            }

            public string Code
            {
                get
                {
                    return str_code;
                }
                set
                {
                    str_code = value;
                }
            }
        }

        internal class PanelNoScrollOnFocus : Panel
        {

            protected override Point ScrollToControl(Control activeControl)
            {
                return base.DisplayRectangle.Location;
            }

        }

        internal class DataGridViewButtonProductsColumn : DataGridViewColumn
        {


            public override DataGridViewCell CellTemplate
            {
                get
                {
                    return base.CellTemplate;
                }
                set
                {
                    if (value != null && !(value.GetType().IsAssignableFrom(typeof(DataGridViewButtonProductsCell))))
                    {
                        throw new InvalidCastException("Must be a ButtonProductsCell");
                    }
                    base.CellTemplate = value;
                }
            }

            public override object Clone()
            {

                DataGridViewButtonProductsColumn col;

                col = (DataGridViewButtonProductsColumn)base.Clone();

                return col;

            }


            public DataGridViewButtonProductsColumn()
                : base(new DataGridViewButtonProductsCell())
            {


            }

        }

        internal class DataGridViewButtonProductsCell : DataGridViewButtonCell
        {

            private string str_comm;

            public string Command
            {
                get
                {
                    return str_comm;
                }
            }

            public override object Clone()
            {

                DataGridViewButtonProductsCell c = (DataGridViewButtonProductsCell)base.Clone();

                c.str_comm = str_comm;

                return c;

            }

            protected override void Paint(
                Graphics graphics,
                Rectangle clipBounds,
                Rectangle cellBounds,
                int rowIndex,
                DataGridViewElementStates cellState,
                object value,
                object formattedValue,
                string errorText,
                DataGridViewCellStyle cellStyle,
                DataGridViewAdvancedBorderStyle advancedBorderStyle,
                DataGridViewPaintParts paintParts)
            {

                int primo = value.ToString().IndexOf("/");
                int secondo = value.ToString().IndexOf("/", primo + 1);

                string[] temp2 = value.ToString().Substring(0, primo).Split(',');
                string colour = temp2[0];
                string icon = temp2[1];

                str_comm = value.ToString().Substring(primo + 1, secondo - primo - 1);

                value = value.ToString().Substring(secondo + 1).ToUpper();

                formattedValue = value;

                if (string.IsNullOrEmpty(colour) || colour.Equals("0")) colour = "164";

                cellStyle.BackColor = Color.FromKnownColor((KnownColor)Convert.ToInt32(colour));

                if (cellStyle.SelectionBackColor != Color.Lime)
                {
                    cellStyle.SelectionBackColor = cellStyle.BackColor;
                }

                base.Paint(graphics, clipBounds, cellBounds, rowIndex, cellState,
          value, formattedValue, errorText, cellStyle,
          advancedBorderStyle, paintParts);


                string path = Engine.dirpath + "images\\" + icon + ".gif";

                if (!(System.IO.File.Exists(path)))
                {
                    path = Engine.dirpath + "images\\0.gif";
                }

                graphics.DrawImage(Image.FromFile(path), new Point(cellBounds.X + 34, cellBounds.Y + 6));

            }

        }


        private void InitializeUI(TabPage container)
        {
            
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Speed Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.pnlSpeed = new PanelNoScrollOnFocus();
            this.pnlFakeSpeed = new Panel();
            this.dgwSpeed = new DataGridView();
            this.Column1 = new DataGridViewButtonProductsColumn();
            this.Column2 = new DataGridViewButtonProductsColumn();
            this.Column3 = new DataGridViewButtonProductsColumn();
            this.Column4 = new DataGridViewButtonProductsColumn();
            this.Column5 = new DataGridViewButtonProductsColumn();
            this.Column6 = new DataGridViewButtonProductsColumn();
            this.Column7 = new DataGridViewButtonProductsColumn();
            this.Column8 = new DataGridViewButtonProductsColumn();
            this.Column9 = new DataGridViewButtonProductsColumn();
            this.Column10 = new DataGridViewButtonProductsColumn();
            this.Column11 = new DataGridViewButtonProductsColumn();
            this.Column12 = new DataGridViewButtonProductsColumn();
            this.Column13 = new DataGridViewButtonProductsColumn();
            this.Column14 = new DataGridViewButtonProductsColumn();
            this.Column15 = new DataGridViewButtonProductsColumn();
            this.btnPg1 = new Button();
            this.btnPg2 = new Button();
            this.btnPg3 = new Button();
            this.btnPg4 = new Button();
            this.btnPg5 = new Button();
            this.GroupPage = new Panel();
            this.ContextMenuStrip = new ContextMenuStrip();
            this.SpeedModMenuItem = new ToolStripMenuItem();
            this.SpeedRemMenuItem = new ToolStripMenuItem();
            this.PageRemMenuItem = new ToolStripMenuItem();
            this.btnSwitchMode = new Button();
            this.btnSpeedMod = new Button();
            this.btnSpeedRem = new Button();
            this.btnPageRem = new Button();
            this.GroupSpeed = new Panel();
            ((ISupportInitialize)this.dgwSpeed).BeginInit();

            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Display Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.lblDisplay = new Label();
            this.lblTotale = new Label();
            this.lblTot = new Label();
            this.dgdDisplay = new DataGridView();
            this.pnlFake = new Panel();
            this.pnlDisplay = new PanelNoScrollOnFocus();
            this.ColQty = new DataGridViewTextBoxColumn();
            this.ColDesc = new DataGridViewTextBoxColumn();
            this.ColPrice = new DataGridViewTextBoxColumn();
            this.ColAmount = new DataGridViewTextBoxColumn();
            ((ISupportInitialize)this.dgdDisplay).BeginInit();

            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Keyboard Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.btn1 = new Button();
            this.btn2 = new Button();
            this.btn3 = new Button();
            this.btn4 = new Button();
            this.btn5 = new Button();
            this.btn6 = new Button();
            this.btn7 = new Button();
            this.btn8 = new Button();
            this.btn9 = new Button();
            this.btn0 = new Button();
            this.btn00 = new Button();
            this.btnPerc = new Button();
            this.btnVoid = new Button();
            this.btnPLU = new Button();
            this.btnPre = new Button();
            this.btnDot = new Button();
            this.btnPiu = new Button();
            this.btnMeno = new Button();
            this.btnMoltiplica = new Button();
            this.btnCancel = new Button();
            this.btnSub = new Button();
            this.btnPay1 = new PaymentButton();
            this.btnPay2 = new PaymentButton();
            this.btnPay3 = new PaymentButton();
            this.btnPay4 = new PaymentButton();
            this.GroupKeypad = new Panel();

            //this.SuspendLayout()
            //
            //btn1
            //
            this.btn1.BackColor = SystemColors.GradientActiveCaption;
            this.btn1.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn1.Location = new Point(83, 219);
            this.btn1.Name = "btn1";
            this.btn1.Size = new Size(77, 66);
            this.btn1.TabStop = false;
            this.btn1.Text = "1";
            this.btn1.TextAlign = ContentAlignment.MiddleCenter;
            this.btn1.Click += new EventHandler(Keypad_Click);
            this.btn1.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn2
            //
            this.btn2.BackColor = SystemColors.GradientActiveCaption;
            this.btn2.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn2.Location = new Point(166, 219);
            this.btn2.Name = "btn2";
            this.btn2.Size = new Size(77, 66);
            this.btn2.TabStop = false;
            this.btn2.Text = "2";
            this.btn2.TextAlign = ContentAlignment.MiddleCenter;
            this.btn2.Click += new EventHandler(Keypad_Click);
            this.btn2.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn3
            //
            this.btn3.BackColor = SystemColors.GradientActiveCaption;
            this.btn3.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn3.Location = new Point(249, 219);
            this.btn3.Name = "btn3";
            this.btn3.Size = new Size(77, 66);
            this.btn3.TabStop = false;
            this.btn3.Text = "3";
            this.btn3.TextAlign = ContentAlignment.MiddleCenter;
            this.btn3.Click += new EventHandler(Keypad_Click);
            this.btn3.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn4
            //
            this.btn4.BackColor = SystemColors.GradientActiveCaption;
            this.btn4.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn4.Location = new Point(83, 146);
            this.btn4.Name = "btn4";
            this.btn4.Size = new Size(77, 66);
            this.btn4.TabStop = false;
            this.btn4.Text = "4";
            this.btn4.TextAlign = ContentAlignment.MiddleCenter;
            this.btn4.Click += new EventHandler(Keypad_Click);
            this.btn4.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn5
            //
            this.btn5.BackColor = SystemColors.GradientActiveCaption;
            this.btn5.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn5.Location = new Point(166, 146);
            this.btn5.Name = "btn5";
            this.btn5.Size = new Size(77, 66);
            this.btn5.TabStop = false;
            this.btn5.Text = "5";
            this.btn5.TextAlign = ContentAlignment.MiddleCenter;
            this.btn5.Click += new EventHandler(Keypad_Click);
            this.btn5.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn6
            //
            this.btn6.BackColor = SystemColors.GradientActiveCaption;
            this.btn6.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn6.Location = new Point(249, 146);
            this.btn6.Name = "btn6";
            this.btn6.Size = new Size(77, 66);
            this.btn6.TabStop = false;
            this.btn6.Text = "6";
            this.btn6.TextAlign = ContentAlignment.MiddleCenter;
            this.btn6.Click += new EventHandler(Keypad_Click);
            this.btn6.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn7
            //
            this.btn7.BackColor = SystemColors.GradientActiveCaption;
            this.btn7.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn7.Location = new Point(83, 73);
            this.btn7.Name = "btn7";
            this.btn7.Size = new Size(77, 66);
            this.btn7.TabStop = false;
            this.btn7.Text = "7";
            this.btn7.TextAlign = ContentAlignment.MiddleCenter;
            this.btn7.Click += new EventHandler(Keypad_Click);
            this.btn7.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn8
            //
            this.btn8.BackColor = SystemColors.GradientActiveCaption;
            this.btn8.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn8.Location = new Point(166, 73);
            this.btn8.Name = "btn8";
            this.btn8.Size = new Size(77, 66);
            this.btn8.TabStop = false;
            this.btn8.Text = "8";
            this.btn8.TextAlign = ContentAlignment.MiddleCenter;
            this.btn8.Click += new EventHandler(Keypad_Click);
            this.btn8.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn9
            //
            this.btn9.BackColor = SystemColors.GradientActiveCaption;
            this.btn9.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn9.Location = new Point(249, 73);
            this.btn9.Name = "btn9";
            this.btn9.Size = new Size(77, 66);
            this.btn9.TabStop = false;
            this.btn9.Text = "9";
            this.btn9.TextAlign = ContentAlignment.MiddleCenter;
            this.btn9.Click += new EventHandler(Keypad_Click);
            this.btn9.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn0
            //
            this.btn0.BackColor = SystemColors.GradientActiveCaption;
            this.btn0.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn0.Location = new Point(83, 292);
            this.btn0.Name = "btn0";
            this.btn0.Size = new Size(77, 66);
            this.btn0.TabStop = false;
            this.btn0.Text = "0";
            this.btn0.TextAlign = ContentAlignment.MiddleCenter;
            this.btn0.Click += new EventHandler(Keypad_Click);
            this.btn0.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btn00
            //
            this.btn00.BackColor = SystemColors.GradientActiveCaption;
            this.btn00.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btn00.Location = new Point(166, 292);
            this.btn00.Name = "btn00";
            this.btn00.Size = new Size(77, 66);
            this.btn00.TabStop = false;
            this.btn00.Text = "00";
            this.btn00.TextAlign = ContentAlignment.MiddleCenter;
            this.btn00.Click += new EventHandler(Keypad_Click);
            this.btn00.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btnDot
            //
            this.btnDot.BackColor = SystemColors.GradientActiveCaption;
            this.btnDot.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnDot.Location = new Point(249, 292);
            this.btnDot.Name = "btnDot";
            this.btnDot.Size = new Size(77, 66);
            this.btnDot.TabStop = false;
            this.btnDot.Text = ".";
            this.btnDot.TextAlign = ContentAlignment.MiddleCenter;
            //this.btnDot.Click += new EventHandler(Keypad_Click);
            //this.btnDot.DoubleClick += new EventHandler(Keypad_DoubleClick);
            //
            //btnPay1
            //
            this.btnPay1.BackColor = Color.FromArgb(((int)(((byte)(192)))), ((int)(((byte)(255)))), ((int)(((byte)(192)))));
            this.btnPay1.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPay1.Image = (Image)resources.GetObject("money");
            this.btnPay1.ImageAlign = ContentAlignment.TopCenter;
            this.btnPay1.Location = new Point(332, Properties.Settings.Default.btnPay1Top);
            this.btnPay1.Name = "btnPay1";
            this.btnPay1.Size = new Size(108, 66);
            this.btnPay1.Text = resources.GetString("btnPay1.Text");
            this.btnPay1.Paid = Convert.ToBoolean(resources.GetString("btnPay1.Paid"));
            this.btnPay1.Code = resources.GetString("btnPay1.Code");
            this.btnPay1.TextAlign = ContentAlignment.BottomCenter;
            this.btnPay1.Click += new EventHandler(btnPay1_Click);
            //
            //btnPay2
            //
            this.btnPay2.BackColor = Color.FromArgb(((int)(((byte)(192)))), ((int)(((byte)(255)))), ((int)(((byte)(192)))));
            this.btnPay2.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPay2.ForeColor = SystemColors.ControlText;
            this.btnPay2.Image = (Image)resources.GetObject("credit");
            this.btnPay2.ImageAlign = ContentAlignment.TopCenter;
            this.btnPay2.Location = new Point(332, Properties.Settings.Default.btnPay2Top);
            this.btnPay2.Name = "btnPay2";
            this.btnPay2.Size = new Size(108, 66);
            this.btnPay2.Text = resources.GetString("btnPay2.Text");
            this.btnPay2.Paid = Convert.ToBoolean(resources.GetString("btnPay2.Paid"));
            this.btnPay2.Code = resources.GetString("btnPay2.Code");
            this.btnPay2.TextAlign = ContentAlignment.BottomCenter;
            this.btnPay2.Click += new EventHandler(btnPay1_Click);
            //
            //btnPay3
            //
            this.btnPay3.BackColor = Color.FromArgb(((int)(((byte)(192)))), ((int)(((byte)(255)))), ((int)(((byte)(192)))));
            this.btnPay3.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPay3.ForeColor = SystemColors.ControlText;
            this.btnPay3.Image = (Image)resources.GetObject("voucher");
            this.btnPay3.ImageAlign = ContentAlignment.TopCenter;
            this.btnPay3.Location = new Point(332, Properties.Settings.Default.btnPay3Top);
            this.btnPay3.Name = "btnPay3";
            this.btnPay3.Size = new Size(108, 66);
            this.btnPay3.Text = resources.GetString("btnPay3.Text");
            this.btnPay3.Paid = Convert.ToBoolean(resources.GetString("btnPay3.Paid"));
            this.btnPay3.Code = resources.GetString("btnPay3.Code");
            this.btnPay3.TextAlign = ContentAlignment.BottomCenter;
            this.btnPay3.Click += new EventHandler(btnPay1_Click);
            //
            //btnPay4
            //
            this.btnPay4.BackColor = Color.FromArgb(((int)(((byte)(192)))), ((int)(((byte)(255)))), ((int)(((byte)(192)))));
            this.btnPay4.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPay4.ForeColor = SystemColors.ControlText;
            this.btnPay4.Image = (Image)resources.GetObject("piggy");
            this.btnPay4.ImageAlign = ContentAlignment.TopCenter;
            this.btnPay4.Location = new Point(332, Properties.Settings.Default.btnPay4Top);
            this.btnPay4.Name = "btnPay4";
            this.btnPay4.Size = new Size(108, 66);
            this.btnPay4.Text = resources.GetString("btnPay4.Text");
            this.btnPay4.Paid = Convert.ToBoolean(resources.GetString("btnPay4.Paid"));
            this.btnPay4.Code = resources.GetString("btnPay4.Code");
            this.btnPay4.TextAlign = ContentAlignment.BottomCenter;
            this.btnPay4.Click += new EventHandler(btnPay1_Click);
            //
            //btnSub
            //
            this.btnSub.BackColor = Color.Yellow;
            this.btnSub.Font = new Font("Microsoft Sans Serif", 25.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnSub.Location = new Point(332, Properties.Settings.Default.btnSubTop);
            this.btnSub.Name = "btnSub";
            this.btnSub.Size = new Size(108, 66);
            this.btnSub.Text = "SUB";
            this.btnSub.TextAlign = ContentAlignment.MiddleCenter;
            this.btnSub.Click += new EventHandler(btnSub_Click);
            //
            //btnCancel
            //
            this.btnCancel.BackColor = Color.IndianRed;
            this.btnCancel.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnCancel.Location = new Point(83, 0);
            this.btnCancel.Name = "btnCancel";
            this.btnCancel.Size = new Size(77, 66);
            this.btnCancel.TabStop = false;
            this.btnCancel.Text = "C";
            this.btnCancel.TextAlign = ContentAlignment.MiddleCenter;
            btnCancel.Click += new EventHandler(btnCancel_Click);
            //
            //btnVoid
            //
            this.btnVoid.BackColor = Color.IndianRed;
            this.btnVoid.Font = new Font("Microsoft Sans Serif", 23.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnVoid.Location = new Point(166, 0);
            this.btnVoid.Name = "btnVoid";
            this.btnVoid.Size = new Size(77, 66);
            this.btnVoid.TabStop = false;
            this.btnVoid.Text = resources.GetString("btnVoid.Text");
            this.btnVoid.TextAlign = ContentAlignment.MiddleCenter;
            this.btnVoid.Click += new EventHandler(btnVoid_Click);
            //
            //btnPre
            //
            this.btnPre.BackColor = Color.Green;
            this.btnPre.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPre.Location = new Point(249, 0);
            this.btnPre.Name = "btnPre";
            this.btnPre.Size = new Size(77, 66);
            this.btnPre.TabStop = false;
            this.btnPre.Text = resources.GetString("btnPre.Text");
            this.btnPre.TextAlign = ContentAlignment.MiddleCenter;
            this.btnPre.Click += new EventHandler(btnPre_Click);
            //
            //btnPLU
            //
            this.btnPLU.BackColor = Color.White;
            this.btnPLU.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPLU.Location = new Point(0, 0);
            this.btnPLU.Name = "btnPLU";
            this.btnPLU.Image = (Image)resources.GetObject("code");
            this.btnPLU.ImageAlign = ContentAlignment.TopCenter;
            this.btnPLU.Size = new Size(77, 66);
            this.btnPLU.TabStop = false;
            this.btnPLU.Text = "PLU";
            this.btnPLU.TextAlign = ContentAlignment.BottomCenter;
            this.btnPLU.Click += new EventHandler(btnPLU_Click);
            //
            //btnPiu
            //
            this.btnPiu.BackColor = Color.Yellow;
            this.btnPiu.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPiu.Location = new Point(0, 292);
            this.btnPiu.Name = "btnPiu";
            this.btnPiu.Size = new Size(77, 66);
            this.btnPiu.TabStop = false;
            this.btnPiu.Text = "+";
            this.btnPiu.TextAlign = ContentAlignment.MiddleCenter;
            this.btnPiu.Click += new EventHandler(btnPiu_Click);
            //
            //btnMeno
            //
            this.btnMeno.BackColor = Color.Yellow;
            this.btnMeno.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnMeno.Location = new Point(0, 219);
            this.btnMeno.Name = "btnMeno";
            this.btnMeno.Size = new Size(77, 66);
            this.btnMeno.TabStop = false;
            this.btnMeno.Text = "-";
            this.btnMeno.TextAlign = ContentAlignment.MiddleCenter;
            this.btnMeno.Click += new EventHandler(btnMeno_Click);
            //
            //btnMoltiplica
            //
            this.btnMoltiplica.BackColor = Color.Yellow;
            this.btnMoltiplica.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnMoltiplica.Location = new Point(0, 73);
            this.btnMoltiplica.Name = "btnMoltiplica";
            this.btnMoltiplica.Size = new Size(77, 66);
            this.btnMoltiplica.TabStop = false;
            this.btnMoltiplica.Text = "X";
            this.btnMoltiplica.TextAlign = ContentAlignment.MiddleCenter;
            this.btnMoltiplica.Click += new EventHandler(btnMoltiplica_Click);
            //
            //btnPerc
            //
            this.btnPerc.BackColor = Color.Yellow;
            this.btnPerc.Font = new Font("Microsoft Sans Serif", 30.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPerc.Location = new Point(0, 146);
            this.btnPerc.Name = "btnPerc";
            this.btnPerc.Size = new Size(77, 66);
            this.btnPerc.TabStop = false;
            this.btnPerc.Text = "%";
            this.btnPerc.TextAlign = ContentAlignment.MiddleCenter;
            this.btnPerc.Click += new EventHandler(btnPerc_Click);
            //
            //GroupKeypad
            //
            this.GroupKeypad.BorderStyle = BorderStyle.None;
            this.GroupKeypad.Controls.Add(this.btn1);
            this.GroupKeypad.Controls.Add(this.btn2);
            this.GroupKeypad.Controls.Add(this.btn3);
            this.GroupKeypad.Controls.Add(this.btn4);
            this.GroupKeypad.Controls.Add(this.btn5);
            this.GroupKeypad.Controls.Add(this.btn6);
            this.GroupKeypad.Controls.Add(this.btn7);
            this.GroupKeypad.Controls.Add(this.btn8);
            this.GroupKeypad.Controls.Add(this.btn9);
            this.GroupKeypad.Controls.Add(this.btn0);
            this.GroupKeypad.Controls.Add(this.btn00);
            this.GroupKeypad.Controls.Add(this.btnVoid);
            this.GroupKeypad.Controls.Add(this.btnPLU);
            this.GroupKeypad.Controls.Add(this.btnPre);
            this.GroupKeypad.Controls.Add(this.btnDot);
            this.GroupKeypad.Controls.Add(this.btnPiu);
            this.GroupKeypad.Controls.Add(this.btnPerc);
            this.GroupKeypad.Controls.Add(this.btnMeno);
            this.GroupKeypad.Controls.Add(this.btnMoltiplica);
            this.GroupKeypad.Controls.Add(this.btnCancel);
            this.GroupKeypad.Controls.Add(this.btnSub);
            this.GroupKeypad.Controls.Add(this.btnPay1);
            this.GroupKeypad.Controls.Add(this.btnPay2);
            this.GroupKeypad.Controls.Add(this.btnPay3);
            this.GroupKeypad.Controls.Add(this.btnPay4);
            this.GroupKeypad.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
            this.GroupKeypad.Top = 323;
            this.GroupKeypad.Name = "GroupKeypad";
            this.GroupKeypad.Size = new Size(440, 358);
            this.GroupKeypad.TabStop = false;
            this.GroupKeypad.Visible = true;
            //
            //btnPg1
            //
            this.btnPg1.BackColor = Color.LightYellow;
            this.btnPg1.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPg1.Location = new Point(1, 0);
            this.btnPg1.Name = "btnPg1";
            this.btnPg1.Text = "PAGE1";
            this.btnPg1.Size = new Size(100, 40);
            this.btnPg1.TabStop = false;
            this.btnPg1.UseVisualStyleBackColor = false;
            this.btnPg1.Click += new EventHandler(btnPg1_Click);
            //
            //btnPg2
            //
            this.btnPg2.BackColor = Color.LightYellow;
            this.btnPg2.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPg2.Location = new Point(101, 0);
            this.btnPg2.Name = "btnPg2";
            this.btnPg2.Text = "PAGE2";
            this.btnPg2.Size = new Size(100, 40);
            this.btnPg2.TabStop = false;
            this.btnPg2.UseVisualStyleBackColor = false;
            this.btnPg2.Click += new EventHandler(btnPg2_Click);
            //
            //btnPg3
            //
            this.btnPg3.BackColor = Color.LightYellow;
            this.btnPg3.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPg3.Location = new Point(201, 0);
            this.btnPg3.Name = "btnPg3";
            this.btnPg3.Text = "PAGE3";
            this.btnPg3.Size = new Size(100, 40);
            this.btnPg3.TabStop = false;
            this.btnPg3.UseVisualStyleBackColor = false;
            this.btnPg3.Click += new EventHandler(btnPg3_Click);
            //
            //btnPg4
            //
            this.btnPg4.BackColor = Color.LightYellow;
            this.btnPg4.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPg4.Location = new Point(301, 0);
            this.btnPg4.Name = "btnPg4";
            this.btnPg4.Text = "PAGE4";
            this.btnPg4.Size = new Size(100, 40);
            this.btnPg4.TabStop = false;
            this.btnPg4.UseVisualStyleBackColor = false;
            this.btnPg4.Click += new EventHandler(btnPg4_Click);
            //
            //btnPg5
            //
            this.btnPg5.BackColor = Color.LightYellow;
            this.btnPg5.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPg5.Location = new Point(401, 0);
            this.btnPg5.Name = "btnPg5";
            this.btnPg5.Text = "PAGE5";
            this.btnPg5.Size = new Size(100, 40);
            this.btnPg5.TabStop = false;
            this.btnPg5.UseVisualStyleBackColor = false;
            this.btnPg5.Click += new EventHandler(btnPg5_Click);
            //
            //GroupPage
            //
            this.GroupPage.BorderStyle = BorderStyle.FixedSingle;
            this.GroupPage.Controls.Add(this.btnPg1);
            this.GroupPage.Controls.Add(this.btnPg2);
            this.GroupPage.Controls.Add(this.btnPg3);
            this.GroupPage.Controls.Add(this.btnPg4);
            this.GroupPage.Controls.Add(this.btnPg5);
            this.GroupPage.Left = Properties.Settings.Default.lefthanded ? 495 : 21;
            this.GroupPage.Top = 13;
            this.GroupPage.Name = "GroupPage";
            this.GroupPage.Size = new Size(503, 42);
            this.GroupPage.TabStop = false;
            this.GroupPage.Visible = true;
            //
            //ContextMenuStrip
            //
            this.ContextMenuStrip.AutoSize = false;
            this.ContextMenuStrip.BackColor = Color.White;
            this.ContextMenuStrip.BackgroundImageLayout = ImageLayout.None;
            this.ContextMenuStrip.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.ContextMenuStrip.Items.AddRange(new ToolStripItem[] { this.SpeedModMenuItem, this.SpeedRemMenuItem, this.PageRemMenuItem });
            this.ContextMenuStrip.Name = "ContextMenuStrip";
            this.ContextMenuStrip.ShowCheckMargin = false;
            this.ContextMenuStrip.ShowImageMargin = false;
            this.ContextMenuStrip.ShowItemToolTips = false;
            this.ContextMenuStrip.Size = new Size(155, 166);
            //
            //SpeedModMenuItem
            //
            this.SpeedModMenuItem.AutoSize = false;
            this.SpeedModMenuItem.Name = "SpeedModMenuItem";
            this.SpeedModMenuItem.Size = new Size(154, 54);
            this.SpeedModMenuItem.Text = resources.GetString("btnSpeedMod.Text");
            //
            //SpeedRemMenuItem
            //
            this.SpeedRemMenuItem.AutoSize = false;
            this.SpeedRemMenuItem.Name = "SpeedRemMenuItem";
            this.SpeedRemMenuItem.Size = new Size(154, 54);
            this.SpeedRemMenuItem.Text = resources.GetString("btnSpeedRem.Text");
            //
            //PageRemMenuItem
            //
            this.PageRemMenuItem.AutoSize = false;
            this.PageRemMenuItem.Name = "PageRemMenuItem";
            this.PageRemMenuItem.Size = new Size(154, 54);
            this.PageRemMenuItem.Text = resources.GetString("btnPageRem.Text");
            //
            //btnSwitchMode
            //
            this.btnSwitchMode.BackColor = Color.IndianRed;
            this.btnSwitchMode.Left = Properties.Settings.Default.lefthanded ? 1002 : 0;
            this.btnSwitchMode.Top = 13;
            this.btnSwitchMode.Name = "btnSwitchMode";
            this.btnSwitchMode.Text = "";
            this.btnSwitchMode.Size = new Size(17, 42);
            this.btnSwitchMode.TabStop = false;
            this.btnSwitchMode.UseVisualStyleBackColor = false;
            this.btnSwitchMode.Click += new EventHandler(btnSwitchMode_Click);
            //
            //btnSpeedMod
            //
            this.btnSpeedMod.BackColor = Color.White;
            this.btnSpeedMod.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnSpeedMod.Location = new Point(0, 0);
            this.btnSpeedMod.Name = "btnSpeedMod";
            this.btnSpeedMod.Size = new Size(116, 54);
            this.btnSpeedMod.TabStop = false;
            this.btnSpeedMod.Text = resources.GetString("btnSpeedMod.Text");
            this.btnSpeedMod.UseVisualStyleBackColor = false;
            this.btnSpeedMod.Click += new EventHandler(btnSpeedMod_Click);
            //
            //btnSpeedRem
            //
            this.btnSpeedRem.BackColor = Color.White;
            this.btnSpeedRem.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnSpeedRem.Location = new Point(122, 0);
            this.btnSpeedRem.Name = "btnSpeedRem";
            this.btnSpeedRem.Size = new Size(116, 54);
            this.btnSpeedRem.TabStop = false;
            this.btnSpeedRem.Text = resources.GetString("btnSpeedRem.Text");
            this.btnSpeedRem.UseVisualStyleBackColor = false;
            this.btnSpeedRem.Click += new EventHandler(btnSpeedRem_Click);
            //
            //btnPageRem
            //
            this.btnPageRem.BackColor = Color.IndianRed;
            this.btnPageRem.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnPageRem.Location = new Point(244, 0);
            this.btnPageRem.Name = "btnPageRem";
            this.btnPageRem.Size = new Size(116, 54);
            this.btnPageRem.TabStop = false;
            this.btnPageRem.Text = resources.GetString("btnPageRem.Text");
            this.btnPageRem.UseVisualStyleBackColor = false;
            this.btnPageRem.Click += new EventHandler(btnPageRem_Click);
            //
            //GroupSpeed
            //
            this.GroupSpeed.BorderStyle = BorderStyle.None;
            this.GroupSpeed.Controls.Add(this.btnSpeedMod);
            this.GroupSpeed.Controls.Add(this.btnSpeedRem);
            this.GroupSpeed.Controls.Add(this.btnPageRem);
            this.GroupSpeed.Left = Properties.Settings.Default.lefthanded ? 495 : 21;
            this.GroupSpeed.Top = 6;
            this.GroupSpeed.Name = "GroupSpeed";
            this.GroupSpeed.Size = new Size(360, 54);
            this.GroupSpeed.TabStop = false;
            this.GroupSpeed.Visible = false;
            //
            //lblDisplay
            //
            this.lblDisplay.BackColor = Color.Black;
            this.lblDisplay.BorderStyle = BorderStyle.FixedSingle;
            this.lblDisplay.Font = new Font("Microsoft Sans Serif", 28.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.lblDisplay.ForeColor = Color.White;
            this.lblDisplay.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
            this.lblDisplay.Top = 255;
            this.lblDisplay.Name = "lblDisplay";
            this.lblDisplay.Size = new Size(440, 65);
            this.lblDisplay.TabStop = false;
            this.lblDisplay.Text = "" + (char)13 + (char)10 + "0";
            this.lblDisplay.TextAlign = ContentAlignment.MiddleRight;
            //
            //lblTotale
            //
            this.lblTotale.BackColor = Color.White;
            this.lblTotale.BorderStyle = BorderStyle.FixedSingle;
            this.lblTotale.Font = new Font("Microsoft Sans Serif", 22.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.lblTotale.Location = new Point(825, 224);
            this.lblTotale.Left = Properties.Settings.Default.lefthanded ? 289 : 825;
            this.lblTotale.Top = 224;
            this.lblTotale.Name = "lblTotale";
            this.lblTotale.Size = new Size(172, 32);
            this.lblTotale.TabStop = false;
            this.lblTotale.Text = String.Format("{0:C2}", 0);
            this.lblTotale.TextAlign = ContentAlignment.MiddleRight;
            //
            //lblTot
            //
            this.lblTot.BackColor = Color.White;
            this.lblTot.BorderStyle = BorderStyle.FixedSingle;
            this.lblTot.Font = new Font("Microsoft Sans Serif", 24.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.lblTot.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
            this.lblTot.Top = 224;
            this.lblTot.Name = "lblTot";
            this.lblTot.Size = new Size(269, 32);
            this.lblTot.TabStop = false;
            this.lblTot.Text = resources.GetString("lblTot.Text");
            this.lblTot.TextAlign = ContentAlignment.MiddleRight;
            //
            //pnlFake
            //
            this.pnlFake.AutoScroll = false;
            this.pnlFake.AutoScrollMargin = new Size(0, 0);
            this.pnlFake.BorderStyle = BorderStyle.FixedSingle;
            this.pnlFake.Controls.Add(pnlDisplay);
            this.pnlFake.Name = "pnlFake";
            this.pnlFake.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
            this.pnlFake.Top = 13;
            this.pnlFake.Size = new Size(440, 212);
            //
            //pnlDisplay
            //
            this.pnlDisplay.AutoScroll = true;
            this.pnlDisplay.AutoScrollMargin = new Size(0, 0);
            this.pnlDisplay.BorderStyle = BorderStyle.None;
            this.pnlDisplay.Controls.Add(dgdDisplay);
            this.pnlDisplay.Name = "pnlDisplay";
            this.pnlDisplay.Size = new Size(458, 212);
            this.pnlDisplay.Location = new Point(0, 0);
            //
            //dgdDisplay
            //
            this.dgdDisplay.AllowUserToAddRows = false;
            this.dgdDisplay.AllowUserToDeleteRows = false;
            this.dgdDisplay.AutoGenerateColumns = false;
            this.dgdDisplay.AutoSize = true;
            this.dgdDisplay.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;
            this.dgdDisplay.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.None;
            this.dgdDisplay.BackgroundColor = Color.White;
            this.dgdDisplay.BorderStyle = BorderStyle.None;
            this.dgdDisplay.CausesValidation = false;
            this.dgdDisplay.CellBorderStyle = DataGridViewCellBorderStyle.None;
            this.dgdDisplay.ColumnHeadersHeight = 30;
            this.dgdDisplay.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            this.dgdDisplay.ColumnHeadersVisible = false;
            this.dgdDisplay.Columns.AddRange(new DataGridViewColumn[] { this.ColQty, this.ColDesc, this.ColPrice, this.ColAmount });
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //DefaultCellStyle
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.dgdDisplay.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
            this.dgdDisplay.DefaultCellStyle.BackColor = Color.White;
            this.dgdDisplay.DefaultCellStyle.Font = new Font("Arial", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.dgdDisplay.DefaultCellStyle.ForeColor = Color.Black;
            this.dgdDisplay.DefaultCellStyle.SelectionBackColor = Color.White;
            this.dgdDisplay.DefaultCellStyle.SelectionForeColor = Color.Black;
            this.dgdDisplay.DefaultCellStyle.WrapMode = DataGridViewTriState.False;
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //EndDefaultCellStyle
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.dgdDisplay.EditMode = DataGridViewEditMode.EditOnEnter;
            this.dgdDisplay.GridColor = Color.Black;
            this.dgdDisplay.Location = new Point(0, 0);
            this.dgdDisplay.MultiSelect = false;
            this.dgdDisplay.Name = "dgdDisplay";
            this.dgdDisplay.ReadOnly = true;
            this.dgdDisplay.RowHeadersVisible = false;
            this.dgdDisplay.RowTemplate.Height = 30;
            this.dgdDisplay.RowTemplate.Resizable = DataGridViewTriState.False;
            this.dgdDisplay.ScrollBars = ScrollBars.None;
            this.dgdDisplay.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            this.dgdDisplay.MaximumSize = new Size(440, 0);
            this.dgdDisplay.MinimumSize = new Size(440, 210);
            this.dgdDisplay.TabStop = false;
            this.dgdDisplay.MouseDown += new MouseEventHandler(dgdDisplay_MouseDown);
            this.dgdDisplay.MouseMove += new MouseEventHandler(dgdDisplay_MouseMove);
            this.dgdDisplay.MouseUp += new MouseEventHandler(dgdDisplay_MouseUp);
            //
            //ColQty
            //
            this.ColQty.DataPropertyName = "Quantity";
            this.ColQty.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.ColQty.HeaderText = resources.GetString("ColQty.HeaderText");
            this.ColQty.Name = "ColQty";
            this.ColQty.ReadOnly = true;
            this.ColQty.Resizable = DataGridViewTriState.False;
            this.ColQty.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColQty.Width = 37;
            //
            //ColDesc
            //
            this.ColDesc.DataPropertyName = "Text";
            this.ColDesc.HeaderText = resources.GetString("ColDesc.HeaderText");
            this.ColDesc.Name = "ColDesc";
            this.ColDesc.ReadOnly = true;
            this.ColDesc.Resizable = DataGridViewTriState.False;
            this.ColDesc.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColDesc.Width = 230;
            //
            //ColPrice
            //
            this.ColPrice.DataPropertyName = "Value";
            this.ColPrice.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            this.ColPrice.DefaultCellStyle.Format = "C2";
            this.ColPrice.DefaultCellStyle.NullValue = null;
            this.ColPrice.HeaderText = resources.GetString("ColPrice.HeaderText");
            this.ColPrice.Name = "ColPrice";
            this.ColPrice.ReadOnly = true;
            this.ColPrice.Resizable = DataGridViewTriState.False;
            this.ColPrice.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColPrice.Width = 85;
            //
            //ColAmount
            //
            this.ColAmount.DataPropertyName = "Amount";
            this.ColAmount.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            this.ColAmount.DefaultCellStyle.Format = "C2";
            this.ColAmount.DefaultCellStyle.NullValue = null;
            this.ColAmount.HeaderText = resources.GetString("ColAmount.HeaderText");
            this.ColAmount.Name = "ColAmount";
            this.ColAmount.ReadOnly = true;
            this.ColAmount.Resizable = DataGridViewTriState.False;
            this.ColAmount.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColAmount.Width = 85;
            //
            //pnlFakeSpeed
            //
            this.pnlFakeSpeed.AutoScroll = false;
            this.pnlFakeSpeed.AutoScrollMargin = new Size(0, 0);
            this.pnlFakeSpeed.BorderStyle = BorderStyle.FixedSingle;
            this.pnlFakeSpeed.Controls.Add(pnlSpeed);
            this.pnlFakeSpeed.Name = "pnlFakeSpeed";
            this.pnlFakeSpeed.Size = new Size(502, 614);
            this.pnlFakeSpeed.Left = Properties.Settings.Default.lefthanded ? 495 : 21;
            this.pnlFakeSpeed.Top = 67;
            //
            //pnlSpeed
            //
            this.pnlSpeed.AutoScroll = true;
            this.pnlSpeed.AutoScrollMargin = new Size(0, 0);
            this.pnlSpeed.BorderStyle = BorderStyle.None;
            this.pnlSpeed.Controls.Add(dgwSpeed);
            this.pnlSpeed.Name = "pnlSpeed";
            this.pnlSpeed.Size = new Size(501, 633); //2 pixels for visual styles
            this.pnlSpeed.Location = new Point(0, 0);
            //
            //dgwSpeed
            //
            this.dgwSpeed.AllowUserToAddRows = false;
            this.dgwSpeed.AllowUserToDeleteRows = false;
            this.dgwSpeed.AllowUserToResizeColumns = false;
            this.dgwSpeed.AllowUserToResizeRows = false;
            this.dgwSpeed.BorderStyle = BorderStyle.None;
            this.dgwSpeed.StandardTab = true;
            this.dgwSpeed.AutoGenerateColumns = false;
            this.dgwSpeed.ContextMenuStrip = ContextMenuStrip;
            this.dgwSpeed.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.AutoSize;
            this.dgwSpeed.ColumnHeadersVisible = false;
            this.dgwSpeed.Columns.AddRange(new DataGridViewColumn[] {this.Column1, this.Column2, this.Column3, this.Column4, this.Column5,
                                                                     this.Column6, this.Column7, this.Column8, this.Column9, this.Column10,
                                                                     this.Column11, this.Column12, this.Column13, this.Column14, this.Column15});
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //DefaultCellStyle
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.dgwSpeed.DefaultCellStyle.Alignment = DataGridViewContentAlignment.BottomCenter;
            this.dgwSpeed.DefaultCellStyle.BackColor = SystemColors.Window;
            this.dgwSpeed.DefaultCellStyle.Font = new Font("Microsoft Sans Serif", 11.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.dgwSpeed.DefaultCellStyle.ForeColor = SystemColors.ControlText;
            this.dgwSpeed.DefaultCellStyle.SelectionBackColor = Color.White;
            this.dgwSpeed.DefaultCellStyle.SelectionForeColor = Color.Black;
            this.dgwSpeed.DefaultCellStyle.WrapMode = DataGridViewTriState.True;
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //EndDefaultCellStyle
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            this.dgwSpeed.Location = new Point(0, 0);
            this.dgwSpeed.Name = "dgwSpeed";
            this.dgwSpeed.ReadOnly = true;
            this.dgwSpeed.RowHeadersVisible = false;
            this.dgwSpeed.RowTemplate.Height = 68;
            this.dgwSpeed.ScrollBars = ScrollBars.None;
            this.dgwSpeed.SelectionMode = DataGridViewSelectionMode.CellSelect;
            this.dgwSpeed.MultiSelect = false;
            this.dgwSpeed.ShowCellErrors = false;
            this.dgwSpeed.ShowCellToolTips = false;
            this.dgwSpeed.ShowEditingIcon = false;
            this.dgwSpeed.ShowRowErrors = false;
            this.dgwSpeed.Size = new Size(1503, 615);
            this.dgwSpeed.TabStop = false;
            this.dgwSpeed.CellClick += new DataGridViewCellEventHandler(dgwSpeed_CellClick);
            this.dgwSpeed.CellDoubleClick += new DataGridViewCellEventHandler(dgwSpeed_CellDoubleClick);
            this.dgwSpeed.MouseDown += new MouseEventHandler(dgwSpeed_MouseDown);
            this.dgwSpeed.MouseMove += new MouseEventHandler(dgwSpeed_MouseMove);
            this.dgwSpeed.MouseUp += new MouseEventHandler(dgwSpeed_MouseUp);
            //
            //Column1
            //
            this.Column1.DataPropertyName = "uno";
            this.Column1.HeaderText = "Column1";
            this.Column1.Name = "Column1";
            this.Column1.ReadOnly = true;
            this.Column1.Resizable = DataGridViewTriState.False;
            //
            //Column2
            //
            this.Column2.DataPropertyName = "due";
            this.Column2.HeaderText = "Column2";
            this.Column2.Name = "Column2";
            this.Column2.ReadOnly = true;
            this.Column2.Resizable = DataGridViewTriState.False;
            //
            //Column3
            //
            this.Column3.DataPropertyName = "tre";
            this.Column3.HeaderText = "Column3";
            this.Column3.Name = "Column3";
            this.Column3.ReadOnly = true;
            this.Column3.Resizable = DataGridViewTriState.False;
            //
            //Column4
            //
            this.Column4.DataPropertyName = "quattro";
            this.Column4.HeaderText = "Column4";
            this.Column4.Name = "Column4";
            this.Column4.ReadOnly = true;
            this.Column4.Resizable = DataGridViewTriState.False;
            //
            //Column5
            //
            this.Column5.DataPropertyName = "cinque";
            this.Column5.HeaderText = "Column5";
            this.Column5.Name = "Column5";
            this.Column5.ReadOnly = true;
            this.Column5.Resizable = DataGridViewTriState.False;
            //
            //Column6
            //
            this.Column6.DataPropertyName = "sei";
            this.Column6.HeaderText = "Column6";
            this.Column6.Name = "Column6";
            this.Column6.ReadOnly = true;
            this.Column6.Resizable = DataGridViewTriState.False;
            //
            //Column7
            //
            this.Column7.DataPropertyName = "sette";
            this.Column7.HeaderText = "Column7";
            this.Column7.Name = "Column7";
            this.Column7.ReadOnly = true;
            this.Column7.Resizable = DataGridViewTriState.False;
            //
            //Column8
            //
            this.Column8.DataPropertyName = "otto";
            this.Column8.HeaderText = "Column8";
            this.Column8.Name = "Column8";
            this.Column8.ReadOnly = true;
            this.Column8.Resizable = DataGridViewTriState.False;
            //
            //Column9
            //
            this.Column9.DataPropertyName = "nove";
            this.Column9.HeaderText = "Column9";
            this.Column9.Name = "Column9";
            this.Column9.ReadOnly = true;
            this.Column9.Resizable = DataGridViewTriState.False;
            //
            //Column10
            //
            this.Column10.DataPropertyName = "dieci";
            this.Column10.HeaderText = "Column10";
            this.Column10.Name = "Column10";
            this.Column10.ReadOnly = true;
            this.Column10.Resizable = DataGridViewTriState.False;
            //
            //Column11
            //
            this.Column11.DataPropertyName = "undici";
            this.Column11.HeaderText = "Column11";
            this.Column11.Name = "Column11";
            this.Column11.ReadOnly = true;
            this.Column11.Resizable = DataGridViewTriState.False;
            //
            //Column12
            //
            this.Column12.DataPropertyName = "dodici";
            this.Column12.HeaderText = "Column12";
            this.Column12.Name = "Column12";
            this.Column12.ReadOnly = true;
            this.Column12.Resizable = DataGridViewTriState.False;
            //
            //Column13
            //
            this.Column13.DataPropertyName = "tredici";
            this.Column13.HeaderText = "Column13";
            this.Column13.Name = "Column13";
            this.Column13.ReadOnly = true;
            this.Column13.Resizable = DataGridViewTriState.False;
            //
            //Column14
            //
            this.Column14.DataPropertyName = "quattordici";
            this.Column14.HeaderText = "Column14";
            this.Column14.Name = "Column14";
            this.Column14.ReadOnly = true;
            this.Column14.Resizable = DataGridViewTriState.False;
            //
            //Column15
            //
            this.Column15.DataPropertyName = "quindici";
            this.Column15.HeaderText = "Column15";
            this.Column15.Name = "Column15";
            this.Column15.ReadOnly = true;
            this.Column15.Resizable = DataGridViewTriState.False;

            //
            //TabCassa
            //
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Speed Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            container.Controls.Add(this.pnlFakeSpeed);
            container.Controls.Add(this.btnSwitchMode);
            container.Controls.Add(this.GroupPage);
            container.Controls.Add(this.GroupSpeed);
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Display Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            container.Controls.Add(this.lblTotale);
            container.Controls.Add(this.lblTot);
            container.Controls.Add(this.lblDisplay);
            container.Controls.Add(this.pnlFake);
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //Keyboard Group
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            container.Controls.Add(this.GroupKeypad);

            container.Text = resources.GetString("container.Text");

        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Speed Group
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private PanelNoScrollOnFocus pnlSpeed;
        private Panel pnlFakeSpeed;
        private DataGridView dgwSpeed;
        private DataGridViewButtonProductsColumn Column1;
        private DataGridViewButtonProductsColumn Column2;
        private DataGridViewButtonProductsColumn Column3;
        private DataGridViewButtonProductsColumn Column4;
        private DataGridViewButtonProductsColumn Column5;
        private DataGridViewButtonProductsColumn Column6;
        private DataGridViewButtonProductsColumn Column7;
        private DataGridViewButtonProductsColumn Column8;
        private DataGridViewButtonProductsColumn Column9;
        private DataGridViewButtonProductsColumn Column10;
        private DataGridViewButtonProductsColumn Column11;
        private DataGridViewButtonProductsColumn Column12;
        private DataGridViewButtonProductsColumn Column13;
        private DataGridViewButtonProductsColumn Column14;
        private DataGridViewButtonProductsColumn Column15;
        private Button btnPg1;
        private Button btnPg2;
        private Button btnPg3;
        private Button btnPg4;
        private Button btnPg5;
        private Panel GroupPage;
        private ContextMenuStrip ContextMenuStrip;
        private ToolStripMenuItem SpeedModMenuItem;
        private ToolStripMenuItem SpeedRemMenuItem;
        private ToolStripMenuItem PageRemMenuItem;
        private Button btnSwitchMode;
        private Button btnSpeedMod;
        private Button btnSpeedRem;
        private Button btnPageRem;
        private Panel GroupSpeed;
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Display Group
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private Label lblTotale;
        private Label lblTot;
        private Label lblDisplay;
        private PanelNoScrollOnFocus pnlDisplay;
        private Panel pnlFake;
        private DataGridView dgdDisplay;
        private DataGridViewTextBoxColumn ColQty;
        private DataGridViewTextBoxColumn ColDesc;
        private DataGridViewTextBoxColumn ColPrice;
        private DataGridViewTextBoxColumn ColAmount;
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Keyboard Group
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private Button btn1;
        private Button btn2;
        private Button btn3;
        private Button btn4;
        private Button btn5;
        private Button btn6;
        private Button btn7;
        private Button btn8;
        private Button btn9;
        private Button btn0;
        private Button btn00;
        private Button btnPerc;
        private Button btnVoid;
        private Button btnPLU;
        private Button btnPre;
        private Button btnDot;
        private Button btnPiu;
        private Button btnMeno;
        private Button btnMoltiplica;
        private Button btnCancel;
        private Button btnSub;
        private PaymentButton btnPay1;
        private PaymentButton btnPay2;
        private PaymentButton btnPay3;
        private PaymentButton btnPay4;
        private Panel GroupKeypad;

    }

}