﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using POS.CashPlugin.Objects;
using POS.Framework;
using POS.Interfaces;

namespace POS.CashPlugin{

    public partial class CashPlugin : IPlugin{

        private System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(CashPlugin));
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Globals
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private string buffer = "0";
        private double buffermolti = 1;
        private int page = 0;
        private bool subtot;
        private Document m_doc;
        private int m_id;
        private bool printcoupon=true;
        
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Plugin Interface Functions
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public void Install(){

            if (Engine.HasAccess(resources.GetString("Checkout_Install")))
            {
    
            }

        }

        public void Uninstall(){

            if (Engine.HasAccess(resources.GetString("Checkout_Uninstall")))
            {
    
            }

        }


        public int Initialize(TabPage container){

            if (Engine.HasAccess(resources.GetString("Checkout_Load")))
            {

                InitializeUI(container);

                LoadSettings();

                ResetDevices();

                //temp solution until plugin dependencies
                //are not implemented. Get saved receipt number
                DataSet ds = ((IDb)Engine.DbService).GetData("SELECT total_value FROM jrn_totals WHERE total_name='" + resources.GetString("receiptnumber") + "'", 0, 0);
                if(ds.Tables[0].Rows.Count > 0){
                    m_id = Convert.ToInt32(ds.Tables[0].Rows[0][0]);
                }

                if(m_id <= 0){
                    m_id = 1;
                }

                m_doc = new Document(m_id);

                this.dgdDisplay.DataSource = m_doc.Transactions;

                m_doc.Transactions.ListChanged+=DocChanged;

                //it's bounded but doesn't paint.
                //it doesn't work
                //GetSpeed(page)

                return 0;

            }

            return 1;

        }

        public void TabEnter(){

            //why do you reload everytime ??
            //actually this is for first time
            //because in intialize above doesn//t work
            GetSpeed(page);

        }

        public int Message(int func, object data){
            //se e pieno viene cancellato quindi come clear
            if(func == 1){
                m_id = 1;

                this.lblTotale.Text = String.Format("{0:C2}", 0);

                ResetGlob();

                m_doc = new Document(m_id);
                dgdDisplay.DataSource = m_doc.Transactions;
                m_doc.Transactions.ListChanged+=DocChanged;
                return 0;
            }else{
                return -1;
            }
        }

        public void PosData(object dev, string data){

            subtot = false;

            try
            {
                m_doc.Transactions.Add(new SellingTransaction(data, buffermolti));
            }
            catch (ArgumentException ex)
            {
                // barcode not found
            }

            ResetGlob();

            lblDisplay.Text = "\r\n" + "0";

        }

        /// <summary>
        /// Temp solution, in future there will be a custom settings provider
        /// that pull settings from sql and is initialized in Engine
        /// </summary>
        public void SetProperties(){

            if(Engine.HasAccess(resources.GetString("Checkout_Settings"))){

                SettingsWindow frm = new SettingsWindow();
                
                // USER SCOPE SETTINGS
                // load current settings for payment buttons
                frm.DataGridViewPay.Rows[(this.btnPay2.Top) / 73].Cells[0].Value = this.btnPay2.Text;
                frm.DataGridViewPay.Rows[(this.btnPay2.Top) / 73].Cells[0].Tag = this.btnPay2;
                frm.DataGridViewPay.Rows[(this.btnPay4.Top) / 73].Cells[0].Value = this.btnPay4.Text;
                frm.DataGridViewPay.Rows[(this.btnPay4.Top) / 73].Cells[0].Tag = this.btnPay4;
                frm.DataGridViewPay.Rows[(this.btnPay3.Top) / 73].Cells[0].Value = this.btnPay3.Text;
                frm.DataGridViewPay.Rows[(this.btnPay3.Top) / 73].Cells[0].Tag = this.btnPay3;
                frm.DataGridViewPay.Rows[(this.btnSub.Top) / 73].Cells[0].Value = "SUB";
                frm.DataGridViewPay.Rows[(this.btnSub.Top) / 73].Cells[0].Tag = this.btnSub;
                frm.DataGridViewPay.Rows[(this.btnSub.Top) / 73].DefaultCellStyle.BackColor = Color.Yellow;
                frm.DataGridViewPay.Rows[(this.btnPay1.Top) / 73].Cells[0].Value = this.btnPay1.Text;
                frm.DataGridViewPay.Rows[(this.btnPay1.Top) / 73].Cells[0].Tag = this.btnPay1;

                frm.chklefthand.DataBindings.Clear();
                frm.chklefthand.DataBindings.Add("checked", Properties.Settings.Default, "lefthanded");

                //APPLICATION SCOPE
                // can't use binding because if the user press cancel
                // I have to reload old settings from db
                frm.text1.Text = btnPg1.Text;
                frm.text2.Text = btnPg2.Text;
                frm.text3.Text = btnPg3.Text;
                frm.text4.Text = btnPg4.Text;
                frm.text5.Text = btnPg5.Text;

                frm.chkcoupon.Checked = printcoupon;

                DialogResult ret = frm.ShowDialog();

                if (ret == DialogResult.OK)
                {

                    // update and save
                    printcoupon = frm.chkcoupon.Checked;
                    if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + printcoupon.ToString() + "',type='CashPlugin' where name='printcoupon'") == 0)
                    {
                        ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('printcoupon','" + printcoupon.ToString() + "','CashPlugin')");
                    }

                    if (Engine.HasAccess(resources.GetString("Checkout_RenameSpeedPage")))
                    {

                        // update page buttons text
                        btnPg1.Text = frm.text1.Text;
                        btnPg2.Text = frm.text2.Text;
                        btnPg3.Text = frm.text3.Text;
                        btnPg4.Text = frm.text4.Text;
                        btnPg5.Text = frm.text5.Text;

                        // save page buttons text
                        if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + btnPg1.Text + "',type='CashPlugin' where name='btnPg1.Text'") == 0)
                        {
                            ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('btnPg1.Text','" + btnPg1.Text + "','CashPlugin')");
                        }
                        if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + btnPg2.Text + "',type='CashPlugin' where name='btnPg2.Text'") == 0)
                        {
                            ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('btnPg2.Text','" + btnPg2.Text + "','CashPlugin')");
                        }
                        if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + btnPg3.Text + "',type='CashPlugin' where name='btnPg3.Text'") == 0)
                        {
                            ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('btnPg3.Text','" + btnPg3.Text + "','CashPlugin')");
                        }
                        if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + btnPg4.Text + "',type='CashPlugin' where name='btnPg4.Text'") == 0)
                        {
                            ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('btnPg4.Text','" + btnPg4.Text + "','CashPlugin')");
                        }
                        if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + btnPg5.Text + "',type='CashPlugin' where name='btnPg5.Text'") == 0)
                        {
                            ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('btnPg5.Text','" + btnPg5.Text + "','CashPlugin')");
                        }
                        

                    }

                    //reload left handed setting
                    this.GroupKeypad.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
                    this.GroupPage.Left = Properties.Settings.Default.lefthanded ? 495 : 21;
                    this.btnSwitchMode.Left = Properties.Settings.Default.lefthanded ? 1002 : 0;
                    this.GroupSpeed.Left = Properties.Settings.Default.lefthanded ? 495 : 21;
                    this.lblDisplay.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
                    this.lblTotale.Left = Properties.Settings.Default.lefthanded ? 289 : 825;
                    this.lblTot.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
                    this.pnlFake.Left = Properties.Settings.Default.lefthanded ? 21 : 557;
                    this.pnlFakeSpeed.Left = Properties.Settings.Default.lefthanded ? 495 : 21;

                    // manual set back values (binding is not working)
                    Properties.Settings.Default.btnPay1Top = this.btnPay1.Top;
                    Properties.Settings.Default.btnPay2Top = this.btnPay2.Top;
                    Properties.Settings.Default.btnPay3Top = this.btnPay3.Top;
                    Properties.Settings.Default.btnPay4Top = this.btnPay4.Top;
                    Properties.Settings.Default.btnSubTop = this.btnSub.Top;

                    // Save Settings
                    Properties.Settings.Default.Save();

                }
                else if (ret == DialogResult.Cancel)
                {
                    Properties.Settings.Default.Reload();
                }

                Engine.HideKeyboard();

            }

        }

        /// <summary>
        /// Temp solution, in future there will be a custom settings provider
        /// that pull settings from sql and is initialized in Engine
        /// </summary>
        public void LoadSettings()
        {

            DataSet config_ds = ((IDb)Engine.DbService).GetData("SELECT * FROM posxp_settings WHERE type='CashPlugin'", 0, 0);

            if (config_ds.Tables[0].Rows.Count > 0)
            {
                DataColumn[] pk = new DataColumn[1];
                pk[0] = config_ds.Tables[0].Columns["Name"];
                config_ds.Tables[0].PrimaryKey = pk;

                DataRow found;

                found=config_ds.Tables[0].Rows.Find("btnPg1.Text");
                if(found!=null) btnPg1.Text = Convert.ToString(found["value"]);

                found = config_ds.Tables[0].Rows.Find("btnPg2.Text");
                if (found != null) btnPg2.Text = Convert.ToString(found["value"]);

                found = config_ds.Tables[0].Rows.Find("btnPg3.Text");
                if (found != null) btnPg3.Text = Convert.ToString(found["value"]);

                found = config_ds.Tables[0].Rows.Find("btnPg4.Text");
                if (found != null) btnPg4.Text = Convert.ToString(found["value"]);

                found = config_ds.Tables[0].Rows.Find("btnPg5.Text");
                if (found != null) btnPg5.Text = Convert.ToString(found["value"]);

                found = config_ds.Tables[0].Rows.Find("printcoupon");
                if (found != null) printcoupon = Convert.ToBoolean(found["value"]);
            }
            // default values assigned in designer

            config_ds = null;

        }


        private void DocChanged(object sender, ListChangedEventArgs e){

            this.lblTotale.Text = String.Format("{0:C2}", m_doc.Transactions.Totale);

            //update display, itemremoved is handled by Sub OnRemove because after that
            //item is not accessible anymore
            if(Engine.defaultdisplay!=null && e.ListChangedType != ListChangedType.ItemDeleted && 
        e.ListChangedType != ListChangedType.Reset){

    #if CONFIG_POSNET
                Engine.defaultdisplay.ClearText();
    #else
                    ////////////////////////////////////////
                    //Custom Syntax modify at your needs
                    ///////////////////////////////////////

                    //clear display
                    char[] bb = new char[1];
                    bb[0] = (char)12;
                    Engine.defaultdisplay.Write(bb, 0, 1);
                    
    #endif
                if(m_doc.Transactions.Count > 0){
                    //it doesnt use newindex because when amount changes
                    //the handler raisevent listchanged but doesn't pass newindex!!!!!
                    string str = m_doc.Transactions[m_doc.Transactions.Count - 1].Quantity + " " + m_doc.Transactions[m_doc.Transactions.Count - 1].Text;

                    str = str.PadRight(20);

                    if(str.Length > 20){
                        str = str.Substring(0, 20);
                    }

                    string str2 = String.Format("{0:f}", m_doc.Transactions[m_doc.Transactions.Count - 1].Amount) + System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol;

    #if CONFIG_POSNET
                    Engine.defaultdisplay.DisplayText(str, Microsoft.PointOfService.DisplayTextMode.Normal);
                    Engine.defaultdisplay.DisplayText(str2.PadRight(20), Microsoft.PointOfService.DisplayTextMode.Normal);
    #else
                        ////////////////////////////////////////
                        //Custom Syntax modify at your needs
                        ///////////////////////////////////////

                        //write transaction 
                        Engine.defaultdisplay.Write(str);
                        Engine.defaultdisplay.Write(str2.PadRight(20));
    #endif
                }

            }

        }

        public int BatchAdd(string text){

            int i = 0;
            String[] arr = text.Split(new String[] {"*"}, StringSplitOptions.RemoveEmptyEntries);

            foreach(string ar in arr){

                object[] param = ar.Split(new String[] {","}, StringSplitOptions.RemoveEmptyEntries);
                
                try{
                    TransactionType type = (TransactionType)Convert.ToInt32(param[0]); 
                    int pid = Convert.ToInt32(param[1]);
                    double qty = Convert.ToDouble(param[2]);
                    int value = Convert.ToInt32(param[3]);

                    if(type == TransactionType.vendita){
                        m_doc.Transactions.Add(new SellingTransaction(pid, qty, value));
                    }else{
                        m_doc.Transactions.Add(new ModifierTransaction(type, qty, value));
                    }

                    i += 1;

                }catch(ArgumentException mex){
                    //transaction not created
                }catch(Exception ex){
                    ErrorLogger.WriteToErrorLog(ex.Message, ex.StackTrace);
                }

            }

            return i;

        }

        private void ResetGlob(){

            buffer = "0";

            buffermolti = 1;

            subtot = false;

        }

    
        private void ResetDevices(){

            if(Engine.defaultprinter!=null){

    #if CONFIG_POSNET
    #if CONFIG_FISCAL
                Engine.defaultprinter.ResetPrinter();
    #else
                Engine.defaultprinter.ClearOutput();
    #endif
    #else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////
                Engine.defaultprinter.Write("k");
                Engine.defaultprinter.Write("K");
    #endif
            }


            //reset display
            if(Engine.defaultdisplay!=null){

    #if CONFIG_POSNET
                //Set and display time is a display function and depends on hardware
                // there is no POS .NET way to do this automatically 
                //char[] bb = new char[4];
                //bb[0] = (char)31;
                //bb[1] = (char)84;
                //bb[2] = (char)(DateTime.Now.Hour);
                //bb[3] = (char)(DateTime.Now.Minute);
                //Engine.defaultdisplay.Write(bb, 0, 4);

                //display date
                Engine.defaultdisplay.ClearText();
                Engine.defaultdisplay.DisplayText(DateTime.Now.ToString("ddd dd MMM yyyy").ToUpper().PadRight(20), Microsoft.PointOfService.DisplayTextMode.Normal);
    #else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////

                    //Set and display time
                    char[] bb = new char[4];
                    bb[0] = (char)31;
                    bb[1] = (char)84;
                    bb[2] = (char)(DateTime.Now.Hour);
                    bb[3] = (char)(DateTime.Now.Minute);
                    Engine.defaultdisplay.Write(bb, 0, 4);

                    //display date
                    Engine.defaultdisplay.Write(DateTime.Now.ToString("ddd dd MMM yyyy").ToUpper().PadRight(20));
    #endif

            }

        }


        private void Keypad_DoubleClick(object sender, EventArgs e){

            Keypad_Click(sender, e);

        }

        private void Keypad_Click(object sender, EventArgs e){
            
            if(buffer.Length < 12){

                //remove leading zero
                if (buffer.Equals("0")) buffer = "";

                switch(((Control)sender).Name){

                    case "btn1":
                        buffer+= "1";
                        break;
                    case "btn2":
                        buffer += "2";
                        break;
                    case "btn3":
                        buffer+= "3";
                        break;
                    case "btn4":
                        buffer += "4";
                        break;
                    case "btn5":
                        buffer += "5";
                        break;
                    case "btn6":
                        buffer += "6";
                        break;
                    case "btn7":
                        buffer += "7";
                        break;
                    case "btn8":
                        buffer += "8";
                        break;
                    case "btn9":
                        buffer += "9";
                        break;
                    case "btn0":
                        if(!buffer.Equals("0")) buffer += "0";
                        break;
                    case "btn00":
                        // avoid empty buffer if 00 clicked
                        if (buffer.Equals("")) buffer = "0";
                        if(!buffer.Equals("0")) buffer += "00";
                        break;
                    //case "btnDot":
                    //    buffer=Convert.ToDouble(buffer.ToString() + ",");
                    //    break;
                }
            }

            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            //update display label 
            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            if(!buffer.Equals("0")){

                lblDisplay.Text = resources.GetString("qtytxt") +"     " + buffermolti + " X" + "\r\n" + resources.GetString("pricetxt") + "     " + String.Format("{0:c2}", (Convert.ToDouble(buffer) / 100));

            }

        }

        private void btnVoid_Click(object sender, EventArgs e){
            
            m_doc.Transactions.Clear();

            ResetGlob();

            lblDisplay.Text = "\r\n" + "0";

            Application.DoEvents();

            ResetDevices();

        }

        private void btnCancel_Click(object sender, EventArgs e){

            if(buffer.Equals("0")){
                lblDisplay.Text = "\r\n" + "0";
                buffermolti = 1;
                subtot = false;
            }else{
                buffer = "0";

                lblDisplay.Text = resources.GetString("qtytxt") + "     " + buffermolti + " X" + "\r\n" + resources.GetString("pricetxt") + "     " + buffer;

            }

        }

        private void btnMoltiplica_Click(object sender, EventArgs e){

            if(!String.IsNullOrEmpty(buffer)){
                buffermolti *= Convert.ToDouble(buffer);
                buffer = "0";

                lblDisplay.Text = resources.GetString("qtytxt") + "     " + buffermolti + " X" + "\r\n" + resources.GetString("pricetxt") + "     " + buffer;

            }

            dgdDisplay.Refresh();

        }

        private void btnPre_Click(object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Checkout_Preprint")))
            {
    
                if(dgdDisplay.Rows.Count > 0){

                    PrintThread cl = new PrintThread();
                    cl.p_doc = m_doc;
                    
                    #if CONFIG_FISCAL
                    System.Threading.Thread t = new System.Threading.Thread(cl.PrintNonFiscal);
                    t.Start();
                    #else
                    cl.printcoupon = false;
                    System.Threading.Thread t = new System.Threading.Thread(cl.Print);
                    t.Start();
                    #endif

                }

            }

        }

        private void btnPLU_Click(object sender, EventArgs e){

            subtot = false;

            if(!buffer.Equals("0")){

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //usare funzione script invece che add
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                try
                {
                    m_doc.Transactions.Add(new SellingTransaction(buffer, buffermolti));
                }
                catch (ArgumentException ex)
                {
                }

                ResetGlob();

                lblDisplay.Text = "\r\n" + "0";

            }

        }

        private void btnPerc_Click(object sender, EventArgs e){

            if(!String.IsNullOrEmpty(buffer)){
                try{
                    if(subtot == false){
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.scontopercento, 1, Convert.ToInt32(buffer)));
                    }else{
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.scontotot, 1, Convert.ToInt32(buffer)));
                    }
                }catch(ArgumentException ex){
                    //for example when discount value > 100
                }

                ResetGlob();
                lblDisplay.Text = "\r\n" + "0";
                pnlDisplay.AutoScrollPosition = new Point(0, dgdDisplay.Size.Height - pnlDisplay.Height - 12);
            }

            subtot = false;

        }

        private void btnMeno_Click(object sender, EventArgs e){

            if (!String.IsNullOrEmpty(buffer))
            {
                try{
                    if(subtot == false){
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.scontovalore, buffermolti, Convert.ToInt32(buffer)));
                    }else{
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.scontovaltot, buffermolti, Convert.ToInt32(buffer)));
                    }
                }catch(Exception ex){

                }

                ResetGlob();
                lblDisplay.Text = "\r\n" + "0";
                pnlDisplay.AutoScrollPosition = new Point(0, dgdDisplay.Size.Height - pnlDisplay.Height - 12);
            }

        }

        private void btnPiu_Click(object sender, EventArgs e){

            if (!String.IsNullOrEmpty(buffer))
            {
                try{
                    if(subtot == false){
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.maggvalore, buffermolti, Convert.ToInt32(buffer)));
                    }else{
                        m_doc.Transactions.Add(new ModifierTransaction(TransactionType.maggvaltot, buffermolti, Convert.ToInt32(buffer)));
                    }
                }catch(Exception ex){

                }

                ResetGlob();
                lblDisplay.Text = "\r\n" + "0";
                pnlDisplay.AutoScrollPosition = new Point(0, dgdDisplay.Size.Height - pnlDisplay.Height - 12);
            }

            subtot = false;

        }

        private void btnSub_Click(object sender, EventArgs e){

            if (m_doc.Transactions.Count > 0)
            {

                if(Engine.defaultdisplay!=null){
                    #if CONFIG_POSNET
                    Engine.defaultdisplay.ClearText();
                    Engine.defaultdisplay.DisplayText(resources.GetString("lblTot.Text").PadRight(20));
                    Engine.defaultdisplay.DisplayText(String.Format("{0:c}", m_doc.Transactions.Totale).PadRight(20));
                    #else
                    ////////////////////////////////////////
                    //Custom Syntax modify at your needs
                    ///////////////////////////////////////

                    //Clear display
                    char[] bb = new char[1];
                    bb[0] = (char)12;
                    Engine.defaultdisplay.Write(bb, 0, 1);
                    //show subtotal
                    Engine.defaultdisplay.Write(resources.GetString("lblTot.Text").PadRight(20));
                    Engine.defaultdisplay.Write(String.Format("{0:c}", m_doc.Transactions.Totale).PadRight(20));

                    #endif
                
                }

                ResetGlob();
            
                subtot = true;

                lblDisplay.Text = resources.GetString("subtottxt") + "\r\n" + String.Format("{0:c2}", m_doc.Transactions.Totale);

            }

        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Salva il documento e lo stampa 
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private void btnPay1_Click(object sender, EventArgs e){

            if(m_doc.Transactions.Count > 0){

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //Aggiungi la cifra versata nel tipo di pagamento
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                PaymentButton temp = (PaymentButton)sender;

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //aggiunge quello che manca per chiudere il conto
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                if(buffer.Equals("0")){
                    //this should be m_doc.Change but I have to fix problems with doubles in transactions
                    buffer = Convert.ToString(m_doc.Transactions.Totale * 100 - m_doc.Payments.Totale);
                }

                m_doc.Payments.Add(new Payment(temp.Text, Convert.ToInt32(buffer), temp.Paid, temp.Code));

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //se i pagamenti non pagano tutto il conto
                //esci dalla sub
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                if(m_doc.Change < 0){

                    lblDisplay.Text = temp.Text + " " + String.Format("{0:C2}",(Convert.ToDouble(buffer) / 100)) + "\r\n" +
                    resources.GetString("duetxt") + " " + String.Format("{0:C2}", (m_doc.Change / (double)100 * (-1)));

                    ResetGlob();
                    return;
                }

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //se l'ammontare e uguale o superiore mostra il resto
                //e stampa il documento
                //
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                lblDisplay.Text = temp.Text + " " + String.Format("{0:C2}", (Convert.ToDouble(buffer) / 100)) + "\r\n" +
                resources.GetString("changetxt") + " " + String.Format("{0:C2}", (m_doc.Change / (double)100));

                //Print change on display
                if(Engine.defaultdisplay!=null){
    #if CONFIG_POSNET
                    Engine.defaultdisplay.ClearText();
                    Engine.defaultdisplay.DisplayText(resources.GetString("changetxt").PadRight(20));
                    Engine.defaultdisplay.DisplayText(String.Format("{0:C2}", (m_doc.Change / (double)100)).PadRight(20));
    #else
                                ////////////////////////////////////////
                                //Custom Syntax modify at your needs
                                ///////////////////////////////////////

                                //clear display
                                char[] bb= new char[1];
                                bb[0] = (char)12;
                                Engine.defaultdisplay.Write(bb, 0, 1);
                                //show change
                                Engine.defaultdisplay.Write(resources.GetString("changetxt").PadRight(20));
                                Engine.defaultdisplay.Write(String.Format("{0:C2}",(m_doc.Change / 100)).PadRight(20));
#endif
                }

                PrintThread cl = new PrintThread();
                cl.p_doc = m_doc;
                cl.printcoupon = printcoupon;
                System.Threading.Thread t = new System.Threading.Thread(cl.PrintandSave);
                t.Start();

                this.lblTotale.Text = String.Format("{0:C2}", 0);

                ResetGlob();

                m_id += 1;
                m_doc = new Document(m_id);

                dgdDisplay.DataSource = m_doc.Transactions;

                m_doc.Transactions.ListChanged+=DocChanged;

            }

        }




        private void btnPg1_Click(object sender, EventArgs e){
            page = 0;
            GetSpeed(page);
        }

        private void btnPg2_Click(object sender, EventArgs e){
            page = 1;
            GetSpeed(page);
        }

        private void btnPg3_Click(object sender, EventArgs e){
            page = 2;
            GetSpeed(page);
        }

        private void btnPg4_Click(object sender, EventArgs e){
            page = 3;
            GetSpeed(page);
        }

        private void btnPg5_Click(object sender, EventArgs e){
            page = 4;
            GetSpeed(page);
        }

        public void GetSpeed(int l_page){

            string filepathL, filepathC, filepathR;

            if(System.IO.File.Exists(Engine.datapath + "speed" + Convert.ToString(l_page - 1) + ".xml")){
                filepathL = Engine.datapath + "speed" + Convert.ToString(l_page - 1) + ".xml";
            }else{
                filepathL = Engine.datapath + "speed.xml";
            }
            if(System.IO.File.Exists(Engine.datapath + "speed" + Convert.ToString(l_page) + ".xml")){
                filepathC = Engine.datapath + "speed" + Convert.ToString(l_page) + ".xml";
            }else{
                filepathC = Engine.datapath + "speed.xml";
            }
            if(System.IO.File.Exists(Engine.datapath + "speed" + Convert.ToString(l_page) + 1 + ".xml")){
                filepathR = Engine.datapath + "speed" + Convert.ToString(l_page + 1) + ".xml";
            }else{
                filepathR = Engine.datapath + "speed.xml";
            }

            DataSet dsL = new DataSet();
            DataSet dsC = new DataSet();
            DataSet dsR = new DataSet();
            
            dsL.ReadXml(filepathL);
            dsC.ReadXml(filepathC);
            dsR.ReadXml(filepathR);

            dsL.Tables[0].Columns.Add("sei");
            dsL.Tables[0].Columns.Add("sette");
            dsL.Tables[0].Columns.Add("otto");
            dsL.Tables[0].Columns.Add("nove");
            dsL.Tables[0].Columns.Add("dieci");
            dsL.Tables[0].Columns.Add("undici");
            dsL.Tables[0].Columns.Add("dodici");
            dsL.Tables[0].Columns.Add("tredici");
            dsL.Tables[0].Columns.Add("quattordici");
            dsL.Tables[0].Columns.Add("quindici");

            for(int i = 0;i<=8;i++){
                dsL.Tables[0].Rows[i][5] = dsC.Tables[0].Rows[i][0];
                dsL.Tables[0].Rows[i][6] = dsC.Tables[0].Rows[i][1];
                dsL.Tables[0].Rows[i][7] = dsC.Tables[0].Rows[i][2];
                dsL.Tables[0].Rows[i][8] = dsC.Tables[0].Rows[i][3];
                dsL.Tables[0].Rows[i][9] = dsC.Tables[0].Rows[i][4];
                dsL.Tables[0].Rows[i][10] = dsR.Tables[0].Rows[i][0];
                dsL.Tables[0].Rows[i][11] = dsR.Tables[0].Rows[i][1];
                dsL.Tables[0].Rows[i][12] = dsR.Tables[0].Rows[i][2];
                dsL.Tables[0].Rows[i][13] = dsR.Tables[0].Rows[i][3];
                dsL.Tables[0].Rows[i][14] = dsR.Tables[0].Rows[i][4];
            }

            dsC = null;
            dsR = null;
    
            dsL.Tables[0].Rows[8][9] = "164,0//" + Convert.ToString(page + 1);
    
            dgwSpeed.DataSource = dsL.Tables[0];

            pnlSpeed.AutoScrollPosition = new Point(500, 0);

            //page number
            dgwSpeed[9, 8].Style.Font = new Font("Microsoft Sans Serif", 60.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            //this is to avoid that cells outside visible area get selected
            //see also btnSpeedMod click handler
            dgwSpeed[5, 0].Selected = true;

        }

        private void btnSpeedMod_Click(object sender, EventArgs e){

            SpeedWindow speedform = new SpeedWindow();

            DataGridViewButtonCell dr = (DataGridViewButtonCell)dgwSpeed.CurrentCell;

            //cells in the swipe 'buffer' can't be modified
            //it doesn't happen but if someone uses keyboard
            //to move selected cell
            if((dr.ColumnIndex > 4) && (dr.ColumnIndex < 10) && !((dr.ColumnIndex%5) == 4 && dr.RowIndex == 8)){


                if(System.IO.File.Exists(Engine.datapath + "speed" + Convert.ToString(page) + ".xml")){
                }else{
                    System.IO.File.Copy(Engine.datapath + "speed.xml", Engine.datapath + "speed" + Convert.ToString(page) + ".xml");
                }
                
                string filePath = Engine.datapath + "speed" + Convert.ToString(page) + ".xml";

                DataSet ds = new DataSet();

                ds.ReadXml(filePath);

                speedform.speedstring = Convert.ToString(ds.Tables[0].Rows[dr.RowIndex][dr.ColumnIndex % 5]);

                if(speedform.ShowDialog() == DialogResult.OK){
                    ds.Tables[0].Rows[dr.RowIndex][dr.ColumnIndex % 5] = speedform.speedcolor + "," + speedform.speedimage + "/" + speedform.speedstring + "/" + speedform.speedtext;
                }else{
                    return;
                }

                ds.WriteXml(filePath);

                GetSpeed(page);

                speedform = null;

            }

        }

        private void btnSpeedRem_Click(object sender, EventArgs e){

            DataGridViewButtonCell dr = (DataGridViewButtonCell)dgwSpeed.CurrentCell;

            //cells in the swipe 'buffer' can't be modified
            //it doesn't happen but if someone uses keyboard
            //to move selected cell
            if((dr.ColumnIndex > 4) && (dr.ColumnIndex < 10) && !((dr.ColumnIndex % 5) == 4 && dr.RowIndex == 8)){

                PopBox pop = new PopBox();

                if(pop.ShowDialog(resources.GetString("msgspeedremtxt"), new Point(465, 300)) == DialogResult.Yes){

                    string filePath = Engine.datapath + "speed" + Convert.ToString(page) + ".xml";

                    DataSet ds = new DataSet();

                    ds.ReadXml(filePath);
   
                    ds.Tables[0].Rows[dr.RowIndex][dr.ColumnIndex % 5] = "164,0//";

                    ds.WriteXml(filePath);

                    GetSpeed(page);

                }

            }

        }

        private void btnPageRem_Click(object sender, EventArgs e){

            PopBox pop = new PopBox();

            if(pop.ShowDialog(resources.GetString("msgpageremtxt"), new Point(465, 300)) == DialogResult.Yes){

                string filePath = Engine.datapath + "speed" + Convert.ToString(page) + ".xml";

                if(System.IO.File.Exists(filePath)){

                    System.IO.File.Delete(filePath);
                    //what is this for??
                    GetSpeed(page);

                }

            }

        }

        private void btnSwitchMode_Click(object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Checkout_ModifySpeed")))
            {

                if(GroupSpeed.Visible){
                    this.GroupPage.Visible = true;
                    this.GroupSpeed.Visible = false;
                    dgwSpeed.DefaultCellStyle.SelectionBackColor = Color.White;
                }else{
                    this.GroupPage.Visible = false;
                    this.GroupSpeed.Visible = true;
                    dgwSpeed.DefaultCellStyle.SelectionBackColor = Color.Lime;
                    //this is to avoid that cells outside visible area get selected
                    //see also btnSpeedMod click handler
                    dgwSpeed[5, 0].Selected = true;
                }

            }

        }

        private void dgwSpeed_CellClick(object sender, DataGridViewCellEventArgs e){

            if(GroupPage.Visible && !((e.ColumnIndex % 5) == 4 && e.RowIndex == 8)){

                DataGridViewButtonProductsCell dr = (DataGridViewButtonProductsCell)dgwSpeed.CurrentCell;

                string[] arr = dr.Command.Split(new String[] {"*"}, StringSplitOptions.RemoveEmptyEntries);

                string comm = "";

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //se sono stati immessi la quantita o il prezzo
                //modifico la stringa comando
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                if(buffermolti != 1 || !buffer.Equals("0")){

                    foreach(string ar in arr){

                        string[] param = ar.Split(new String[] {","}, StringSplitOptions.RemoveEmptyEntries);

                        //cancella param[0] == 1 && , cosi moltiplica anche sconti e maggiorazioni
                        if(buffermolti != 1){
                            param[2] = buffermolti.ToString();
                        }

                        if (param[0].Equals("1") && !buffer.Equals("0"))
                        {
                            param[3] = buffer;
                        }

                        comm += "*" + param[0] + "," + param[1] + "," + param[2] + "," + param[3] + "*";

                    }

                    this.BatchAdd(comm);

                    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                    //altrimenti la invio cosi com'e'
                    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                }else{

                    this.BatchAdd(dr.Command);

                }

                ResetGlob();

                lblDisplay.Text = "\r\n" + "0";

                ///////////////////////////////////////
                //datagridview is a crap
                ///////////////////////////////////////
                dgdDisplay.Refresh();

                pnlDisplay.AutoScrollPosition = new Point(0, dgdDisplay.Size.Height - pnlDisplay.Height - 12);

            }

        }

        private void dgwSpeed_CellDoubleClick(object sender, DataGridViewCellEventArgs e){
            dgwSpeed_CellClick(this, new DataGridViewCellEventArgs(e.ColumnIndex, e.RowIndex));
        }

        //this is for cell context menu
        private void SpeedModMenuItem_Click(object sender, EventArgs e){

        }

        private void ContextMenuStrip_Closing(object sender, ToolStripDropDownClosingEventArgs e){
            dgwSpeed.DefaultCellStyle.SelectionBackColor = Color.White;
        }

        private void dgwSpeed_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e){
            if(e.Button == MouseButtons.Right){
                dgwSpeed.DefaultCellStyle.SelectionBackColor = Color.Lime;
                dgwSpeed[e.ColumnIndex, e.RowIndex].Selected = true;
            }
        }

        #region TouchScroll

        private Point mouse_down;
        private Rectangle dragBoxFromMouseDown;
        private bool switched;
        private Point temp; 

        private void dgwSpeed_MouseDown(object sender, MouseEventArgs e){
            mouse_down = new Point(e.X, e.Y);
            temp = pnlSpeed.AutoScrollPosition;
            switched = false;
            Size dragSize = new Size(200, 200);
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void dgwSpeed_MouseMove(object sender, MouseEventArgs e){

            if(e.Button == MouseButtons.Left && !switched){

                if(!(Math.Abs(temp.X - pnlSpeed.AutoScrollPosition.X) > dragBoxFromMouseDown.Width)){
                    int dX = (mouse_down.X - e.X);
                    int dY = (mouse_down.Y - e.Y);
                    pnlSpeed.AutoScrollPosition = new Point((dX - pnlSpeed.AutoScrollPosition.X), (dY - pnlSpeed.AutoScrollPosition.Y));
                }else{

                    switched = true;

                    if((temp.X - pnlSpeed.AutoScrollPosition.X) < 0){
                        if(page > 0){
                            page -= 1;
                            GetSpeed(page);
                        }else{
                            //put the scroll back, there is no -1 page
                            pnlSpeed.AutoScrollPosition = new Point(500, 0);
                        }
                    }else{
                        page += 1;
                        GetSpeed(page);
                    }

                }

            }

        }

        private void dgwSpeed_MouseUp(object sender, MouseEventArgs e){

            //put the scroll back
            if(!switched){
                pnlSpeed.AutoScrollPosition = new Point(500, 0);
            }

        }

        private void dgdDisplayCellClick(Point point){

            int row = dgdDisplay.HitTest(point.X, point.Y).RowIndex;

            if(row > -1){

                PopBox pop = new PopBox();
                dgdDisplay.Rows[row].DefaultCellStyle.SelectionBackColor = Color.Red;

                if (pop.ShowDialog(resources.GetString("msgdeltranstxt"), new Point(635, 300)) == DialogResult.Yes){

                    dgdDisplay.Rows.RemoveAt(row);
                    ResetGlob();
                    lblDisplay.Text = "\r\n" + "0";

                }

                //this way works even when row is forced to not get deleted
                if(row < dgdDisplay.RowCount){
                    dgdDisplay.Rows[row].DefaultCellStyle.SelectionBackColor = Color.White;
                }

            }

        }

        private void dgdDisplay_MouseDown(object sender, MouseEventArgs e){
            mouse_down = new Point(e.X, e.Y);
            switched = false;
            temp = pnlDisplay.AutoScrollPosition;
            Size dragSize = SystemInformation.DragSize;
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void dgdDisplay_MouseMove(object sender, MouseEventArgs e){

            if(e.Button == MouseButtons.Left){
                int dX = (mouse_down.X - e.X);
                int dY = (mouse_down.Y - e.Y);
                pnlDisplay.AutoScrollPosition = new Point((dX - pnlDisplay.AutoScrollPosition.X), (dY - pnlDisplay.AutoScrollPosition.Y));
                if(Math.Abs(temp.Y - pnlDisplay.AutoScrollPosition.Y) > dragBoxFromMouseDown.Height || Math.Abs(mouse_down.Y - e.Y) > dragBoxFromMouseDown.Height){
                    switched = true;
                }
            }

        }

        //this handles one click if mouse was not dragged
        private void dgdDisplay_MouseUp(object sender, MouseEventArgs e){
            //delete the row
            if(!switched){
                dgdDisplayCellClick(e.Location);
            }
        }

        #endregion

    }

}