﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using POS.Framework;
using POS.ProductPlugin.Objects;

namespace POS.CashPlugin.Objects{

    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    //Document Class to manage all the documents
    //
    //
    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    public class Document{

        private int int_id;
        private PaymentCollection coll_pay = new PaymentCollection();
        private TransactionCollection coll_trans = new TransactionCollection();
        //private BindingContext Bindcont;
        //private ControlBindingsCollection Bindcoll;

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //CREA un NUOVO documento DA inserire con valori di DEFAULT
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public Document(int id){
            int_id = id;
        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Proprietà ID classe Transaction
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public int Id{

            get{
                return int_id;
            }

            set{
                int_id = value;
            }

        }

        public DateTime Data{

            get{
                return DateTime.Now;
            }

        }

        public int Change{

            get{
                return (this.Payments.Totale - Convert.ToInt32(this.Transactions.Totale * 100));
            }

        }

        public PaymentCollection Payments{

            get{
                return coll_pay;
            }

            set{
                coll_pay = value;
            }

        }

        public TransactionCollection Transactions{

            get{
                return coll_trans;
            }

            set{
                coll_trans = value;
            }

        }

    }



    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    //Base Transaction Class
    //
    //
    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

    public enum TransactionType{
        vendita = 1,
        scontovalore = 2,
        scontopercento = 3,
        scontotot = 4,
        scontovaltot = 5,
        maggvalore = 6,
        maggpercento = 7,
        maggvaltot = 8,
        maggtot = 9
    }

    public abstract class Transaction{

        private double dbl_quantity;
        private double dbl_prezzo;
        public delegate void AmountchangedDelegate(double diff);
        public AmountchangedDelegate Amountchanged;

        public abstract TransactionType Type
        {
            get;
        }

        public abstract string Text
        {
            get;
        }

        public double Quantity{

            get{
                return dbl_quantity;
            }

            set{
                double old = this.Amount;
                dbl_quantity = value;
                if (Amountchanged != null)
                {
                    Amountchanged(-(old - this.Amount));
                }
            }

        }

        public double Value{

            get{
                return dbl_prezzo;
            }
            set{
                double old = this.Amount;
                dbl_prezzo = value / 100;
                if (Amountchanged != null)
                {
                    Amountchanged(-(old - this.Amount));
                }
            }

        }

        public double Amount{

            get{
                return Quantity * Value;
            }

        }

#if !CONFIG_POSNET
        public abstract string Code{
            get;
        }
#endif

    }

    public class SellingTransaction : Transaction{

        private Product prod_prod;

        public override TransactionType Type{
            get
            {
                return TransactionType.vendita;
            }
        }

        public override string Text
        {
            get
            {
                return this.Product.Name;
            }
        }

        public Product Product{

            get{
                return prod_prod;
            }

        }


        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //new transaction constructor
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public SellingTransaction(int pid){
            
            try{
                prod_prod = new Product(pid);
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }

            base.Quantity = 1;
            
            base.Value = this.Product.Price;

        }

        public SellingTransaction(int pid, double quantity){
            
            try{
                prod_prod = new Product(pid);
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }

            //just to be sure..
            if (quantity < 1){
                base.Quantity = 1;
            }else{
                base.Quantity = quantity;
            }

            base.Value = this.Product.Price;

        }

        public SellingTransaction(int pid, double quantity, double prezzo){
            
            try{
                prod_prod = new Product(pid);
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }
            
            //just to be sure..
            if (quantity < 1){
                base.Quantity = 1;
            }else{
                base.Quantity = quantity;
            }

            if(prezzo!=0){
                base.Value = prezzo;
            }else{
                base.Value = this.Product.Price;
            }

        }

        /// <summary>
        /// Transaction constructor from EAN code
        /// </summary>
        /// <param name="ean"></param>
        public SellingTransaction(string ean){
            //Optional ByVal quantity As Double = 1
            try{
                prod_prod = new Product(ean);
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }

            base.Quantity = 1;

            base.Value = this.Product.Price;

        }

        public SellingTransaction(string ean, double quantity){
            
            try{
                prod_prod = new Product(ean);
            }catch(ArgumentException ex){
                throw ex;
            }

            //just to be sure..
            if (quantity < 1){
                base.Quantity = 1;
            }else{
                base.Quantity = quantity;
            }

            base.Value = this.Product.Price;

        }

#if !CONFIG_POSNET
        protected override string GetCode(){
            return "\"" + this.Product.Name + "\"" + this.Quantity + "*" + this.Value * 100 + "H1R";
        }
#endif

    }

    public class ModifierTransaction : Transaction{

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(CashPlugin));
        private TransactionType type_type;
        private string str_text;
        private int int_percent;

        public override TransactionType Type
        {
            get
            {
                return type_type;
            }
        }

        public override string Text
        {
            get
            {
                return str_text;
            }
        }

        public int Percentage{
            get{
                return int_percent;
            }
        }

        public ModifierTransaction(TransactionType type , double quantity, int value){

            if (!(1 < (int)type && (int)type < 10))
            {
                throw new ArgumentException();
            }

            type_type = type;

            //just to be sure..
            if(quantity < 1) base.Quantity = 1;

            base.Quantity = quantity;

            if(value <= 0){
                throw new ArgumentException();
            }

            this.Value = value;

            if((type == TransactionType.scontopercento || type == TransactionType.scontotot ||
            type == TransactionType.maggpercento || type == TransactionType.maggtot) && value >= 100){
                throw new ArgumentException();
            }

            switch(type){
                case TransactionType.scontovalore:
                    str_text = resources.GetString("discounttxt"); //+ value / 100 + " " + Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol
                    this.Value = -value;
                    break;
                case TransactionType.scontovaltot:
                    str_text = resources.GetString("totaldistxt"); //+ value / 100 + " " + Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol
                    this.Value = -value;
                    break;
                case TransactionType.maggvalore:
                    str_text = resources.GetString("surchargetxt"); //+ value / 100 + " " + Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol
                    break;
                case TransactionType.maggvaltot:
                    str_text = resources.GetString("totalsurtxt"); //+ value / 100 + " " + Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol
                    break;
                case TransactionType.scontopercento:
                    str_text = resources.GetString("discounttxt") + value + "%";
                    this.int_percent = value;
                    break;
                case TransactionType.scontotot:
                    str_text = resources.GetString("totaldistxt") + value + "%";
                    this.int_percent = value;
                    break;
                case TransactionType.maggpercento:
                    str_text = resources.GetString("surchargetxt") + value + "%";
                    this.int_percent = value;
                    break;
                case TransactionType.maggtot:
                    str_text = resources.GetString("totalsurtxt") + value + "%";
                    this.int_percent = value;
                    break;
            }

        }

#if !CONFIG_POSNET
        protected override string GetCode(){

            string res = "";
            //there is an error with total percentage modifiers and non fiscal products
            //modifier value is calculated on all products fiscal and non-fiscal
            //but only fiscal ones are sent to the printer.
            //this has been solved only for pos .net version
            switch(this.Type){
                case TransactionType.scontovalore:
                    res = "\"" + this.Text + "\"" + Math.Abs(this.Amount) * 100 + "H*3M";
                    break;
                case TransactionType.scontovaltot:
                    res = "\"" + this.Text + "\"" + Math.Abs(this.Amount) * 100 + "H*4M";
                    break;
                case TransactionType.maggvalore:
                    res = "\"" + this.Text + "\"" + Math.Abs(this.Amount) * 100 + "H*7M";
                    break;
                case TransactionType.maggvaltot:
                    res = "\"" + this.Text + "\"" + Math.Abs(this.Amount) * 100 + "H*8M";
                    break;
                case TransactionType.scontopercento:
                    res = "\"" + this.Text + "\"" + this.Percentage + "*1M";
                    break;
                case TransactionType.scontotot:
                    res = "\"" + this.Text + "\"" + this.Percentage + "*2M";
                    break;
                case TransactionType.maggpercento:
                    res = "\"" + this.Text + "\"" + this.Percentage + "*5M";
                    break;
                case TransactionType.maggtot:
                    res = "\"" + this.Text + "\"" + this.Percentage + "*6M";
                    break;

            }

            return res;

        }
#endif

    }


    /// <summary>
    /// TransactionCollection Class to group transactions for EVERY DOCUMENT
    /// </summary>
    public class TransactionCollection : CollectionBase, IBindingList{

        private double dbl_tot;
        private List<int> trans_dis = new List<int>();

        public TransactionCollection(){

        }

        public Transaction this[int index]{
            get{
                return (Transaction) this.List[index];
            }
        }

        /// <summary>
        /// Add transaction to the collection.
        /// All the ECR logic is almost done here
        /// </summary>
        public void Add(Transaction transazione){

            //zero priced transaction are not valid (this check should never get inside)
            if(transazione.Value == 0) return;

            //bug: discount as first transaction
            if(Count == 0 && transazione.Type != TransactionType.vendita) return;

            if(this.Count > 0){

                if(this[Count - 1].Value == transazione.Value && this[Count - 1].Type == transazione.Type){

                    if(transazione.Type == TransactionType.vendita){
                        if(((SellingTransaction)this[Count - 1]).Product.Id == ((SellingTransaction)transazione).Product.Id){
                            this[Count - 1].Quantity += transazione.Quantity;
                            // return without adding new transaction
                            return;
                        }

                    }

                }

                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                //
                //transaction modifier: discount, surcharge
                //
                //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                if((int)transazione.Type > 1){

                    if(this[Count - 1].Type != TransactionType.vendita) return;

                    switch(transazione.Type){

                        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        //value surcharge on item
                        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.maggvalore:
                            break;
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //value surcharge on total
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.maggvaltot:

                            if(this.List.Count == 1) return;
                            break;
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //value discount on item
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.scontovalore:

                            if(transazione.Amount * -1 >= this[Count - 1].Amount){
                                return;
                            }
                            break;
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //value discount on total
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.scontovaltot:

                            if(transazione.Amount * -1 >= this.Totale || this.List.Count == 1){
                                return;
                            }
                            break;
                            ///////////////////////////////////////////
                            //PERCENTAGE MODIFIER DO NOT GET MULTIPLIED
                            //BECAUSE THEY ARE PERCENTAGE, SO THEY
                            //WORK CORRECTLY WITH MULTIPLE ITEMS AND SPEED BUTTONS
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //percentage discount on total
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.scontotot:

                            //for the moment only one total discount max
                            if(SubTotal.Count > 0 || this.List.Count == 1){
                                return;
                            }

                            transazione.Quantity = 1;

                            transazione.Value = Totale * 100 * transazione.Value * -1;

                            SubTotal.Add(this.List.Count);

                            break;

                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //percentage discount on item
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.scontopercento:

                            transazione.Value = this[Count - 1].Amount * 100 * transazione.Value * -1;
                            break;

                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //percentage surcharge on total
                            //not supported for the moment
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.maggtot:

                            return;

                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                            //percentage surcharge on item
                            //(actually you can't apply this with the keyboard only speed button)
                            //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
                        case TransactionType.maggpercento:

                            transazione.Value = this[Count - 1].Amount * 100 * transazione.Value;

                            break;
                    }

                }

            }

            List.Add(transazione);

        }

        public void Remove(Transaction transazione){

            List.Remove(transazione);

        }

        public List<int> SubTotal{
            get{
                return trans_dis;
            }
        }

        public double Totale{
            get{
                return Math.Round(dbl_tot, 2);
            }
        }

        private void Amount(double diff){

            dbl_tot = dbl_tot + diff;

            ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemChanged, 0));

        }


#region IBindingList
        //
        //
        //
        public event ListChangedEventHandler ListChanged;

        //
        //
        //
        bool IBindingList.SupportsChangeNotification
        {
            //Implements System.ComponentModel.IBindingList.SupportsChangeNotification
            get{
                return true;
            }
        }

        // <summary>
        // Returns True to indicate that we support automatic addition
        // of new line item objects to the collection.
        // </summary>
        bool IBindingList.AllowNew
        {
            get{
                return false;
            }
        }

        // <summary>
        // Returns True to indicate that we allow in-place editing of
        // line item objects.
        // </summary>
        // <returns></returns>
        bool IBindingList.AllowEdit
        {
            get{
                return false;
            }
        }

        // <summary>
        // Returns True to indicate that we support automatic removal
        // of line item objects from the collection.
        // </summary>
        bool IBindingList.AllowRemove
        {
            get{
                return true;
            }
        }

        // <summary>
        // Adds a new item to the collection on request by the UI control.
        // </summary>
        // <returns>The newly added line item object.</returns>
        object IBindingList.AddNew()
        {
            throw new InvalidOperationException();
        }


        // the remainder of the IBindingList methods are not implemented
        // we do not support searching or sorting
        bool IBindingList.SupportsSearching
        {
            get{
                return false;
            }
        }

        bool IBindingList.SupportsSorting
        {
            get{
                return false;
            }
        }

        bool IBindingList.IsSorted
        {
            get{
                return false;
            }
        }

        void IBindingList.AddIndex(PropertyDescriptor property)
        {
            throw new NotSupportedException();
        }

        void IBindingList.ApplySort(PropertyDescriptor property, ListSortDirection direction)
        {
            throw new NotSupportedException();
        }

        int IBindingList.Find(PropertyDescriptor property, object key)
        {
            throw new NotSupportedException();
        }

        void IBindingList.RemoveIndex(PropertyDescriptor property)
        {
            throw new NotSupportedException();
        }

        void IBindingList.RemoveSort()
        {
            throw new NotSupportedException();
        }

        ListSortDirection IBindingList.SortDirection
        {
            get{
                throw new NotSupportedException();
            }
        }

        PropertyDescriptor IBindingList.SortProperty
        {
            get{
                throw new NotSupportedException();
            }
        }

#endregion

#region Base Class Events

        // <summary>
        // Raises events to indicate that the collection has changed
        // and that our total amount has changed.
        // </summary>
        protected override void OnClearComplete(){

            dbl_tot = 0;

            SubTotal.Clear();

            ListChanged(this, new ListChangedEventArgs(ListChangedType.Reset, 0));

        }

        // <summary>
        // Raises event to indicate that the collection has changed
        // and our total amount has changed.
        // </summary>
        protected override void OnInsertComplete(int index, object value){

            ((Transaction)value).Amountchanged+= new Transaction.AmountchangedDelegate(Amount);

            dbl_tot = dbl_tot + ((Transaction)value).Amount;

            ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemAdded, index));

        }

        protected override void OnRemove(int index, object value){

            if(this.Totale - this[index].Amount <= 0){
                this.List.Insert(index, value);
                return;
            }

            if((index + 1) < this.List.Count){
                if(this[index + 1].Type == TransactionType.scontopercento ||
                this[index + 1].Type == TransactionType.scontovalore ||
                this[index + 1].Type == TransactionType.maggpercento ||
                this[index + 1].Type == TransactionType.maggvalore){
                    this.Remove(this[index + 1]);
                }
            }

            for(int i = 0;i<=SubTotal.Count - 1;i++){
                if(index < SubTotal[i]){
                    this[SubTotal[i]].Value = this[SubTotal[i]].Value * 100 + (this[index].Amount * ((ModifierTransaction)this[SubTotal[i]]).Percentage);
                    if(Math.Round(this[SubTotal[i]].Value, 2) == 0){
                        this.Remove(this[SubTotal[i]]);
                    }else{
                        this.SubTotal[i] -= 1;
                    }
                    break;
                }
            }

            if(this[index].Type == TransactionType.scontotot) SubTotal.Remove(index);
            
            //this is here because after the item is removed
            //it can't be acccessed anymore
            if(Engine.defaultdisplay!=null){
#if CONFIG_POSNET
                Engine.defaultdisplay.ClearText();
#else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////

                //clear display
                char[] bb = new char[1];
                bb[0] = (char)12;
                Engine.defaultdisplay.Write(bb, 0, 1);
#endif

                if(this.List.Count > 0){

                    string Str = "-" + this[index].Quantity + " " + this[index].Text;

                    Str = Str.PadRight(20);

                    if(Str.Length > 20) Str = Str.Substring(0, 20);

                    string str2 = String.Format("f",-this[index].Amount) + System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol;

#if CONFIG_POSNET
                    Engine.defaultdisplay.DisplayText(Str, Microsoft.PointOfService.DisplayTextMode.Normal);
                    Engine.defaultdisplay.DisplayText(str2.PadRight(20), Microsoft.PointOfService.DisplayTextMode.Normal);
#else
                    ////////////////////////////////////////
                    //Custom Syntax modify at your needs
                    ///////////////////////////////////////

                    //write transaction 
                    Engine.defaultdisplay.Write(str);
                    Engine.defaultdisplay.Write(str2.PadRight(20));
#endif
                }

            }

        }

        /// <summary>
        /// Raises event to indicate that the collection has changed 
        /// and our total amount has changed too.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="value"></param>
        protected override void OnRemoveComplete(int index, object value){

            dbl_tot = dbl_tot - ((Transaction)value).Amount;

            ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemDeleted, index));

        }

        // <summary>
        // Raises event to indicate that the collection has changed
        // and our total amount has changed too.
        // </summary>
        protected override void OnSetComplete(int index, object oldValue, object newValue){

            ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemChanged, index));

        }

#endregion

    }



    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    //Payments Classes
    //
    //
    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

    public class Payment{

        private string str_name;
        private int int_amount;
        private bool bool_paid;
        private string str_code;

        public Payment(string name, int amount, bool paid, string code){
            str_name = name;
            int_amount = amount;
            bool_paid = paid;
            str_code = code;
        }

        public string Name{

            get{
                return str_name;
            }

        }

        public int Amount{

            get{
                return int_amount;
            }

            set{
                int_amount = value;
            }

        }

        public bool Paid{

            get{
                return bool_paid;
            }

        }

#if !CONFIG_POSNET
        public string Code{
            get{
                return this.Amount + "H" + str_code;
            }
        }
#endif

    }

    
    public class PaymentCollection : CollectionBase{

        private int int_tot;

        public Payment this[int index]{
            get{
                return (Payment)List[index]; 
            }
        }

        public void Add(Payment pagamento){

            if(pagamento.Amount > 0){
                List.Add(pagamento);
                int_tot = int_tot + pagamento.Amount;
            }

        }

        public void Remove(Payment pagamento){

            List.Remove(pagamento);
            int_tot = int_tot - pagamento.Amount;

        }

        public int Totale{
            get{
                return int_tot;
            }
        }

    }

}