﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using POS.CashPlugin;
using POS.CashPlugin.Objects;
using POS.Framework;

namespace POS.CashPlugin
{

    class PrintThread
    {

        private static System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(CashPlugin));
        public Document p_doc;
        public bool printcoupon;

        public void PrintandSave()
        {
            Print();
            //may be I can do this on a separate thread
            POS.Objects.Message journal = new POS.Objects.Message();
            journal.func = 1;
            journal.pointer = p_doc;
            journal.MessageFiscalPlugin();
        }


        private void PrintCoupon(int price)
        {
#if CONFIG_POSNET
#if CONFIG_FISCAL

                Engine.defaultprinter.BeginNonFiscal();

                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");

                //this has to be tested, on some printers print big characters
                foreach(string Str in Engine.printer_header.Split('@')){
                    Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, Str + "\r\n");
                }

                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "             COUPON             " + "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");

                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "VALORE EURO             " + String.Format("{0:F2}", (price / (double)100)).ToString().PadLeft(8) + "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.EndNonFiscal();
#else

            Engine.defaultprinter.BeginInsertion(0);
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");

            //this has to be tested, on some printers print big characters
            foreach (string Str in Engine.printer_header.Split('@'))
            {
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, Str + "\r\n");
            }

            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "             COUPON             " + "\r\n");
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");

            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "VALORE EURO             " + String.Format("{0:F2}", (price / (double)100)).ToString().PadLeft(8) + "\r\n");
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
            Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, DateTime.Now.ToString("dd/MM/yy hh:mm").ToUpper().PadRight(20) + "SNF. 0000".PadLeft(8) + "\r\n");
            Engine.defaultprinter.EndInsertion();
#endif
#else
                if(Engine.defaultprinter!=null){

                    Engine.defaultprinter.Write("j@@" + Engine.printer_header + "\"" + "             COUPON             " + "\"" + "@@");

                    //stampa dell//ultima riga
                    Engine.defaultprinter.Write("\"" + "VALORE EURO             " + String.Format("{0:F2}", (price / 100)).ToString().PadLeft(8) + "\"@@");

                    Engine.defaultprinter.Write("J");

                }
#endif
        }


        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //PRINT the document
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public void Print()
        {

            if (Engine.defaultprinter != null)
            {
#if CONFIG_POSNET
#if CONFIG_FISCAL

                    bool cfis = false;
                    double fistot = 0;

                    Transaction tr;
                    for(int i=0;i<=p_doc.Transactions.Count - 1;i++){
                        tr = p_doc.Transactions[i];
                        switch(tr.Type){
                            case(TransactionType.vendita):
                                if(((SellingTransaction)tr).Product.Fiscale){
                                    if(!cfis){
                                        cfis = true;
                                        //BeginFiscalReceipt must be called one time only
                                        Engine.defaultprinter.BeginFiscalReceipt(false);
                                    }
                                    fistot += tr.Amount;
                                    Engine.defaultprinter.PrintRecItem(tr.Text, Convert.ToDecimal(tr.Amount * 100), Convert.ToInt32(tr.Quantity * 1000), 0, Convert.ToDecimal(tr.Value * 100), "");

                                }else{
                                    if(i < p_doc.Transactions.Count - 1){
                                        //if there is an item modifier on this non-fiscal item jump it
                                        if(p_doc.Transactions[i + 1].Type != TransactionType.scontotot &&
                                        p_doc.Transactions[i + 1].Type != TransactionType.scontovaltot &&
                                        p_doc.Transactions[i + 1].Type != TransactionType.vendita){
                                            i += 1;
                                        }
                                    }
                                }
                                break;
                                //////////////////////////////////
                                //item modifiers
                                /////////////////////////////////
                            case TransactionType.scontopercento:
                            case TransactionType.scontovalore:
                                fistot += tr.Amount;
                                Engine.defaultprinter.PrintRecItemAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountDiscount, tr.Text, Convert.ToDecimal(tr.Amount * (-100)), 0);
                                break;
                            case TransactionType.maggpercento:
                            case TransactionType.maggvalore:
                                fistot += tr.Amount;
                                Engine.defaultprinter.PrintRecItemAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountSurcharge, tr.Text, Convert.ToDecimal(tr.Amount * (100)), 0);
                                break;
                                //////////////////////////////////
                                //subtotal modifiers
                                /////////////////////////////////
                            case TransactionType.scontotot:
                                if(cfis){
                                    //recalculate percentage modifier
                                    int scount = ((ModifierTransaction)tr).Percentage * (int)fistot;
                                    fistot -= scount / 100;
                                    //word TOTALE is not accepted so replace E
                                    Engine.defaultprinter.PrintRecSubtotalAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountDiscount, tr.Text.Replace("E", ""), scount);
                                }
                                break;
                            case TransactionType.maggtot:
                                if(cfis){
                                    //recalculate percentage modifier
                                    int scount = ((ModifierTransaction)tr).Percentage * (int)fistot;
                                    fistot += scount / 100;
                                    //word TOTALE is not accepted so replace E
                                    Engine.defaultprinter.PrintRecSubtotalAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountSurcharge, tr.Text.Replace("E", ""), scount);
                                }
                                break;
                            case TransactionType.scontovaltot:
                                //questo crea il problema grosso
                                if(cfis){
                                    if(fistot > -tr.Amount){
                                        fistot -= tr.Amount;
                                        Engine.defaultprinter.PrintRecSubtotalAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountDiscount, tr.Text.Replace("E", ""), Convert.ToDecimal(tr.Amount * (-100)));
                                    }
                                }
                                break;
                            case TransactionType.maggvaltot:
                                if(cfis){
                                    Engine.defaultprinter.PrintRecSubtotalAdjustment(Microsoft.PointOfService.FiscalAdjustment.AmountSurcharge, tr.Text.Replace("E", ""), Convert.ToDecimal(tr.Amount * (100)));
                                }
                                break;
                            }

                    }

                    //se ce almeno un prodotto fiscale devo inviare le stringhe pagamenti
                    //qui non ha senso il loop al contrario perche' le forme di pagamento
                    //non hanno un indice fisso
                    if(cfis){
                        double temp = p_doc.Transactions.Totale * 100;
                        foreach(Payment i in p_doc.Payments){
                            if(i.Paid){
                                Engine.defaultprinter.PrintRecTotal(Convert.ToDecimal(temp), i.Amount, i.Name);
                            }else{
                                Engine.defaultprinter.PrintRecNotPaid(i.Name, i.Amount);
                            }
                            temp -= i.Amount;
                        }

                        Engine.defaultprinter.EndFiscalReceipt(true);
                    }else{
                        //flag for document processing
                        //to not increment counter
                        p_doc.Id = 0;
                    }

                    if(p_doc.Change > 0 && p_doc.Payments[p_doc.Payments.Count - 1].Paid == false && printcoupon){
                        PrintCoupon(p_doc.Change);
                    }

#else
                ////////////////////////////////////////////////////
                //Receipt printing customize for your needs
                //
                ////////////////////////////////////////////////////
                Engine.defaultprinter.BeginInsertion(0);
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");

                //this has to be tested, on some printers print big characters
                foreach (string Str in Engine.printer_header.Split('@'))
                {
                    Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, Str + "\r\n");
                }

                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");

                //header line
                System.Globalization.RegionInfo ri = new System.Globalization.RegionInfo(System.Threading.Thread.CurrentThread.CurrentCulture.LCID);
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, ri.ISOCurrencySymbol.PadLeft(32) + "\r\n");

                foreach (Transaction i in p_doc.Transactions)
                {
                    //riga quantita solo se > 1
                    if (i.Quantity > 1)
                    {
                        Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, i.Quantity.ToString().PadRight(3) + "X " + String.Format("{0:F2}", (i.Value)) + "\r\n");
                    }

                    Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, i.Text.PadRight(24) + String.Format("{0:F2}", (i.Amount)).PadLeft(8) + "\r\n");

                }

                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, resources.GetString("totaltxt").PadRight(24) + String.Format("{0:F2}", (p_doc.Transactions.Totale)).PadLeft(8) + "\r\n");

                foreach (Payment i in p_doc.Payments)
                {
                    Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, i.Name.PadRight(24) + String.Format("{0:F2}", (i.Amount / (double)100)).PadLeft(8) + "\r\n");
                }

                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, resources.GetString("changetxt").PadRight(24) + String.Format("{0:F2}", (p_doc.Change / (double)100)).PadLeft(8) + "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, DateTime.Now.ToString("dd/MM/yy hh:mm").ToUpper().PadRight(20) + ("#" + p_doc.Id).PadLeft(8) + "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.EndInsertion();
#endif
#else
#if CONFIG_FISCAL
                    ////////////////////////////////////////
                    //Custom Syntax modify for your needs
                    ///////////////////////////////////////
                    bool cfis = false;

                    foreach(Transaction i in p_doc.Transactions){

                        if(i.Type == TransactionType.vendita){
                            if(((SellingTransaction)i).Product.Fiscale){
                                cfis = true;

                                Engine.defaultprinter.Write(i.Code);

                            }
                        }else{
                            Engine.defaultprinter.Write(i.Code);
                        }

                    }

                    //se ce almeno un prodotto fiscale devo inviare le stringhe pagamenti
                    //qui non ha senso il loop al contrario perche// le forme di pagamento
                    //non hanno un indice fisso
                    if(cfis){
                        double temp = p_doc.Transactions.Totale * 100;
                        foreach(Payment i in p_doc.Payments){
                            Engine.defaultprinter.Write(i.Code);
                            temp -= i.Amount;
                        }
                    }else{
                        //flag for document processing
                        //to not increment counter
                        p_doc.Id = 0;
                    }

                    if(p_doc.Change > 0 && p_doc.Payments[p_doc.Payments.Count - 1].Paid == false && printcoupon){
                        PrintCouponFormat(p_doc.Change);
                    }
#else
                // CONFIG_FISCAL ^ Custom Syntax over serial ports
#endif

#endif
            }

        }

#if CONFIG_FISCAL
        //////////////////////////////////////////////////////////////
        //PRINT in non fiscal mode
        //only for fiscal countries
        //////////////////////////////////////////////////////////////
        public void PrintNonFiscal(){

            //what's this for??
            //SubTotal()

#if CONFIG_POSNET

            Engine.defaultprinter.BeginNonFiscal();
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");

            //this has to be tested, on some printers print big characters
            foreach(string Str in Engine.printer_header.Split('@')){
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, Str + "\r\n");
            }

            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "            " + resources.GetString("preprinttxt") + "            " + "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");

            //header line 
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, resources.GetString("desctxt").PadRight(11) + "EURO".PadLeft(21) + "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");

            foreach(Transaction i in p_doc.Transactions){

                //riga quantita solo se > 1
                if(i.Quantity > 1){
                    Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, i.Quantity.ToString().PadRight(3) + "X " + String.Format("{0:F2}", (i.Value)) + "\r\n");
                }

                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, i.Text.PadRight(24) + String.Format("{0:F2}", (i.Amount)).PadLeft(8) + "\r\n");

            }

            //riga di spazio prima dell'ultima riga
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "--------------------------------" + "\r\n");

            //Comment for Italy only: POS .NET driver doesn't check for TOTALE string so TOTALE prints right
            //instead of TOTAL. This is not permitted however by laws. Don't know for other fiscal countries.
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, resources.GetString("totaltxt").PadRight(24) + String.Format("{0:F2}", (p_doc.Transactions.Totale)).PadLeft(8) + "\r\n");
            Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
            Engine.defaultprinter.EndNonFiscal();

#else

            ////////////////////////////////////////
            //Custom Syntax modify at your needs
            ///////////////////////////////////////
            if (Engine.defaultprinter != null)
            {

                Engine.defaultprinter.Write("j@@" + Engine.printer_header + "\"" + "            PRECONTO            " + "\"" + "@@");

                //riga euro
                Engine.defaultprinter.Write("\"" + "                            EURO" + "\"@@");

                foreach(Transaction i in doc.Transactions){
                    //riga quantita solo se > 1
                    if(i.Quantity > 1){
                        Engine.defaultprinter.Write("\"" + i.Quantity.ToString().PadRight(3) + "X " + String.Format("{0:F2}", (i.Value)) + "\"@");
                    }

                    Engine.defaultprinter.Write("\"" + i.Text.PadRight(24) + String.Format("{0:F2}", (i.Amount)).PadLeft(8) + "\"@");

                }

                //riga di spazio prima dell//ultima riga
                Engine.defaultprinter.Write("@\"--------------------------------\"@");

                Engine.defaultprinter.Write("\"" + "TOTAL                   " + String.Format("{0:F2}", (doc.Transactions.Totale)).PadLeft(8) + "\"@@");

                Engine.defaultprinter.Write("J");

            }
#endif
        }

#endif

    }

}