﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using POS.Framework;

namespace POS.CashPlugin
{
    public partial class SettingsWindow : Form
    {
        private Rectangle dragBoxFromMouseDown;
        private int rowIndexFromMouseDown;
        private int rowIndexOfItemUnderMouseToDrop;

        public SettingsWindow()
        {
            InitializeComponent();
        }

        private void SettingsWindow_Shown(object sender, EventArgs e)
        {
            Engine.ShowKeyboard();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void btnCanc_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private void DataGridViewPay_MouseMove(object sender, MouseEventArgs e)
        {

            DataGridView dgw = (DataGridView)sender;

            if (e.Button == MouseButtons.Left)
            {

                // If the mouse moves outside the rectangle, start the drag.
                if (dragBoxFromMouseDown != Rectangle.Empty && !dragBoxFromMouseDown.Contains(e.X, e.Y))
                {

                    // Proceed with the drag and drop, passing in the list item.                    
                    DragDropEffects dropEffect = dgw.DoDragDrop(dgw.Rows[rowIndexFromMouseDown], DragDropEffects.Move);
                }
            }
        }

        private void DataGridViewPay_MouseDown(object sender, MouseEventArgs e)
        {

            // Get the index of the item the mouse is below.
            rowIndexFromMouseDown = DataGridViewPay.HitTest(e.X, e.Y).RowIndex;
            if (rowIndexFromMouseDown != -1)
            {

                //// Remember the point where the mouse down occurred. 
                //// The DragSize indicates the size that the mouse can move 
                // before a drag event should be started.                
                Size dragSize = SystemInformation.DragSize;
                // Create a rectangle using the DragSize, with the mouse position being
                // at the center of the rectangle.
                dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);

            }
            else
            {
                // Reset the rectangle if the mouse is not over an item in the ListBox.
                dragBoxFromMouseDown = Rectangle.Empty;
            }
        }

        private void DataGridViewPay_DragOver(object sender, DragEventArgs e)
        {

            e.Effect = DragDropEffects.Move;

        }

        private void DataGridViewPay_SelectionChanged(object sender, EventArgs e)
        {
            if (DataGridViewPay.CurrentCell.Selected == true)
            {
                DataGridViewPay.CurrentCell.Selected = false;
            }

        }

        private void DataGridViewPay_DragDrop(object sender, DragEventArgs e)
        {

            // The mouse locations are relative to the screen, so they must be 
            // converted to client coordinates.
            Point clientPoint = DataGridViewPay.PointToClient(new Point(e.X, e.Y));
            // Get the row index of the item the mouse is below. 
            rowIndexOfItemUnderMouseToDrop = DataGridViewPay.HitTest(clientPoint.X, clientPoint.Y).RowIndex;
            // If the drag operation was a move then remove and insert the row.
            if (e.Effect == DragDropEffects.Move)
            {

                DataGridViewRow rowToMove = (DataGridViewRow)e.Data.GetData(typeof(DataGridViewRow));
                DataGridViewPay.Rows.RemoveAt(rowIndexFromMouseDown);
                DataGridViewPay.Rows.Insert(rowIndexOfItemUnderMouseToDrop, rowToMove);

                foreach (DataGridViewRow row in DataGridViewPay.Rows)
                {
                    Button tempbtn = (Button)row.Cells[0].Tag;
                    tempbtn.Top = (73 * row.Index);
                }

            }
        }

    }
}
