﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using POS.Framework;
using POS.ProductPlugin.Objects;

namespace POS.CashPlugin{
    
    //////////////////////////////////////////
    //Speed Buttons Management Form
    //
    //////////////////////////////////////////
    public partial class SpeedWindow : Form{

        public string speedstring = "";
        public string speedcolor = "";
        public string speedimage = "";
        public string speedtext = "";
        public string speedcommand = "";

        public DataTable arr = new DataTable();

        [DllImport("user32.dll")]
        static extern IntPtr GetFocus();
        [DllImport("user32.dll")]
        static extern bool CreateCaret(IntPtr hWnd, IntPtr hBitmap, int nWidth, int nHeight);
        [DllImport("user32.dll")]
        static extern bool ShowCaret(IntPtr hWnd);

        private void ShowCustomCaret(IntPtr h, int width, int height){
            CreateCaret(h,(IntPtr)0, width, height);
            ShowCaret(h);
        }

        internal SpeedWindow()
        {

            InitializeComponent();

        }

        private void All_Labels_Paint(object sender, EventArgs e){
            ((Label)sender).Left = (this.ClientSize.Width / 2) - (((Label)sender).Width / 2);
        }

        private void Unstring(string value){

            int primo = value.ToString().IndexOf("/");
            int secondo = value.ToString().IndexOf("/", primo + 1);

            string[] temp2 = value.ToString().Substring(0, primo).Split(',');

            speedcolor = temp2[0];

            speedimage = temp2[1];
            
            speedcommand = value.Substring(primo + 1, secondo - primo - 1);

            arr.Rows.Clear();

            string[] arrays = speedcommand.Split(new String[] {"*"}, StringSplitOptions.RemoveEmptyEntries);

            foreach(string pe in arrays){

                string[] param = pe.Split(new String[] {","}, StringSplitOptions.RemoveEmptyEntries);

                arr.Rows.Add(param[2], param[0], param[1], param[3]);

            }

            speedtext = value.Substring(secondo + 1).ToUpper();

            txtSpeedName.Text = speedtext;

        }

        private void SpeedWindow_Shown(object sender, EventArgs e)
        {
            
            DataTable ds = Product.GetCombo();
            ds.DefaultView.Sort = "prodotto ASC";

            ColProdotto.DataSource = ds;
            ColProdotto.DisplayMember = "prodotto";
            ColProdotto.ValueMember = "id";

            DataTable typetable = new DataTable();
            typetable.Columns.AddRange(new DataColumn[] { new DataColumn("id", Type.GetType("System.Int32")), new DataColumn("Type", Type.GetType("System.String")) });

            typetable.Rows.Add(new Object[] { 1, resources.GetString("typeven") });
            typetable.Rows.Add(new Object[] { 2, resources.GetString("typesceuro") + System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol });
            typetable.Rows.Add(new Object[] { 3, resources.GetString("typescperc") });
            typetable.Rows.Add(new Object[] { 4, resources.GetString("typesctotperc") });
            typetable.Rows.Add(new Object[] { 5, resources.GetString("typesctoteuro") });
            typetable.Rows.Add(new Object[] { 6, resources.GetString("typemaeuro") + System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol });
            typetable.Rows.Add(new Object[] { 7, resources.GetString("typemaperc") });

            ColTipo.DataSource = typetable;
            ColTipo.DisplayMember = "Type";
            ColTipo.ValueMember = "id";

            Unstring(speedstring);

            DataGridSpeed.DataSource = arr;

            //populate icons datagridview
            DataGridViewImageColumn colicon = new DataGridViewImageColumn();

            colicon.Width = 70;
            dgwIcons.Columns.Add(colicon);
            dgwIcons.Columns.Add((DataGridViewColumn)colicon.Clone());
            dgwIcons.Columns.Add((DataGridViewColumn)colicon.Clone());
            dgwIcons.Columns.Add((DataGridViewColumn)colicon.Clone());

            for (int i = 0; i <= 49; i++)
            {
                dgwIcons.Rows.Add(4 * i, (4 * i) + 1, (4 * i) + 2, (4 * i) + 3);
            }

            //populate colors datagridview
            DataGridViewButtonColumn col = new DataGridViewButtonColumn();

            col.Width = 70;
            dgwColors.Columns.Add(col);
            dgwColors.Columns.Add((DataGridViewColumn)col.Clone());
            dgwColors.Columns.Add((DataGridViewColumn)col.Clone());
            dgwColors.Columns.Add((DataGridViewColumn)col.Clone());

            dgwColors.Rows.Add("141", "79", "123", "140");
            dgwColors.Rows.Add("127", "105", "37", "107");
            dgwColors.Rows.Add("166", "155", "48", "164");


            Engine.ShowKeyboard();

            TabSpeed.SelectedIndex = 0;

            btnProdNext.Text = resources.GetString("btnProdNext.Text");

        }

        private void txtSpeedName_GotFocus(object sender, EventArgs e){
            ShowCustomCaret(txtSpeedName.Handle, 4, 29);
        }

        private void txtSpeedName_TextChanged(object sender, EventArgs e){
            if(txtSpeedName.Text.Length > 22){
                txtSpeedName.Text = txtSpeedName.Text.Substring(0, 22);
                txtSpeedName.SelectionStart = 22;
            }
        }

        private void btnProdNext_Click(object sender, EventArgs e){

            //check transactions inserted
            if(TabSpeed.SelectedIndex == 0){
                if(DataGridSpeed.Rows[0].IsNewRow || Convert.ToInt32(DataGridSpeed.Rows[0].Cells[1].Value)!=1 || DataGridSpeed.Rows[0].Cells[2].Value==DBNull.Value){
                    return;
                }else{
                    if(speedcommand.Equals("")){
                        txtSpeedName.Text = DataGridSpeed.Rows[0].Cells[2].FormattedValue.ToString().Substring(0, (DataGridSpeed.Rows[0].Cells[2].FormattedValue.ToString().IndexOf(System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol) - 2));
                    }
                }

            }else if(TabSpeed.SelectedIndex == 1){
                //show current color
                btnColor.Text = txtSpeedName.Text;
                if (!string.IsNullOrEmpty(speedcolor)){

                    btnColor.BackColor = Color.FromKnownColor((KnownColor)Convert.ToInt32(speedcolor));
                }else{
                    //handle it anyway
                    btnColor.BackColor = Color.White;

                }
                //show current icon
                if (!string.IsNullOrEmpty(speedimage))
                {
                    btnColor.Image = Image.FromFile(Engine.dirpath + "images\\" + speedimage + ".gif");
                }

             }else if(TabSpeed.SelectedIndex == 2){
                
                //save speed
                speedstring = "";

                foreach(DataRow rr in arr.Rows){
                    if(Convert.IsDBNull(rr[2])){
                        rr[2] = 0;
                    }
                    speedstring += "*" + rr[1] + "," + rr[2] + "," + rr[0] + "," + rr[3] + "*";
                }

                Engine.HideKeyboard();
                this.Hide();
                this.DialogResult = DialogResult.OK;
                return;

            }

            TabSpeed.SelectedIndex++;
        }

        private void btnProdBack_Click(object sender, EventArgs e){
            if(TabSpeed.SelectedIndex > 0) TabSpeed.SelectedIndex--;
        }

        private void btnProdCancel_Click(object sender, EventArgs e){
            Engine.HideKeyboard();
            this.Hide();
        }

        private void TabSpeed_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch(TabSpeed.SelectedIndex){
                case 0:
                    btnProdNext.BackColor = SystemColors.Control;
                    btnProdNext.Text = resources.GetString("btnProdNext.Text");
                    break;
                case 1:
                    btnProdNext.BackColor = SystemColors.Control;
                    txtSpeedName.Focus();
                    txtSpeedName.SelectionLength = 0;
                    btnProdNext.Text = resources.GetString("btnProdNext.Text");
                    break;
                case 2:
                    speedtext = txtSpeedName.Text;
                    btnProdNext.BackColor = Color.LightGreen;
                    btnProdNext.Text = resources.GetString("btnProdNextSave.Text");
                    break;
            }
        }

        #region Datagrid Stuff

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Datagrid Stuff
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private void DataGridSpeed_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e){
            if(e.ColumnIndex == 3){
                if(Convert.ToInt32(DataGridSpeed.Rows[e.RowIndex].Cells[1].Value) == 3 || Convert.ToInt32(DataGridSpeed.Rows[e.RowIndex].Cells[1].Value) == 4 || Convert.ToInt32(DataGridSpeed.Rows[e.RowIndex].Cells[1].Value) == 7){
                    if(DataGridSpeed.Rows[e.RowIndex].Cells[3].IsInEditMode){
                        // what is this for ??
                        e.Value = e.Value.ToString();
                    }else{
                        e.Value = e.Value.ToString() + "%";
                    }
                    e.FormattingApplied = true;
                }else{
                    if(DataGridSpeed.Rows[e.RowIndex].Cells[3].IsInEditMode){
                        e.Value = e.Value.ToString();
                        e.FormattingApplied = true;
                    }else{
                        if(Convert.ToDouble(e.Value) == 0){
                            e.Value = "AUTO";
                            e.FormattingApplied = true;
                        }else{
                            e.Value = Convert.ToDouble(e.Value) / (double)100;
                            e.FormattingApplied = false;
                        }
                    }
                }
            }else if(e.ColumnIndex == 1 && e.RowIndex >= 0){
                if(Convert.ToInt32(DataGridSpeed.Rows[e.RowIndex].Cells[1].Value) > 1){
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].ReadOnly = true;
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].Style.BackColor = Color.Gray;
                }else{
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].Style.BackColor = Color.White;
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].ReadOnly = false;
                }
            }
        }

        private void DataGridSpeed_CellValueChanged(object sender, DataGridViewCellEventArgs e){
            if(e.ColumnIndex == 1 && e.RowIndex >= 0){
                // set combobox according to transaction type
                DataGridSpeed.Rows[e.RowIndex].Cells[3].Value = 0;
                if(Convert.ToInt32(DataGridSpeed.Rows[e.RowIndex].Cells[1].Value) > 1){
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].ReadOnly = true;
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].Style.BackColor = Color.Gray;
                }else{
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].Style.BackColor = Color.White;
                    DataGridSpeed.Rows[e.RowIndex].Cells[2].ReadOnly = false;
                }
            }else if(e.ColumnIndex == 2 && e.RowIndex >= 0){
                // reset price to 0/auto if product changed
                DataGridSpeed.Rows[e.RowIndex].Cells[3].Value = 0;
            }
        }

        // handle delete-row button click
        private void DataGridSpeed_Click(object sender, EventArgs e){
            if(DataGridSpeed.CurrentCell.ColumnIndex == 4){
                if(!DataGridSpeed.CurrentRow.IsNewRow){
                    DataGridSpeed.Rows.RemoveAt(DataGridSpeed.CurrentCell.RowIndex);
                }
            }
        }

        private void DataGridSpeed_CurrentCellDirtyStateChanged(object sender, EventArgs e){

            if(DataGridSpeed.IsCurrentCellDirty){
                DataGridSpeed.CommitEdit(DataGridViewDataErrorContexts.Commit);
                DataGridSpeed.Refresh();
            }

        }

        private void DataGridSpeed_DataError(object sender, DataGridViewDataErrorEventArgs e){

        }

        private void DataGridSpeed_DefaultValuesNeeded(object sender, DataGridViewRowEventArgs e){
            e.Row.Cells[0].Value = 1;
            e.Row.Cells[1].Value = 1;
            e.Row.Cells[3].Value = 0;
            e.Row.Cells[4].Value = "X";
        }

        #endregion

        #region Scroll
        /////////////////////////////////////////////
        //touch scroll stuff
        /////////////////////////////////////////////
        private bool switched;
        private Point mouse_down, temp;
        private Rectangle dragBoxFromMouseDown;

        private void DataGridSpeed_UserAddedRow(object sender, EventArgs e){
            pnlSpeed.AutoScrollPosition = new Point(0, DataGridSpeed.Size.Height - pnlSpeed.Height - 16);
        }

        private void DataGridSpeed_MouseDown(object sender, MouseEventArgs e){
            mouse_down = new Point(e.X, e.Y);
            switched = false;
            temp = pnlSpeed.AutoScrollPosition;
            Size dragSize = SystemInformation.DragSize;
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void DataGridSpeed_MouseMove(object sender, MouseEventArgs e){

            if(e.Button == MouseButtons.Left){
                int dX = (mouse_down.X - e.X);
                int dY = (mouse_down.Y - e.Y);
                pnlSpeed.AutoScrollPosition = new Point((dX - pnlSpeed.AutoScrollPosition.X), (dY - pnlSpeed.AutoScrollPosition.Y));
                if(Math.Abs(temp.Y - pnlSpeed.AutoScrollPosition.Y) > dragBoxFromMouseDown.Height || Math.Abs(mouse_down.Y - e.Y) > dragBoxFromMouseDown.Height){
                    switched = true;
                }
            }

        }

        private void DataGridSpeed_MouseUp(object sender, MouseEventArgs e){
            //delete row
            if(!switched) DataGridSpeed.BeginEdit(false);
        }

        #endregion

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Datagrid Button Color
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private void dgwColors_CellClick(object sender, DataGridViewCellEventArgs e)
        {

            speedcolor = dgwColors.CurrentCell.Tag.ToString();

            btnColor.BackColor = Color.FromKnownColor((KnownColor)Convert.ToInt32(speedcolor));

        }

        private void dgwColors_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e){

            dgwColors[e.ColumnIndex, e.RowIndex].Tag = e.Value;
            e.CellStyle.BackColor = Color.FromKnownColor((KnownColor)Convert.ToInt32(e.Value));
            e.CellStyle.SelectionBackColor = Color.FromKnownColor((KnownColor)Convert.ToInt32(e.Value));
            e.Value = "";
            e.FormattingApplied = true;

        }


        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Datagrid Button Icon
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private void dgwIconsCellClick()
        {

            speedimage = dgwIcons.CurrentCell.Tag.ToString();

            string path = Engine.dirpath + "images\\" + speedimage + ".gif";

            if(!(System.IO.File.Exists(path))) path = Engine.dirpath + "images\\0.gif";

            btnColor.Image = Image.FromFile(path);

        }

        private void dgwIcons_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {

            string path = Engine.dirpath + "images\\" + e.Value.ToString() + ".gif";

            if(!(System.IO.File.Exists(path))) path = Engine.dirpath + "images\\0.gif";

            dgwIcons[e.ColumnIndex, e.RowIndex].Tag = e.Value;
            e.Value = Image.FromFile(path);
            e.FormattingApplied = true;

        }

        private void dgwIcons_MouseDown(object sender, MouseEventArgs e)
        {
            mouse_down = new Point(e.X, e.Y);
            switched = false;
            temp = pnldgwIcons.AutoScrollPosition;
            Size dragSize = SystemInformation.DragSize;
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void dgwIcons_MouseMove(object sender, MouseEventArgs e)
        {

            if (e.Button == MouseButtons.Left)
            {
                int dX = (mouse_down.X - e.X);
                int dY = (mouse_down.Y - e.Y);
                pnldgwIcons.AutoScrollPosition = new Point((dX - pnldgwIcons.AutoScrollPosition.X), (dY - pnldgwIcons.AutoScrollPosition.Y));
                if (Math.Abs(temp.Y - pnldgwIcons.AutoScrollPosition.Y) > dragBoxFromMouseDown.Height || Math.Abs(mouse_down.Y - e.Y) > dragBoxFromMouseDown.Height)
                {
                    switched = true;
                }
            }

        }

        private void dgwIcons_MouseUp(object sender, MouseEventArgs e)
        {
            if (!switched) dgwIconsCellClick();
        }

    }
}