﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General public License for more details.
 * 
 * You should have received a copy of the GNU General public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using POS.FiscalPlugin.Objects;
using POS.Framework;
using POS.Interfaces;


namespace POS.FiscalPlugin{

    public class FiscalPlugin : IPlugin{

        internal class PanelNoScrollOnFocus : Panel
        {

            protected override Point ScrollToControl(Control activeControl)
            {
                return base.DisplayRectangle.Location;
            }

        }

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FiscalPlugin));
        internal Button BtnSearch;
        internal Button BtnResetTot;
        internal Button BtnPrint;
        internal Button BtnReset;
    #if CONFIG_FISCAL
        internal GroupBox GrpPrinter;
        internal Button BtnXRep;
        internal Button BtnZRep;
    #endif
    #if CONFIG_NETWORK
        internal GroupBox GrpJournalSource;
        internal RadioButton RdbtnLocal;
        internal RadioButton RdbtnRemote;
    #endif
        internal GroupBox GrpRepType;
        internal RadioButton RdbtnPayments;
        internal RadioButton RdbtnPid;
        internal RadioButton RdbtnVAT;
        internal RadioButton RdbtnDep;
        internal RadioButton RdbtnXRep;
        internal Panel pnlFake;
        internal PanelNoScrollOnFocus pnlJournal;
        internal DataGridView dgdJournal;


        private Journal j = new Journal();

        // IPlugin.Install implementation
        public void Install(){
    
            if(Engine.HasAccess(resources.GetString("Journal_Install"))){
    
            }

        }

        // IPlugin.UnInstall implementation
        public void Uninstall(){

            if(Engine.HasAccess(resources.GetString("Journal_Uninstall"))){
    
            }

        }

        // IPlugin.PosData implementation
        public void PosData(object dev, string data){
        

        }

        // IPlugin.SetProperties implementation
        public void SetProperties(){

            if(Engine.HasAccess(resources.GetString("Journal_Settings"))){
    
            }

        }

        // IPlugin.TabEnter implementation
        public void TabEnter(){


        }

        // IPlugin.Message implementation
        public int Message(int func, object data){

            if(func == 1){
                return j.ProcessDocument(data);
            }else if(func == 2){
                return j.GetReceiptNumber();
            }else{
                return -1;
            }

        }

        // IPlugin.Initialize implementation
        public int Initialize(TabPage container){

            if(Engine.HasAccess(resources.GetString("Journal_Load"))){

                InitializeUI(container);

                return 0;

            }
            
            return 1;

        }

        // TODO: db tables creation 
        public void InitializeDB(){

        }

        private void InitializeUI(TabPage container){

            this.BtnSearch = new Button();
            this.BtnResetTot = new Button();
            this.BtnPrint = new Button();
            this.BtnReset = new Button();
    #if CONFIG_FISCAL
            this.GrpPrinter = new GroupBox();
            this.BtnZRep = new Button();
            this.BtnXRep = new Button();
    #endif

    #if CONFIG_NETWORK
            this.GrpJournalSource = new GroupBox();
            this.RdbtnLocal = new RadioButton();
            this.RdbtnRemote = new RadioButton();
    #endif
            this.GrpRepType = new GroupBox();
            this.RdbtnPayments = new RadioButton();
            this.RdbtnPid = new RadioButton();
            this.RdbtnVAT = new RadioButton();
            this.RdbtnDep = new RadioButton();
            this.RdbtnXRep = new RadioButton();
            this.pnlFake = new Panel();
            this.pnlJournal = new PanelNoScrollOnFocus();
            this.dgdJournal = new DataGridView();
            //
            //TabFiscal
            //
            container.Controls.Add(this.BtnResetTot);
            container.Controls.Add(this.BtnPrint);
            container.Controls.Add(this.BtnReset);
            container.Controls.Add(this.pnlFake);
    #if CONFIG_FISCAL
            container.Controls.Add(this.GrpPrinter);
    #endif
    #if CONFIG_NETWORK
            container.Controls.Add(this.GrpJournalSource);
            container.Controls.Add(this.RdbtnLocal);
            container.Controls.Add(this.RdbtnRemote);
    #endif
            container.Controls.Add(this.GrpRepType);
            container.Controls.Add(this.BtnPrint);
            container.Controls.Add(this.BtnSearch);

            container.Text = resources.GetString("container.Text");


            //
            //BtnSearch
            //
            this.BtnSearch.BackColor =  Color.LightGreen;
            this.BtnSearch.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
    #if CONFIG_NETWORK
            this.BtnSearch.Location = new  Point(736, 277);
    #else
            this.BtnSearch.Location = new  Point(736, 207);
    #endif
            this.BtnSearch.Name = "BtnSearch";
            this.BtnSearch.Size = new Size(120, 54);
            this.BtnSearch.TabIndex = 0;
            this.BtnSearch.Text = resources.GetString("BtnSearch.Text");
            this.BtnSearch.UseVisualStyleBackColor = false;
            this.BtnSearch.Click += new System.EventHandler(this.BtnSearch_Click);
            //
            //BtnPrint
            //
            this.BtnPrint.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
    #if CONFIG_NETWORK
            this.BtnPrint.Location = new Point(885, 277);
    #else
            this.BtnPrint.Location = new Point(885, 207);
    #endif
            this.BtnPrint.Name = "BtnPrint";
            this.BtnPrint.Size = new Size(120, 54);
            this.BtnPrint.TabIndex = 7;
            this.BtnPrint.Text = resources.GetString("BtnPrint.Text");
            this.BtnPrint.UseVisualStyleBackColor = false;
            this.BtnPrint.Click += new System.EventHandler(this.BtnPrint_Click);
            //
            //BtnReset
            //
            this.BtnReset.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
    #if CONFIG_NETWORK
            this.BtnReset.Location = new Point(736, 357);
    #else
            this.BtnReset.Location = new Point(736, 287);
    #endif
            this.BtnReset.Name = "BtnReset";
            this.BtnReset.Size = new  Size(120, 54);
            this.BtnReset.TabIndex = 7;
            this.BtnReset.Text = resources.GetString("BtnReset.Text");
            this.BtnReset.UseVisualStyleBackColor = false;
            this.BtnReset.Click += new System.EventHandler(this.BtnReset_Click);
            //
            //BtnResetTot
            //
            this.BtnResetTot.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
    #if CONFIG_NETWORK
            this.BtnResetTot.Location = new Point(885, 357);
    #else
            this.BtnResetTot.Location = new Point(885, 287);
    #endif
            this.BtnResetTot.Name = "BtnResetTot";
            this.BtnResetTot.Size = new  Size(120, 54);
            this.BtnResetTot.TabIndex = 8;
            this.BtnResetTot.Text = resources.GetString("BtnResetTot.Text");
            this.BtnResetTot.UseVisualStyleBackColor = false;
            this.BtnResetTot.Click += new System.EventHandler(this.BtnResetTot_Click);
            //
            //GrpRepType
            //
            this.GrpRepType.AutoSize = false;
            this.GrpRepType.Controls.Add(this.RdbtnPayments);
            this.GrpRepType.Controls.Add(this.RdbtnPid);
            this.GrpRepType.Controls.Add(this.RdbtnVAT);
            this.GrpRepType.Controls.Add(this.RdbtnDep);
            this.GrpRepType.Controls.Add(this.RdbtnXRep);
            this.GrpRepType.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
    #if CONFIG_NETWORK
            this.GrpRepType.Location = new Point(736, 80);
    #else
            this.GrpRepType.Location = new Point(736, 10);
    #endif
            this.GrpRepType.Name = "GrpRepType";
            this.GrpRepType.Size = new Size(270, 180);
            this.GrpRepType.Text = resources.GetString("GrpRepType.Text");
            
            //
            //RdbtnPayments
            //
            this.RdbtnPayments.AutoSize = false;
            this.RdbtnPayments.BackColor = Color.AliceBlue;
            this.RdbtnPayments.Checked = true;
            this.RdbtnPayments.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnPayments.Image = (Image)resources.GetObject("report");
            this.RdbtnPayments.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnPayments.Location = new Point(10, 25);
            this.RdbtnPayments.Appearance = Appearance.Button;
            this.RdbtnPayments.Name = "RdbtnPayments";
            this.RdbtnPayments.Size = new Size(121, 30);
            this.RdbtnPayments.Text = resources.GetString("RdbtnPayments.Text");
            this.RdbtnPayments.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnPayments.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnPayments.UseVisualStyleBackColor = false;
            //
            //RdbtnPid
            //
            this.RdbtnPid.AutoSize = false;
            this.RdbtnPid.BackColor = Color.AliceBlue;
            this.RdbtnPid.Appearance = Appearance.Button;
            this.RdbtnPid.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnPid.Image = (Image)resources.GetObject("report");
            this.RdbtnPid.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnPid.Location = new Point(10, 61);
            this.RdbtnPid.Name = "RdbtnPid";
            this.RdbtnPid.Size = new Size(121, 30);
            this.RdbtnPid.TabIndex = 4;
            this.RdbtnPid.Text = resources.GetString("RdbtnPid.Text");
            this.RdbtnPid.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnPid.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnPid.UseVisualStyleBackColor = false;
            //
            //RdbtnVAT
            //
            this.RdbtnVAT.AutoSize = false;
            this.RdbtnVAT.Appearance = Appearance.Button;
            this.RdbtnVAT.BackColor = Color.AliceBlue;
            this.RdbtnVAT.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnVAT.Image = (Image)resources.GetObject("report");
            this.RdbtnVAT.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnVAT.Location = new Point(10, 97);
            this.RdbtnVAT.Name = "RdbtnVAT";
            this.RdbtnVAT.Size = new Size(121, 30);
            this.RdbtnVAT.TabIndex = 5;
            this.RdbtnVAT.Text = resources.GetString("RdbtnVAT.Text");
            this.RdbtnVAT.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnVAT.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnVAT.UseVisualStyleBackColor = false;
            //
            //RdbtnDep
            //
            this.RdbtnDep.AutoSize = false;
            this.RdbtnDep.Appearance = Appearance.Button;
            this.RdbtnDep.BackColor = Color.AliceBlue;
            this.RdbtnDep.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnDep.Image = (Image)resources.GetObject("report");
            this.RdbtnDep.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnDep.Location = new Point(10, 133);
            this.RdbtnDep.Name = "RdbtnDep";
            this.RdbtnDep.Size = new Size(121, 30);
            this.RdbtnDep.TabIndex = 6;
            this.RdbtnDep.Text = resources.GetString("RdbtnDep.Text");
            this.RdbtnDep.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnDep.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnDep.UseVisualStyleBackColor = false;
            //
            //RdbtnXRep
            //
            this.RdbtnXRep.AutoSize = false;
            this.RdbtnXRep.Appearance = Appearance.Button;
            this.RdbtnXRep.BackColor = Color.AliceBlue;
            this.RdbtnXRep.Font = new  Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnXRep.Image = (Image)resources.GetObject("report");
            this.RdbtnXRep.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnXRep.Location = new Point(141, 25);
            this.RdbtnXRep.Name = "RdbtnXRep";
            this.RdbtnXRep.Size = new Size(121, 30);
            this.RdbtnXRep.TabIndex = 6;
            this.RdbtnXRep.Text = resources.GetString("RdbtnXRep.Text");
            this.RdbtnXRep.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnXRep.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnXRep.UseVisualStyleBackColor = false;

    #if CONFIG_FISCAL
            //
            //GrpPrinter
            //
            this.GrpPrinter.AutoSize = false;
            this.GrpPrinter.Controls.Add(this.BtnXRep);
            this.GrpPrinter.Controls.Add(this.BtnZRep);
            this.GrpPrinter.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.GrpPrinter.Location = new Point(736, 432);
            this.GrpPrinter.Name = "GrpPrinter";
            this.GrpPrinter.Size = new Size(270, 250);
            this.GrpPrinter.Text = resources.GetString("GrpPrinter.Text");
            //
            //BtnXRep
            //
            this.BtnXRep.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.BtnXRep.Location = new Point(10, 25);
            this.BtnXRep.Name = "BtnXRep";
            this.BtnXRep.Size = new Size(120, 54);
            this.BtnXRep.TabIndex = 7;
            this.BtnXRep.Text = resources.GetString("BtnXRep.Text");
            this.BtnXRep.UseVisualStyleBackColor = false;
            this.BtnXRep.Click += new System.EventHandler(this.BtnXRep_Click);
            //
            //BtnZRep
            //
            this.BtnZRep.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.BtnZRep.Location = new Point(140, 25);
            this.BtnZRep.Name = "BtnZRep";
            this.BtnZRep.Size = new Size(120, 54);
            this.BtnZRep.TabIndex = 7;
            this.BtnZRep.Text = resources.GetString("BtnZRep.Text");
            this.BtnZRep.UseVisualStyleBackColor = false;
            this.BtnZRep.Click += new System.EventHandler(this.BtnZRep_Click);
    #endif


    #if CONFIG_NETWORK
            //
            //GrpJournalSource
            //
            this.GrpJournalSource.AutoSize = false;
            this.GrpJournalSource.Controls.Add(this.RdbtnLocal);
            this.GrpJournalSource.Controls.Add(this.RdbtnRemote);
            this.GrpJournalSource.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.GrpJournalSource.Location = new Point(736, 10);
            this.GrpJournalSource.Name = "GrpJournalSource";
            this.GrpJournalSource.Size = new Size(270, 70);
            this.GrpJournalSource.Text = resources.GetString("GrpJournalSource.Text");
            //
            //RdbtnLocal
            //
            this.RdbtnLocal.AutoSize = false;
            this.RdbtnLocal.Appearance = Appearance.Button;
            this.RdbtnLocal.BackColor = Color.LightYellow;
            this.RdbtnLocal.Checked = true;
            this.RdbtnLocal.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnLocal.Image = (Image)resources.GetObject("database");
            this.RdbtnLocal.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnLocal.Location = new Point(10, 25);
            this.RdbtnLocal.Name = "RdbtnLocal";
            this.RdbtnLocal.Size = new Size(121, 30);
            this.RdbtnLocal.Text = resources.GetString("RdbtnLocal.Text");
            this.RdbtnLocal.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnLocal.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnLocal.UseVisualStyleBackColor = false;
            //
            //RdbtnRemote
            //
            this.RdbtnRemote.AutoSize = false;
            this.RdbtnRemote.Appearance = Appearance.Button;
            this.RdbtnRemote.BackColor = Color.LightYellow;
            this.RdbtnRemote.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.RdbtnRemote.Image = (Image)resources.GetObject("database_link");
            this.RdbtnRemote.ImageAlign = ContentAlignment.TopLeft;
            this.RdbtnRemote.Location = new Point(141, 25);
            this.RdbtnRemote.Name = "RdbtnRemote";
            this.RdbtnRemote.Size = new Size(121, 30);
            this.RdbtnRemote.Text = resources.GetString("RdbtnRemote.Text");
            this.RdbtnRemote.TextAlign = ContentAlignment.MiddleLeft;
            this.RdbtnRemote.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.RdbtnRemote.UseVisualStyleBackColor = false;
#endif
            //
            //pnlFake
            //
            this.pnlFake.AutoScroll = false;
            this.pnlFake.AutoScrollMargin = new Size(0, 0);
            this.pnlFake.BorderStyle = BorderStyle.FixedSingle;
            this.pnlFake.Controls.Add(this.pnlJournal);
            this.pnlFake.Name = "pnlFake";
            this.pnlFake.Size = new Size(702, 663);
            this.pnlFake.Location = new Point(21, 18);
            //
            //pnlJournal
            //
            this.pnlJournal.AutoScroll = true;
            this.pnlJournal.AutoScrollMargin = new Size(0, 0);
            this.pnlJournal.BorderStyle = BorderStyle.None;
            this.pnlJournal.Controls.Add(dgdJournal);
            this.pnlJournal.Name = "pnlJournal";
            this.pnlJournal.Size = new Size(718, 661);
            this.pnlJournal.Location = new Point(0, 0);
            //
            //dgdJournal
            //
            this.dgdJournal.AllowUserToAddRows = false;
            this.dgdJournal.AllowUserToDeleteRows = false;
            this.dgdJournal.AllowUserToOrderColumns = false;
            this.dgdJournal.AllowUserToResizeRows = false;
            this.dgdJournal.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;
            this.dgdJournal.BorderStyle = BorderStyle.None;
            this.dgdJournal.ColumnHeadersVisible = true;
            this.dgdJournal.ColumnHeadersHeight = 40;
            this.dgdJournal.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            //////////////////////////////////////////////////////
            //HeadersCellStyle
            /////////////////////////////////////////////////////
            this.dgdJournal.ColumnHeadersDefaultCellStyle.Padding = new Padding(10);
            this.dgdJournal.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
            this.dgdJournal.ColumnHeadersDefaultCellStyle.BackColor = SystemColors.Control;
            this.dgdJournal.ColumnHeadersDefaultCellStyle.Font = new Font("Microsoft Sans Serif", 12.0F, FontStyle.Bold, GraphicsUnit.Point, ((Byte)(0)));
            this.dgdJournal.ColumnHeadersDefaultCellStyle.ForeColor = SystemColors.ControlText;
            this.dgdJournal.ColumnHeadersDefaultCellStyle.WrapMode = DataGridViewTriState.False;
            //////////////////////////////////////////////////////
            //DefaultCellStyle
            /////////////////////////////////////////////////////
            this.dgdJournal.DefaultCellStyle.Padding = new System.Windows.Forms.Padding(10);
            this.dgdJournal.DefaultCellStyle.Alignment = System.Windows.Forms.DataGridViewContentAlignment.MiddleLeft;
            this.dgdJournal.DefaultCellStyle.BackColor = SystemColors.Window;
            this.dgdJournal.DefaultCellStyle.Font = new Font("Microsoft Sans Serif", 12.0F, FontStyle.Bold, GraphicsUnit.Point, ((Byte)(0)));
            this.dgdJournal.DefaultCellStyle.ForeColor = SystemColors.ControlText;
            this.dgdJournal.DefaultCellStyle.SelectionBackColor = Color.White;
            this.dgdJournal.DefaultCellStyle.SelectionForeColor = Color.Black;
            this.dgdJournal.DefaultCellStyle.WrapMode = DataGridViewTriState.False;
            //////////////////////////////////////////////////////
            //EndDefaultCellStyle
            /////////////////////////////////////////////////////
            this.dgdJournal.RowTemplate.Height = 40;
            this.dgdJournal.Location = new Point(0, 0);
            this.dgdJournal.Name = "dgdJournal";
            this.dgdJournal.ReadOnly = true;
            this.dgdJournal.RowHeadersVisible = false;
            this.dgdJournal.ScrollBars = ScrollBars.None;
            this.dgdJournal.Size = new Size(700, 1620);
            this.dgdJournal.TabStop = false;
            this.dgdJournal.ColumnAdded += new DataGridViewColumnEventHandler(this.dgdJournal_ColumnAdded);
            this.dgdJournal.MouseDown += new MouseEventHandler(this.dgdJournal_MouseDown);
            this.dgdJournal.MouseMove += new MouseEventHandler(this.dgdJournal_MouseMove);
        }


        private void BtnSearch_Click(object sender, EventArgs e){

            dgdJournal.DataSource = null;

        #if CONFIG_NETWORK
            if(RdbtnPayments.Checked && Engine.HasAccess(resources.GetString("Journal_Payments")) && RdbtnLocal.Checked){
                dgdJournal.DataSource = j.GetPayments();
                dgdJournal.Columns[1].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
#else
            if (RdbtnPayments.Checked && Engine.HasAccess(resources.GetString("Journal_Payments")))
            {
                dgdJournal.DataSource = j.GetPayments();
                dgdJournal.Columns[1].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
        #endif

        #if CONFIG_NETWORK
            }else if(RdbtnPayments.Checked && Engine.HasAccess(resources.GetString("Journal_Payments")) && RdbtnRemote.Checked){

                dgdJournal.DataSource = j.GetNetTotals();
                dgdJournal.Columns[1].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
#endif

#if CONFIG_NETWORK
            }else if(RdbtnPid.Checked && Engine.HasAccess(resources.GetString("Journal_Products")) && RdbtnLocal.Checked){
                dgdJournal.DataSource = j.GetPidTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
#else
            }
            else if(RdbtnPid.Checked && Engine.HasAccess(resources.GetString("Journal_Products"))){
                dgdJournal.DataSource = j.GetPidTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
        #endif


        #if CONFIG_NETWORK
            }else if(RdbtnVAT.Checked  && Engine.HasAccess(resources.GetString("Journal_VAT")) && RdbtnLocal.Checked){
                dgdJournal.DataSource = j.GetAliTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
#else
            }
            else if(RdbtnVAT.Checked && Engine.HasAccess(resources.GetString("Journal_VAT"))){
                dgdJournal.DataSource = j.GetAliTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
        #endif 


        #if CONFIG_NETWORK
            }else if(RdbtnDep.Checked && Engine.HasAccess(resources.GetString("Journal_Departments")) && RdbtnLocal.Checked){
                dgdJournal.DataSource = j.GetRepTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
#else
            }
            else if(RdbtnDep.Checked && Engine.HasAccess(resources.GetString("Journal_Departments"))){
                dgdJournal.DataSource = j.GetRepTotals();
                dgdJournal.Columns[2].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[3].DefaultCellStyle.Format = "c";
                dgdJournal.Columns[4].DefaultCellStyle.Format = "c";
                if(dgdJournal.Rows.Count > 1){
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.BackColor = Color.Salmon;
                    dgdJournal.Rows[dgdJournal.Rows.Count - 1].DefaultCellStyle.SelectionBackColor = Color.Salmon;
                }
        #endif 

        #if CONFIG_NETWORK
            }else if(RdbtnXRep.Checked && Engine.HasAccess(resources.GetString("Journal_XReport")) && RdbtnLocal.Checked){
                dgdJournal.DataSource = j.GetTotals();
#else
            }
            else if(RdbtnXRep.Checked && Engine.HasAccess(resources.GetString("Journal_XReport"))){
                dgdJournal.DataSource = j.GetTotals();
        #endif

            }

        }

        private void BtnReset_Click(object sender, EventArgs e){
        
            if(Engine.HasAccess(resources.GetString("Journal_Reset"))){

                PopBox pop = new PopBox();

                if(pop.ShowDialog(resources.GetString("msgresettxt"), new Point(370, 300)) == DialogResult.Yes){
                    j.ResetTotals(false);
                    dgdJournal.DataSource = null;
                }

            }

        }

        private void BtnResetTot_Click(object sender, EventArgs e){
        
            if(Engine.HasAccess(resources.GetString("Journal_Reset"))){

                PopBox pop = new PopBox();

                if(pop.ShowDialog(resources.GetString("msgresettottxt"), new Point(370, 300)) == DialogResult.Yes){
                    j.ResetTotals(true);
                    dgdJournal.DataSource = null;
                }

            }

        }

    #if CONFIG_FISCAL

        private void BtnXRep_Click(object sender, EventArgs e){
        
            if(Engine.HasAccess(resources.GetString("FiscalPrinter_XReport"))){

                PopBox pop = new PopBox();

                if(pop.ShowDialog(resources.GetString("msgletturatxt"), new Point(370, 300)) == DialogResult.Yes){

                    if(Engine.defaultprinter!=null){
                        #if CONFIG_POSNET
                        if(Engine.defaultprinter.CapXReport){
                            Engine.defaultprinter.PrintXReport();
                        }
                        #else
                        //////////////////////////////////////////
                        ////Custom Syntax modify at your needs
                        /////////////////////////////////////////
                        Engine.defaultprinter.Write("1f");
                        #endif
                    }
                }

            }

        }

        private void BtnZRep_Click(object sender, EventArgs e){
        
            if(Engine.HasAccess(resources.GetString("FiscalPrinter_ZReport"))){

                PopBox pop = new PopBox();

                if(pop.ShowDialog(resources.GetString("msgchiusuratxt"), new Point(370, 300)) == DialogResult.Yes){
                    if(Engine.defaultprinter!=null){
                        #if CONFIG_POSNET
                        Engine.defaultprinter.PrintZReport();
                        #else
                        //////////////////////////////////////////
                        ////Custom Syntax modify at your needs
                        /////////////////////////////////////////
                        Engine.defaultprinter.Write("1F");
                        #endif
                    }
                }

            }

        }

    #endif


        private void BtnPrint_Click(object sender, EventArgs e){

            if(Engine.HasAccess(resources.GetString("Journal_Print"))){

                j.PrintContent();

            }

        }

        private void dgdJournal_ColumnAdded(object sender, DataGridViewColumnEventArgs e){
            e.Column.SortMode = DataGridViewColumnSortMode.NotSortable;
        }

        private Point mouse_down; 
        private Rectangle dragBoxFromMouseDown;

        private void dgdJournal_MouseDown(object sender, MouseEventArgs e){
            mouse_down = new Point(e.X, e.Y);
            Size dragSize = SystemInformation.DragSize;
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void dgdJournal_MouseMove(object sender, MouseEventArgs e){
        
            if(e.Button == MouseButtons.Left){
                int dX = (mouse_down.X - e.X);
                int dY = (mouse_down.Y - e.Y);
                pnlJournal.AutoScrollPosition = new Point((dX - pnlJournal.AutoScrollPosition.X), (dY - pnlJournal.AutoScrollPosition.Y));
            }

        }

    }

}