﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General public License for more details.
 * 
 * You should have received a copy of the GNU General public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System.Data;
using Microsoft.PointOfService;
using POS.CashPlugin.Objects;
using POS.Framework;
using POS.Objects;

namespace POS.FiscalPlugin.Objects{

    public class Journal{

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FiscalPlugin));
        private POS.Interfaces.IFiscal dbin = (POS.Interfaces.IFiscal)Engine.DbService; 
        private POS.Interfaces.IDb dbout = (POS.Interfaces.IDb)Engine.DbService;
        private DataTable jds;
        private string jtype;

        public DataTable Content{ 
            get{
                return jds;
            }
        }

        public string ContentType{
            get{
                return jtype;
            }
        }

        public DataTable GetTotals(){ 
            jds = dbin.GetTotals().Tables[0];
            //tilde is for double height font, works also with pos net 
            //but is printer specific
            jtype = "             TOTALI             ";
            return jds;
        }

        public DataTable GetPayments(){ 
            jds = dbin.GetPayments().Tables[0];
            //tilde is for double height font, works also with pos net 
            //but is printer specific
            jtype = "           PAGAMENTI            ";
            return jds;
        }

#if CONFIG_NETWORK

        public DataTable GetNetTotals(){
            //jds = dbin.GetNetTotals(Engine.GetPeers).Tables[0];
            jtype = "           PAGAMENTI            ";
            return jds;
        }

#endif

        public DataTable GetRepTotals(){
            jds = dbin.GetRepTotals();
            jtype = "            REPARTI             ";
            return jds;
        }

        public DataTable GetPidTotals(){
            jds = dbin.GetPidTotals();
            jtype = "            PRODOTTI            ";
            return jds;
        }

        public DataTable GetAliTotals(){
            jds = dbin.GetAliTotals();
            jtype = "            ALIQUOTE            ";
            return jds;
        }

        /// <summary>
        /// Format and print datagridview content
        /// </summary>
        public void PrintContent(){

            if(this.Content==null || this.Content.Rows.Count<2) return;

            if(Engine.defaultprinter!=null){

                string col0, col1, col2;
                string[] str= new string[5];
                int[] width= new int[4];
                DataRow i;
                int r;

                col0 = "COL0";
                col1 = "COL1";
                col2 = "COL2";

                if(this.ContentType.Contains("PAGAMENTI")){
                    width[0] = 10; //max value 999.999,99
                    width[1] = 10; //max value 999.999,99
                    width[2] = 10; //max value 999.999,99
                }else{
                    width[0] = 14;
                    width[1] = 6; //max value 999,99
                    width[2] = 10; //max value 999.999,99
                }

#if CONFIG_POSNET
#if CONFIG_FISCAL

                Engine.defaultprinter.BeginNonFiscal();
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, (char)27 + this.ContentType + "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "--------------------------------" + "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, col0.PadRight(width[0]) + "|" + col1.PadLeft(width[1]) + "|" + col2.PadLeft(width[2]) + "\r\n");
                Engine.defaultprinter.PrintNormal(Microsoft.PointOfService.FiscalPrinterStations.Receipt, "--------------------------------" + "\r\n");
#else
                Engine.defaultprinter.BeginInsertion(0);
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                //should use escape sequence to double height font and to center text
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, this.ContentType + "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "--------------------------------" + "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, col0.PadRight(width[0]) + "|" + col1.PadLeft(width[1]) + "|" + col2.PadLeft(width[2]) + "\r\n");
                Engine.defaultprinter.PrintImmediate(Microsoft.PointOfService.PrinterStation.Receipt, "--------------------------------" + "\r\n");

#endif
#else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////
                Engine.defaultprinter.Write("j@@@" + "\"" + this.ContentType + "\"" + "@@");
                Engine.defaultprinter.Write("\"--------------------------------\"@");
                Engine.defaultprinter.Write("\"" + col0.PadRight(width[0]) + "|" + col1.PadLeft(width[1]) + "|" + col2.PadLeft(width[2]) + "\"@");
                Engine.defaultprinter.Write("\"--------------------------------\"@");
#endif

                for(r = 0;r<=this.Content.Rows.Count - 2;r++){

                    i = this.Content.Rows[r];

                    //formattazione stringa descrizione riga
                    str[0] = i[0].ToString().ToUpper();
                    if(str[0].Length > width[0]){
                        str[0] = str[0].Substring(0, width[0]);
                    }else{
                        str[0] = str[0].PadRight(width[0]);
                    }

                    for( int s = 1;s<=this.Content.Columns.Count - 1;s++){

                        str[s] = string.Format("{0:n}",i[s]);
                        if(str[s].Length > width[s]){
                            str[s] = str[s].Substring(0, width[s]);
                        }else{
                            str[s] = str[s].PadLeft(width[s]);
                        }

                    }


#if CONFIG_POSNET
#if CONFIG_FISCAL
                    Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, str[0] + "|" + str[1] + "|" + str[2] + "\r\n");
#else
                    Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, str[0] + "|" + str[1] + "|" + str[2] + "\r\n");
#endif
#else
                    ////////////////////////////////////////
                    //Custom Syntax modify at your needs
                    ///////////////////////////////////////
                    //line is formatted to 32 characters fixed lenght
                    Engine.defaultprinter.Write("\"" + str[0] + "|" + str[1] + "|" + str[2] + "\"@")
#endif
                }

#if CONFIG_POSNET
#if CONFIG_FISCAL
                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, "--------------------------------" + "\r\n");
#else
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, "--------------------------------" + "\r\n");
#endif
#else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////
                //empty line before last line
                Engine.defaultprinter.Write("\"--------------------------------\"@");
#endif

                //print last line
                i = this.Content.Rows[r];

                for( int s = 1;s<=this.Content.Columns.Count - 1;s++){

                    str[s] = string.Format("{0:n}",i[s]);
                    if(str[s].Length > width[s]){
                        str[s] = str[s].Substring(0, width[s]);
                    }else{
                        str[s] = str[s].PadLeft(width[s]);
                    }
                }

#if CONFIG_POSNET
#if CONFIG_FISCAL

                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, resources.GetString("totaltxt").PadRight(width[0]) + "|" + str[1] + "|" + str[2] + "\r\n");
                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, "--------------------------------" + "\r\n");
                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.PrintNormal(FiscalPrinterStations.Receipt, "\r\n");
                Engine.defaultprinter.EndNonFiscal();
#else
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, resources.GetString("totaltxt").PadRight(width[0]) + "|" + str[1] + "|" + str[2] + "\r\n");
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, "--------------------------------" + "\r\n");
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.PrintImmediate(PrinterStation.Receipt, "\r\n");
                Engine.defaultprinter.EndInsertion();
#endif
#else
                ////////////////////////////////////////
                //Custom Syntax modify at your needs
                ///////////////////////////////////////
                // scrivere TOTALE e' vietato non lo stampa
                Engine.defaultprinter.Write("\"" + "TOTAL".PadRight(width[0]) + "|" + str[1] + "|" + str[2] + "\"@");
                Engine.defaultprinter.Write("\"--------------------------------\"@@@@");
                Engine.defaultprinter.Write("J");
#endif
            }

        }

        /// <summary>
        /// Save document statistics in the journal
        /// </summary>
        /// <param name="value">Document to save</param>
        /// <returns></returns>
        public int ProcessDocument(object value){

            if(value==null) return -1;

            Document doc = (Document)value; 

            ////////////////////////////////////////////
            //gestisce gli sconti ma va TESTATA bene
            //non gestisce sconti totali
            //se c'e' uno sconto totale da dove scalo ???
            //inoltre non mostra gli sconti, cioe' quanti sconti ho fatto
            //nel giornale
            ////////////////////////////////////////////
            Transaction tr;
            for(int i = 0;i<=doc.Transactions.Count - 1;i++){
                tr = doc.Transactions[i];
                switch (tr.Type){
                    case TransactionType.vendita:
                        //
                        //Item modifiers
                        //
                        if(i < doc.Transactions.Count - 1){
                            if(doc.Transactions[i + 1].Type == TransactionType.scontovalore || doc.Transactions[i + 1].Type == TransactionType.scontopercento){
                                SaveTrans(tr.Amount + doc.Transactions[i + 1].Amount, tr.Quantity, ((SellingTransaction)tr).Product.Id, -doc.Transactions[i + 1].Amount, 0);
                                SaveTotals(resources.GetString("totaldistxt"), -doc.Transactions[i + 1].Amount);
                            }
                            else if (doc.Transactions[i + 1].Type == TransactionType.maggvalore || doc.Transactions[i + 1].Type == TransactionType.maggpercento)
                            {
                                SaveTrans(tr.Amount + doc.Transactions[i + 1].Amount, tr.Quantity, ((SellingTransaction)tr).Product.Id, 0, doc.Transactions[i + 1].Amount);
                                SaveTotals(resources.GetString("totalsurtxt"), doc.Transactions[i + 1].Amount);
                            }
                            else {
                                SaveTrans(tr.Amount, tr.Quantity, ((SellingTransaction)tr).Product.Id, 0, 0);
                            }
                        }else{
                            SaveTrans(tr.Amount, tr.Quantity, ((SellingTransaction)tr).Product.Id, 0, 0);
                        }
                        break;
                        //
                        //Total modifiers
                        //
                    case TransactionType.scontotot:
                    case TransactionType.scontovaltot:
                        SaveTotals(resources.GetString("totaldistxt"), -tr.Amount);
                        break;
                    case TransactionType.maggtot:
                    case TransactionType.maggvaltot:
                        SaveTotals(resources.GetString("totalsurtxt"), tr.Amount);
                        break;
                }
            }

            //for the moment change is subtracted from last payment as in ECRs
            //but there may be strange situations this way
            if(doc.Change > 0) doc.Payments[doc.Payments.Count - 1].Amount -= doc.Change;

            //save payments stats
            foreach(Payment i in doc.Payments){
                SavePayments(i.Name, i.Amount / (double)100);
                SaveTotals(resources.GetString("gdttxt"), i.Amount / (double)100);
                SaveTotals(resources.GetString("dailytotaltxt"), i.Amount / (double)100);
                if(!i.Paid){
                    SaveTotals(resources.GetString("notpaidtxt"), i.Amount / (double)100);
                }
            }

            if(doc.Id!=0){
                SaveTotals(resources.GetString("receipttxt"), 1);
            }

            return 0;

        }

        private int SaveTrans(double value, double qty, int pid, double discount, double surcharge){

            int retval=0;
            if(dbout.SaveData("UPDATE jrn_products SET products_quantity = products_quantity + " + qty + ", " +
                              "products_price = products_price + " + value.ToString(new System.Globalization.CultureInfo("en-US")) + ", " +
                              "products_discount = products_discount + " + discount.ToString(new System.Globalization.CultureInfo("en-US")) + ", " +
                              "products_surcharge = products_surcharge + " + surcharge.ToString(new System.Globalization.CultureInfo("en-US")) + " WHERE products_id=" + pid)>0){
                retval = 1;
            }else{
                retval = dbout.SaveData("INSERT INTO jrn_products values (" + pid + ", " + qty + ", " + value.ToString(new System.Globalization.CultureInfo("en-US")) + ", " + discount.ToString(new System.Globalization.CultureInfo("en-US")) + ", " + surcharge.ToString(new System.Globalization.CultureInfo("en-US")) + ")");
            }

            //update products stock quantity
            retval += dbout.SaveData("UPDATE products SET products_quantity = products_quantity - " + qty + " WHERE products_id=" + pid + " AND products_quantity>0");

            #if CONFIG_NETWORK
            Synchronize("", SyncDirectionOrder.Upload);
            #endif

            return retval;

        }

        private int SavePayments(string pay, double value){

            int retval=0;

            if(dbout.SaveData("UPDATE jrn_payments set payment_total = payment_total + " + value.ToString(new System.Globalization.CultureInfo("en-US")) +
                            ",payment_daily = payment_daily + " + value.ToString(new System.Globalization.CultureInfo("en-US")) + " where payment_name= '" + pay + "'")>0){
                retval=1;
            }else{
                retval=dbout.SaveData("INSERT INTO jrn_payments values ('" + pay + "', " + value.ToString(new System.Globalization.CultureInfo("en-US")) + ", " + value.ToString(new System.Globalization.CultureInfo("en-US")) + ")");
            }

            return retval;

        }

        private int SaveTotals(string name, double value){
            
            int retval=0;

            if(dbout.SaveData("UPDATE jrn_totals set total_value = total_value + " + value.ToString(new System.Globalization.CultureInfo("en-US")) +
                            " where total_name= '" + name + "'")>0){
                retval=1;
            }else{
                retval = dbout.SaveData("INSERT INTO jrn_totals values ('" + name + "', " + value.ToString(new System.Globalization.CultureInfo("en-US")) + ")");
            }
            return retval;
        }

        public int ResetTotals(bool total){

            int retval = 0;

            if(total){
                //this way language is dynamic but is a temp solution anyway.
                //plugin should initialize db
                retval += dbout.SaveData("delete from jrn_payments");
                retval += dbout.SaveData("delete from jrn_totals");
            }else{
                retval += dbout.SaveData("UPDATE jrn_payments SET payment_daily=0");
                retval += dbout.SaveData("UPDATE jrn_totals SET total_value=0 WHERE total_name <> 'GRAN TOTALE'");
            }

            retval += dbout.SaveData("delete from jrn_products");

            Message mess=new Message();
            mess.func = 1;
            mess.MessageCashPlugin();

            return retval;

        }

        public int GetReceiptNumber(){

            return (int)dbout.GetData("SELECT total_value FROM jrn_totals WHERE total_name='" + resources.GetString("receipttxt") + "'", 0, 0).Tables[0].Rows[0][0];

        }

    }

}