/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Data;
using System.Data.SqlServerCe;
using System.Globalization;
using POS.Interfaces;
#if CONFIG_NETWORK
using Microsoft.Synchronization; //2.1
using Microsoft.Synchronization.Data; //3.1
using Microsoft.Synchronization.Data.SqlServerCe; //3.1
using System.Net.PeerToPeer; //3.5
using System.ServiceMode; //3.0
#endif


namespace POS.Db {

#if CONFIG_NETWORK
//    commented because creates problems with resources!!!
//    public class NetworkServices
//    {

//        public static void Start(){

//            ServiceHost host = new ServiceHost(typeof(DbService));

//            host.AddServiceEndpoint(typeof(Interfaces.IProducts), new NetTcpBinding(SecurityMode.None, true), "net.tcp://localhost:9000/prod")

//            host.AddServiceEndpoint(typeof(Interfaces.IFiscal), new NetTcpBinding(SecurityMode.None, true), "net.tcp://localhost:9000/fiscal")

//            host.AddServiceEndpoint(typeof(Interfaces.ISync), new NetTcpBinding(SecurityMode.None, true), "net.tcp://localhost:9000/sync")

//            try{

//                host.Open();

//            }catch(Exception ex){
//                throw ex;
//            }

//        }

//    }
#endif

    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    //Classe DbService per comunicare con il database
    //Implementa le interfacce
    //
    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

#if CONFIG_NETWORK
    // commented because creates problems with resources!!!
    //[ServiceModel.ServiceBehavior(InstanceContextMode=InstanceContextMode.PerSession)]
    //public class DbService : IDb, IProducts, IFiscal, ISync {

    //    System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(DbService));
    //    private const string m_oConnString = "Data Source=|DataDirectory|\base.sdf";
    //    private SqlCeConnection m_oCn = new SqlCeConnection(m_oConnString);
    //    private SqlCeDataAdapter m_oDa = new SqlCeDataAdapter();
    //    private SqlCeSyncProvider m_oSp = new SqlCeSyncProvider();

#else
    public class DbService : IDb, IProducts, IFiscal {

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(DbService));
        private const string m_oConnString = "Data Source=|DataDirectory|\\base.sdf;";
        private SqlCeConnection m_oCn;
        private SqlCeDataAdapter m_oDa;
#endif

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Constructor check db connection
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public DbService() {

            try{

                m_oCn = new SqlCeConnection(m_oConnString);
                m_oDa = new SqlCeDataAdapter();

                m_oCn.Open();


#if CONFIG_NETWORK
                DbSyncScopeDescription scopeDesc = New DbSyncScopeDescription("products");

                scopeDesc.Tables.Add(SqlCeSyncDescriptionBuilder.GetDescriptionForTable("products", m_oCn))

                Dim serverConfig As New SqlCeSyncScopeProvisioning(m_oCn)

                serverConfig.SetCreateTableDefault(DbSyncCreationOption.Skip)

                If Not serverConfig.ScopeExists("products") Then
                    ' Configure the scope and change tracking infrastructure. 
                    serverConfig.PopulateFromScopeDescription(scopeDesc)
                    serverConfig.Apply()
                End If
#endif
            }
            catch(Exception ex){
                throw new Exception(resources.GetString("msgconnecterror"), ex);
            }

        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Ritorna un DATASET da una qualsiasi QUERY
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //------------- Inserire optional connection da passare
        public DataSet GetData(string query, int start, int max){

            try
            {

                SqlCeDataAdapter m_oAda= new SqlCeDataAdapter(query, m_oCn);

                DataSet m_oDs = new DataSet();

                m_oDs.Tables.Add();
            
                m_oAda.Fill(start, max, m_oDs.Tables[0]);

                return m_oDs;

            }
            catch(Exception ex){
                throw new Exception(resources.GetString("msgerror"), ex);
            }

        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Metodo per INSERT o UPDATE o DELETE
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public int SaveData(string query){

            if(m_oCn.State!=ConnectionState.Open){
                try{
                    m_oCn.Open();
                }
                catch(Exception ex){
                    throw new Exception(resources.GetString("msgerror"), ex);
                }
            }

            try{

                SqlCeCommand m_com = new SqlCeCommand(query, m_oCn);

                int rows = m_com.ExecuteNonQuery();

                m_oCn.Close();

                return rows;

            }
            catch(SqlCeException ex){
                if(ex.NativeError == 25025){
                    throw new Exception(resources.GetString("msgresetjournal"));
                }
                else{
                    throw new Exception(resources.GetString("msgerror"),ex);
                }
                    
            }

        }

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Metodo per INSERT o UPDATE o DELETE
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        public int SaveData(SqlCeCommand comm){

            if( m_oCn.State!= ConnectionState.Open){
                try{
                    m_oCn.Open();
                }
                catch(Exception ex){
                    throw new Exception(resources.GetString("msgerror"), ex);
                }
            }

            try{

                comm.Connection = m_oCn;

                comm.Prepare();

                int rows = comm.ExecuteNonQuery();

                m_oCn.Close();

                return rows;

            }
            catch(SqlCeException ex){
                if(ex.NativeError == 25025){
                    throw new Exception(resources.GetString("msgresetjournal"));
                }
                else{
                    throw new Exception(resources.GetString("msgerror"), ex);
                }
                    
            }

        }

#if CONFIG_NETWORK
        public SyncOperationStatistics Synchronize(string uri, string scopename, SyncDirectionOrder direction){

            try{

                m_oSp = new SqlCeSyncProvider(scopename, m_oCn);
                m_oSp.SyncProviderPosition = SyncProviderPosition.Local;
                //m_oSp.Configuration.ConflictResolutionPolicy = ConflictResolutionPolicy.SourceWins;

                //proxy remote provider
                //SyncProxy proxyRemote = new SyncProxy(scopename, uri);
                //proxyRemote.Configuration.ConflictResolutionPolicy = ConflictResolutionPolicy.SourceWins;

                SyncOrchestrator syncAgent = new SyncOrchestrator();

                syncAgent.LocalProvider = m_oSp;
                syncAgent.RemoteProvider = new SqlCeSyncProvider(scopename, new SqlCeConnection("Data Source=|DataDirectory|\\base2.sdf"));
                syncAgent.Direction = direction;

                return syncAgent.Synchronize();

            }catch(DbOutdatedSyncException ex){
                MsgBox(ex.Message);
                MsgBox(("Outdated Knowledge: "
                + (ex.OutdatedPeerSyncKnowledge.ToString + (" Clean up knowledge: " + ex.MissingCleanupKnowledge.ToString))));
            }catch(Exception ex){
                MsgBox(ex.Message);
            }

        }
#endif

#region "IProducts Implementation"
        //////////////////////////////////////////////////////
        //
        //Products Interface Implementation
        //
        //////////////////////////////////////////////////////
        DataTable IProducts.GetProd(int id){

            return GetData("SELECT * FROM products WHERE products_id=" + id, 0, 0).Tables[0];

        }

        DataTable IProducts.GetProdEan(string ean){

            return GetData("SELECT * FROM products WHERE products_ean='" + ean + "'", 0, 0).Tables[0];

        }

        DataSet IProducts.GetProds(int page, int mode){

            if(mode<0){
                if(page < 0) return GetData("SELECT * FROM products", 0, 1000);
                else return (GetData("SELECT products_id as id, products_model as prodotto, products_price/100 as prezzo, aliquota as iva, products_quantity as qta, categories_id as rep, CASE categories_id WHEN 0 THEN products_id ELSE categories_id " +
                                   "END as repord, products_ean as ean, fiscale FROM products ORDER BY repord, rep", page * 18, 54));
            }else{
                if(page < 0) return GetData("SELECT * FROM products", 0, 1000);
                else return (GetData("SELECT products_id as id, products_model as prodotto, products_price/100 as prezzo, aliquota as iva, products_quantity as qta, categories_id as rep, " +
                                   "products_ean as ean, fiscale FROM products ORDER BY prodotto", page * 18, 54));
            }

        }

        int IProducts.SaveProd(bool isnew, int id, string name, string ean, double price, double qty, bool fiscale, int cat, double aliquota){

            int retval=0;

            if(!isnew){

                SqlCeCommand upcomm = new SqlCeCommand();

                upcomm.CommandText = "UPDATE [products] SET [products_model] = @name, [products_ean] = @ean, [products_pri" +
                    "ce] = @price, [categories_id] = @cat, [aliquota] = @aliquota, [fiscale] = @fiscale, [products_q" +
                    "uantity] = @qty, [last_modified] = getdate() WHERE (([products_id] = @pid))";

                upcomm.Parameters.Add("@pid", SqlDbType.Int, 0, "products_id").Value = id;
                upcomm.Parameters.Add("@name", SqlDbType.NVarChar, 22, "products_model").Value = name;
                upcomm.Parameters.Add("@ean", SqlDbType.NVarChar, 32, "products_ean").Value = ean;
                upcomm.Parameters.Add("@price", SqlDbType.Money, 0, "products_price").Value = price;
                upcomm.Parameters.Add("@qty", SqlDbType.Decimal, 19, "products_quantity").Value = qty;
                upcomm.Parameters.Add("@cat", SqlDbType.Int, 0, "categories_id").Value = cat;
                upcomm.Parameters.Add("@aliquota", SqlDbType.Int, 0, "aliquota").Value = aliquota;
                upcomm.Parameters.Add("@fiscale", SqlDbType.TinyInt, 0, "fiscale").Value = Convert.ToInt32(fiscale);

                retval = SaveData(upcomm);


            }else{

                SqlCeCommand insprodcomm = new SqlCeCommand(
                "INSERT INTO products " +
                "(products_id, products_model, products_ean, products_price, " +
                "products_quantity, categories_id, aliquota, fiscale, " +
                "last_modified) VALUES (@pid, @name, @ean, @price, " +
                "@qty, @cat , @aliquota, @fiscale, getdate())");

                insprodcomm.Parameters.Add("@pid", SqlDbType.Int).Value = id;
                insprodcomm.Parameters.Add("@name", SqlDbType.NVarChar, 22).Value = name;
                insprodcomm.Parameters.Add("@ean", SqlDbType.NVarChar, 32).Value = ean;
                insprodcomm.Parameters.Add("@price", SqlDbType.Money).Value = price;
                insprodcomm.Parameters.Add("@qty", SqlDbType.Decimal, 19).Value = qty;
                insprodcomm.Parameters.Add("@cat", SqlDbType.Int).Value = cat;
                insprodcomm.Parameters.Add("@aliquota", SqlDbType.Int).Value = aliquota;
                insprodcomm.Parameters.Add("@fiscale", SqlDbType.TinyInt).Value = Convert.ToInt32(fiscale);

                retval = SaveData(insprodcomm);

            }

            return 0;
            //If retval > 0 Then
            //   Synchronize("", SyncDirectionOrder.Upload)
            //End If

    }

#if CONFIG_NETWORK

        Public Function Clone(ByVal url As String) As Integer

            Dim page As Integer = 0
            Dim address As EndpointAddress = New EndpointAddress(url)
            Dim binding As NetTcpBinding = New NetTcpBinding(SecurityMode.None, True)
            Dim factory As ChannelFactory(Of IProducts) = New ChannelFactory(Of IProducts)(binding, address)
            Dim table As New DataSet

            Try
                table = factory.CreateChannel().GetProds(-1, -1)
            Catch ex As FaultException

            End Try

            SaveData("delete from products_tracking")
            SaveData("delete from jrn_products")
            SaveData("delete from products")

            For Each row As DataRow In table.Tables(0).Rows
                SaveProd(True, row.Item(0), row.Item(1), row.Item(2), row.Item(3), row.Item(7), _
                row.Item(6), row.Item(4), row.Item(5))
            Next


            Return table.Tables(0).Rows.Count

        End Function

        Public Function CloneSpeed(ByVal url As String) As Integer

            Dim dirpath As String
            Dim page As Integer = 0
            Dim address As EndpointAddress = New EndpointAddress(url)
            Dim binding As NetTcpBinding = New NetTcpBinding(SecurityMode.None, True)
            Dim factory As ChannelFactory(Of IProducts) = New ChannelFactory(Of IProducts)(binding, address)
            Dim table As New DataSet

            Try
                table = factory.CreateChannel().GetSpeed
            Catch ex As FaultException

            End Try

            'cancellare tutti gli speed xml
            If Strings.Right(System.Windows.Forms.Application.StartupPath, 1) = "\" Then
                dirpath = System.Windows.Forms.Application.StartupPath
            Else
                dirpath = System.Windows.Forms.Application.StartupPath & "\"
            End If

            'per ogni table nel dataset scrivi lo speed xml
            While page >= 0 And page < table.Tables.Count
                table.Tables(page).WriteXml(dirpath & "speed" & page & ".xml")
                page += 1
            End While

            Return table.Tables(0).Rows.Count

        End Function
#endif

        DataSet IProducts.GetSpeed(){

            DataSet ds = new DataSet();
            //int page=0;
            //string dirpath;

            //if(Application.StartupPath.EndsWith("\\")){
            //    dirpath = Application.StartupPath;
            //}
            //else{
            //    dirpath = Application.StartupPath + "\\";
            //}

            //while(System.IO.File.Exists(dirpath + "speed" + page + ".xml")){

            //    string filepath;
            //    DataTable dt = new DataTable("speed");
            //    filepath = dirpath + "speed" + page + ".xml";

            //    dt.ReadXmlSchema(filepath);
            //    dt.ReadXml(filepath);

            //    dt.TableName = "speed" + page;
            //    ds.Tables.Add(dt);

            //    page++;

            //}

            return ds;

        }

#endregion

#region "IFiscal Implementation"
        //////////////////////////////////////////////////////////////
        //
        //Implementazione Interfaccia Plugin Report Fiscali
        //
        //////////////////////////////////////////////////////////////
        DataSet IFiscal.GetTotals(){

            DataSet datas = GetData("select total_name as " + resources.GetString("totaletxt") + ", total_value as " + resources.GetString("valoretxt") + " from jrn_totals", 0, 0);
            DataTable ds = datas.Tables[0];

            if (ds.Rows.Count <= 0){
                ds.Rows.Add(resources.GetString("noresulttxt"), 0);
            }

            return datas;

        }

#if CONFIG_NETWORK
        Public Function GetNetTotals(ByVal addr As PeerNameRecordCollection) As DataSet

            '// GetPayments() local machine
            Dim ds As DataSet = GetData("select payment_name as " & resources.GetString("pagamentxt") & ", payment_total as " & resources.GetString("totaletxt") & ", payment_daily as " & resources.GetString("giornalierotxt") & " from jrn_payments", 0, 0) ' compute sum(payment_total), sum(payment_daily)")

            For Each rec As Net.PeerToPeer.PeerNameRecord In addr
                Dim address As EndpointAddress = New EndpointAddress("net.tcp://" & rec.EndPointCollection.Item(1).Address.ToString & ":9000/fiscal")
                Dim binding As NetTcpBinding = New NetTcpBinding(SecurityMode.None, True)
                Dim factory As ChannelFactory(Of IFiscal) = New ChannelFactory(Of IFiscal)(binding, address)
                Dim table As DataSet = factory.CreateChannel().GetTotals

                For i As Integer = 0 To table.Tables(0).Rows.Count - 2

                    ds.Tables(0).Rows(i).Item(1) += table.Tables(0).Rows(i).Item(1)
                    ds.Tables(0).Rows(i).Item(2) += table.Tables(0).Rows(i).Item(2)
                Next

            Next

            If ds.Tables(0).Rows.Count > 0 Then
                ds.Tables(0).Rows.Add(resources.GetString("totaletxt"), ds.Tables(0).Compute("SUM(" & resources.GetString("totaletxt") & ")", ""), ds.Tables(0).Compute("SUM(" & resources.GetString("giornalierotxt") & ")", ""))
            End If

            Return ds

        End Function
#endif

        DataSet IFiscal.GetPayments(){

            DataSet datas = GetData("select payment_name as " + resources.GetString("pagamentxt") + ", payment_total as " + resources.GetString("totaletxt") + ", payment_daily as " + resources.GetString("giornalierotxt") + " from jrn_payments", 0, 0); // compute sum(payment_total), sum(payment_daily)")
            DataTable ds = datas.Tables[0];

            if(ds.Rows.Count > 0){
                ds.Rows.Add(resources.GetString("totaletxt"), ds.Compute("SUM(" + resources.GetString("totaletxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("giornalierotxt") + ")", ""));
            }else{
                ds.Rows.Add(resources.GetString("noresulttxt"), 0, 0);
            }

            return datas;

        }

        DataTable IFiscal.GetRepTotals(){

            DataTable ds = GetData("SELECT products.products_model as " + resources.GetString("repartotxt") + ", sum(tbl1.products_quantity) as " + resources.GetString("pezzitxt") + " , sum(tbl1.products_price) as " + resources.GetString("valoretxt") + " , sum(tbl1.products_discount) as " + resources.GetString("scontitxt") + " , sum(tbl1.products_surcharge) as " + resources.GetString("maggtxt") + " FROM " +
            "(SELECT CASE prodotti.categories_id WHEN 0 THEN prodotti.products_id ELSE prodotti.categories_id " +
                           "END as tbl, jrn_products.products_quantity, jrn_products.products_price, jrn_products.products_discount, jrn_products.products_surcharge from jrn_products" +
    " join products as prodotti on prodotti.products_id=jrn_products.products_id) as tbl1 join " +
    "products on tbl1.tbl = products.products_id GROUP BY products.products_model", 0, 0).Tables[0]; //compute sum(sum(tbl1.products_quantity)), sum(sum(tbl1.products_price))")

            if(ds.Rows.Count > 0){
                ds.Rows.Add(resources.GetString("totaletxt"), ds.Compute("SUM(" + resources.GetString("pezzitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("valoretxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("scontitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("maggtxt") + ")", ""));
            }else{
                ds.Rows.Add(resources.GetString("noresulttxt"), 0, 0, 0, 0);
            }

            return ds;

        }


        DataTable IFiscal.GetPidTotals(){

            DataTable ds = GetData("select prodotti.products_model as " + resources.GetString("prodottotxt") + ", jrn_products.products_quantity as " + resources.GetString("pezzitxt") + ",jrn_products.products_price as " + resources.GetString("valoretxt") + ",jrn_products.products_discount as " + resources.GetString("scontitxt") + ",jrn_products.products_surcharge as " + resources.GetString("maggtxt") + " from jrn_products join products as prodotti on jrn_products.products_id=prodotti.products_id ORDER BY " + resources.GetString("prodottotxt"), 0, 0).Tables[0];

            if( ds.Rows.Count > 0){
                ds.Rows.Add(resources.GetString("totaletxt"), ds.Compute("SUM(" + resources.GetString("pezzitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("valoretxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("scontitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("maggtxt") + ")", ""));
            }else{
                ds.Rows.Add(resources.GetString("noresulttxt"), 0, 0, 0, 0);
            }

            return ds;

        }

        DataTable IFiscal.GetAliTotals(){

            DataTable ds = GetData("SELECT CAST(aliquota as NVARCHAR(10)) as " + resources.GetString("aliquotatxt") + ", SUM(jrn_products.products_quantity) as " + resources.GetString("pezzitxt") + ", SUM(jrn_products.products_price) as " + resources.GetString("valoretxt") + ", SUM(jrn_products.products_discount) as " + resources.GetString("scontitxt") + ", SUM(jrn_products.products_surcharge) as " + resources.GetString("maggtxt") + " FROM jrn_products JOIN products AS prodotti ON jrn_products.products_id=prodotti.products_id " +
                                        "GROUP BY aliquota", 0, 0).Tables[0];

            if( ds.Rows.Count > 0){
                ds.Rows.Add(resources.GetString("totaletxt"), ds.Compute("SUM(" + resources.GetString("pezzitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("valoretxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("scontitxt") + ")", ""), ds.Compute("SUM(" + resources.GetString("maggtxt") + ")", ""));
            }else{
                ds.Rows.Add(resources.GetString("noresulttxt"), 0, 0, 0, 0);
            }

            return ds;

        }

#endregion

#if CONFIG_NETWORK
#region "ISync Implementation"

        Public Sub BeginSession(ByVal scopeName As String) Implements ISync.BeginSession

            m_oSp = New SqlCeSyncProvider(scopeName, New SqlCeConnection("Data Source=|DataDirectory|\base.sdf"))

        End Sub


        Public Sub GetKnowledge(ByRef batchSize As System.UInt32, ByRef knowledge As SyncKnowledge) Implements ISync.GetKnowledge

            m_oSp.GetSyncBatchParameters(batchSize, knowledge)

        End Sub


        Public Function GetChanges(ByVal batchSize As System.UInt32, ByVal destinationKnowledge As SyncKnowledge, ByRef changeData As Object) As ChangeBatch Implements ISync.GetChanges

            Return m_oSp.GetChangeBatch(batchSize, destinationKnowledge, changeData)

        End Function


        Public Sub ApplyChanges(ByVal resolutionPolicy As ConflictResolutionPolicy, ByVal sourceChanges As ChangeBatch, ByVal changeData As Object, ByRef sessionStatistics As SyncSessionStatistics) Implements ISync.ApplyChanges

            Dim syncCallback As New SyncCallbacks()
            m_oSp.ProcessChangeBatch(resolutionPolicy, sourceChanges, changeData, syncCallback, sessionStatistics)

        End Sub


        Public Sub EndSession() Implements ISync.EndSession

            m_oSp = Nothing

        End Sub

#endregion
#endif
    }

#if CONFIG_NETWORK

    Public Class SyncProxy
        Inherits KnowledgeSyncProvider

        Private scopeName As String
        Private proxy As ISync
        Private idFormatGroup As SyncIdFormatGroup = Nothing

        Public Sub New(ByVal scopeName As String, ByVal url As String)
            Me.scopeName = scopeName

            Dim address As EndpointAddress = New EndpointAddress(url)
            Dim binding As NetTcpBinding = New NetTcpBinding(SecurityMode.None, True)
            Dim factory As ChannelFactory(Of ISync) = New ChannelFactory(Of ISync)(binding, address)
            proxy = factory.CreateChannel()

        End Sub 'New

        'The IdFormat settings for the peer provider.
        Public Overrides ReadOnly Property IdFormats() As SyncIdFormatGroup
            Get
                If idFormatGroup Is Nothing Then
                    idFormatGroup = New SyncIdFormatGroup()

                    '
                    ' 1 byte change unit ID
                    '
                    idFormatGroup.ChangeUnitIdFormat.IsVariableLength = False
                    idFormatGroup.ChangeUnitIdFormat.Length = 1

                    '
                    ' GUID replica ID
                    '
                    idFormatGroup.ReplicaIdFormat.IsVariableLength = False
                    idFormatGroup.ReplicaIdFormat.Length = 16


                    '
                    ' Global ID for item IDs
                    '
                    idFormatGroup.ItemIdFormat.IsVariableLength = True
                    idFormatGroup.ItemIdFormat.Length = 10 * 1024
                End If

                Return idFormatGroup
            End Get
        End Property


        Public Overrides Sub BeginSession(ByVal position As SyncProviderPosition, ByVal syncSessionContext As SyncSessionContext)
            proxy.BeginSession(scopeName)
        End Sub


        Public Overrides Sub EndSession(ByVal syncSessionContext As SyncSessionContext)
            proxy.EndSession()

        End Sub


        Public Overrides Sub GetSyncBatchParameters(ByRef batchSize As System.UInt32, ByRef knowledge As SyncKnowledge)  'ToDo: Unsigned Integers not supported
            proxy.GetKnowledge(batchSize, knowledge)

        End Sub


        Public Overrides Function GetChangeBatch(ByVal batchSize As System.UInt32, ByVal destinationKnowledge As SyncKnowledge, ByRef changeDataRetriever As Object) As ChangeBatch  'ToDo: Unsigned Integers not supported
            Return proxy.GetChanges(batchSize, destinationKnowledge, changeDataRetriever)

        End Function


        Public Overrides Sub ProcessChangeBatch(ByVal resolutionPolicy As ConflictResolutionPolicy, ByVal sourceChanges As ChangeBatch, ByVal changeDataRetriever As Object, ByVal syncCallback As SyncCallbacks, ByVal sessionStatistics As SyncSessionStatistics)
            Dim remoteSessionStatistics As New SyncSessionStatistics()
            proxy.ApplyChanges(resolutionPolicy, sourceChanges, changeDataRetriever, remoteSessionStatistics)
            sessionStatistics.ChangesApplied = remoteSessionStatistics.ChangesApplied
            sessionStatistics.ChangesFailed = remoteSessionStatistics.ChangesFailed

        End Sub


        Public Overrides Function GetFullEnumerationChangeBatch(ByVal batchSize As System.UInt32, ByVal lowerEnumerationBound As SyncId, ByVal knowledgeForDataRetrieval As SyncKnowledge, ByRef changeDataRetriever As Object) As FullEnumerationChangeBatch  'ToDo: Unsigned Integers not supported
            'Do nothing.
            '
            Throw New NotImplementedException()

        End Function


        Public Overrides Sub ProcessFullEnumerationChangeBatch(ByVal resolutionPolicy As ConflictResolutionPolicy, ByVal sourceChanges As FullEnumerationChangeBatch, ByVal changeDataRetriever As Object, ByVal syncCallback As SyncCallbacks, ByVal sessionStatistics As SyncSessionStatistics)
            'Do nothing.
            '
            Throw New NotImplementedException()

        End Sub

    End Class
#endif

}