﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using POS.Interfaces;


namespace POS.Framework
{
    public partial class MainForm : Form
    {

        [DllImport("user32.dll")]
        private static extern bool CreateCaret(IntPtr hWnd, IntPtr hBitmap, int nWidth, int nHeight);
        [DllImport("user32.dll")]
        private static extern bool ShowCaret(IntPtr hWnd);

        private void ShowCustomCaret(IntPtr h, int width, int height)
        {
            CreateCaret(h, (IntPtr)0, width, height);
            ShowCaret(h);
        }

        [DllImport("kernel32.dll", ExactSpelling = true)]
        private static extern IntPtr GetCurrentProcess();
        [DllImport("advapi32.dll", ExactSpelling = true, SetLastError = true)]
        private static extern bool OpenProcessToken(IntPtr h, int acc, ref IntPtr phtok);
        [DllImport("advapi32.dll", SetLastError = true)]
        private static extern bool LookupPrivilegeValue(string host, string name, ref long pluid);
        [DllImport("advapi32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool AdjustTokenPrivileges(IntPtr TokenHandle,
           [MarshalAs(UnmanagedType.Bool)]bool DisableAllPrivileges,
           ref TOKEN_PRIVILEGES NewState,
           UInt32 Zero,
           IntPtr Null1,
           IntPtr Null2);
        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        private static extern bool ExitWindowsEx(int flg, int rea);


        private struct TOKEN_PRIVILEGES
        {
            public int PrivilegeCount;
            public long Privileges;
            public int Attributes;
        }

        private static void Logoff()
        {

            const int EWX_LOGOFF = 0;

            ExitWindowsEx(EWX_LOGOFF, 0);
        }

        private static void Shutdown()
        {

            const int EWX_FORCE = 4;
            const int EWX_SHUTDOWN = 1;
            //const int EWX_REBOOT = 2;

            TOKEN_PRIVILEGES tp;

            tp.PrivilegeCount = 1;
            tp.Privileges = 0;
            // enable privilege
            tp.Attributes = 0x2;

            IntPtr token_handle = IntPtr.Zero;

            IntPtr handle = GetCurrentProcess();

            OpenProcessToken(handle, 0x20 | 0x8, ref token_handle);

            // get shutdown privilege LUID
            LookupPrivilegeValue(null, "SeShutdownPrivilege", ref tp.Privileges);

            AdjustTokenPrivileges(token_handle, false, ref tp, 0, IntPtr.Zero, IntPtr.Zero);

            ExitWindowsEx(EWX_SHUTDOWN | EWX_FORCE, 0xFFFF);

        }

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Disposed(object sender, EventArgs e)
        {

            Application.Exit();

        }

        private void BtnImpo_Click(object sender, EventArgs e){

            SettingForm temp = new SettingForm();
            temp.ShowDialog();

        }

        private void BtnExit_Click(object sender, EventArgs e)
        {
            PopBox pop = new PopBox();

            if (pop.ShowDialog(resources.GetString("msglogoff"), new Point(370, 320)) == DialogResult.Yes)
            {
                Logoff();
            }
        }

        private void BtnShut_Click(object sender, EventArgs e)
        {
            PopBox pop = new PopBox();

            if (pop.ShowDialog(resources.GetString("msgshutdown"), new Point(370, 320)) == DialogResult.Yes)
            {
                Shutdown();
            }
        }

        private void TabControl1_SelectedIndexChanged(object sender, System.EventArgs e)
        {

            if (!TabControl1.SelectedTab.Name.Equals("TabImpo") && !TabControl1.SelectedTab.Name.Equals("TabBrowser"))
            {
                IPlugin idk = (IPlugin)(Engine.Plugins[TabControl1.SelectedIndex + 1]);
                idk.TabEnter();
            }

            Engine.HideKeyboard();

        }

        private void LstPlugins_Click(object sender, EventArgs e)
        {
            if (LstPlugins.Items.Count > 0)
            {

                try
                {
                    IPlugin idk = (IPlugin)(Engine.Plugins[LstPlugins.SelectedIndex + 1]);

                    idk.SetProperties();

                }
                catch (Exception ex)
                {
                    PopBox pop = new PopBox();
                    pop.ShowDialog(resources.GetString("msgerror") + ex.Message, new Point(370, 300), MessageBoxButtons.OK);
                }

            }
        }

#region HeadingForm

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //PosDevices Settings
        //
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private void PosDevices_Click(object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Printer_Header")))
            {

                Form HeadingForm = new Form();
                TextBox txtintesta1 = new TextBox();
                TextBox txtintesta2 = new TextBox();
                TextBox txtintesta3 = new TextBox();

                Label HeadingFormLbl = new Label();
                Button BtnCanc = new Button();
                Button BtnSave = new Button();

                HeadingFormLbl.AutoSize = true;
                HeadingFormLbl.Text = resources.GetString("HeaderFormLbl.Text");
                HeadingFormLbl.Top = 10;
                HeadingFormLbl.Left = 20;

                txtintesta1.Left = 20;
                txtintesta1.MaxLength = 32;
                txtintesta1.Name = "txtintesta1";
                txtintesta1.Top = 30;
                txtintesta1.Width = 200;

                txtintesta2.Left = 20;
                txtintesta2.MaxLength = 32;
                txtintesta2.Name = "txtintesta2";
                txtintesta2.Top = 50;
                txtintesta2.Width = 200;

                txtintesta3.Left = 20;
                txtintesta3.MaxLength = 32;
                txtintesta3.Name = "txtintesta3";
                txtintesta3.Top = 70;
                txtintesta3.Width = 200;

                //
                // BtnSave
                //
                BtnSave.Name = "BtnSave";
                BtnSave.Font = new Font("Microsoft Sans Serif", 16F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Pixel, ((byte)(0)));
                BtnSave.Size = new Size(100, 50);
                BtnSave.Location = new Point(270, 170);
                BtnSave.Text = resources.GetString("BtnSave.Text");
                BtnSave.Click += salva;
                //
                // BtnCanc
                //
                BtnCanc.Name = "BtnCanc";
                BtnCanc.Font = new Font("Microsoft Sans Serif", 16F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Pixel, ((byte)(0)));
                BtnCanc.Size = new Size(100, 50);
                BtnCanc.Location = new Point(156, 170);
                BtnCanc.Text = resources.GetString("BtnCanc.Text");
                BtnCanc.Click += annulla;

                HeadingForm.Controls.Add(txtintesta1);
                HeadingForm.Controls.Add(txtintesta2);
                HeadingForm.Controls.Add(txtintesta3);
                HeadingForm.Controls.Add(HeadingFormLbl);
                HeadingForm.Controls.Add(BtnCanc);
                HeadingForm.Controls.Add(BtnSave);

                HeadingForm.Width = 400;
                HeadingForm.Height = 260;
                HeadingForm.FormBorderStyle = FormBorderStyle.FixedDialog;
                HeadingForm.StartPosition = FormStartPosition.Manual;
                HeadingForm.Location = new System.Drawing.Point(312, 70);
                HeadingForm.ControlBox = false;
                HeadingForm.Text = resources.GetString("HeaderForm.Text");
                HeadingForm.Load += HeadingForm_Load;

                if (Engine.printer_header.Length > 0)
                {
                    try
                    {
                        string[] temp = Engine.printer_header.Split('@');
                        txtintesta1.Text = temp[0];
                        txtintesta2.Text = temp[1];
                        txtintesta3.Text = temp[2];
                    }
                    catch (Exception ex)
                    {

                    }
                }

                if (HeadingForm.ShowDialog() == DialogResult.OK)
                {

#if CONFIG_POSNET
                    Engine.printer_header = txtintesta1.Text.PadRight(((32 - txtintesta1.Text.Length) / 2) + txtintesta1.Text.Length).PadLeft(32) + "@" +
                    txtintesta2.Text.PadRight(((32 - txtintesta2.Text.Length) / 2) + txtintesta2.Text.Length).PadLeft(32) + "@" +
                    txtintesta3.Text.PadRight(((32 - txtintesta3.Text.Length) / 2) + txtintesta3.Text.Length).PadLeft(32);
#if CONFIG_FISCAL
                    //Engine.defaultprinter.PrintZReport();
                    //Engine.defaultprinter.SetHeaderLine(1, "12345678901234567890123456789012", false);
                    //Engine.defaultprinter.SetHeaderLine(2, "abcdefghijklmno", false);
                    //Engine.defaultprinter.SetHeaderLine(3, "pqrstuvwxyz", false);
                    //Engine.defaultprinter.SetHeaderLine(4, "", false);
                    //Engine.defaultprinter.SetHeaderLine(5, "", false);
                    //Engine.defaultprinter.SetHeaderLine(6, "", false);
#endif
#else
                    Engine.printer_header = """" +
                    txtintesta1.Text.PadRight(((32 - txtintesta1.Text.Length) / 2) + txtintesta1.Text.Length).PadLeft(32) + """@""" +
                    txtintesta2.Text.PadRight(((32 - txtintesta2.Text.Length) / 2) + txtintesta2.Text.Length).PadLeft(32) + """@""" +
                    txtintesta3.Text.PadRight(((32 - txtintesta3.Text.Length) / 2) + txtintesta3.Text.Length).PadLeft(32) + """@@";
#endif

                    if (((IDb)Engine.DbService).SaveData("UPDATE posxp_settings SET value='" + Engine.printer_header + "' where name='printer_header'") == 0)
                    {
                        ((IDb)Engine.DbService).SaveData("INSERT INTO posxp_settings (name,value,type) VALUES ('printer_header','" + Engine.printer_header + "','posxp')");
                    }

                }

                Engine.HideKeyboard();

            }

        }

        private void HeadingForm_Load(object sender, EventArgs e)
        {
            Engine.ShowKeyboard();
        }

        private void salva(object sender, EventArgs e){
            if (sender.GetType() == typeof(Button))
            {
                Button btn = (Button)sender;

                btn.FindForm().DialogResult = DialogResult.OK;

            }

        }

        private void annulla(object sender, EventArgs e){
            if (sender.GetType()==typeof(Button))
            {
                Button btn = (Button)sender;

                btn.FindForm().DialogResult = DialogResult.Cancel;

            }
        }

#endregion


#region Web Browser
        
        ///////////////////////////////////////
        //webbrowser stuff
        //////////////////////////////////////
        private bool keyboardshown = false;

        private void TxtAddress_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(TxtAddress.Handle, 4, 29);
        }

        private void TxtAddress_Click(object sender, EventArgs e){
            Engine.ShowKeyboard();
            keyboardshown = true;
            TxtAddress.Focus();
        }

        // Navigates to the URL in the address box when 
        // the ENTER key is pressed while the ToolStripTextBox has focus.
        private void TxtAddress_KeyDown(object sender, KeyEventArgs e){

            if(e.KeyCode.Equals(Keys.Enter)){
                Navigate(TxtAddress.Text);
                Engine.HideKeyboard();
                keyboardshown = false;
            }

        }

        // Navigates to the given URL if it is valid.
        private void Navigate(string address){

            if(String.IsNullOrEmpty(address)) return;
            if(address.Equals("about:blank")) return;
            if(!address.StartsWith("http://") &&
                !address.StartsWith("https://")){
                address = "http://" + address;
            }

            try{
                WebBrowser.Navigate(new Uri(address));
            }catch(System.UriFormatException ex){
                return;
            }

        }

        // Updates the URL in TextBoxAddress upon navigation.
        private void WebBrowser_Navigated(object sender, WebBrowserNavigatedEventArgs e){

            this.TxtAddress.Text = WebBrowser.Url.ToString();
            
        }

        private void BtnBack_Click(object sender, System.EventArgs e){
            WebBrowser.GoBack();
        }

        private void BtnForw_Click(object sender, System.EventArgs e){
            WebBrowser.GoForward();
        }

        private void BtnStop_Click(object sender, System.EventArgs e){
            WebBrowser.Stop();
        }

        private void BtnRefresh_Click(object sender, System.EventArgs e){
            WebBrowser.Refresh();
        }

        private void BtnKeyboard_Click(object sender, System.EventArgs e){

            if(keyboardshown){
                Engine.HideKeyboard();
                keyboardshown = false;
            }else{
                Engine.ShowKeyboard();
                keyboardshown = true;
            }
        }

#endregion

    }

}
