﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */


#if CONFIG_NETWORK
using System.Net.PeerToPeer;
using Microsoft.Synchronization;
#endif
#if CONFIG_POSNET
using Microsoft.PointOfService;
#endif
using System;
using System.Data;
using System.Security.Principal;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.InteropServices;


namespace POS.Framework{
    
    public static class Engine{

        /////////////////////////////////////////////////////
        /////////// GLOBAL MODULE PUBLIC VARIABLES
        ////////////////////////////////////////////////////
        private static System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(Engine));
        internal static Microsoft.VisualBasic.Collection Plugins = new Microsoft.VisualBasic.Collection();
        private static Dictionary<int, object> threadDbMap = new Dictionary<int, object>();
        //Public DbService As Object
        public static bool Multicassa;
        public static string dirpath, datapath;
        public static string printer_header;

        ///////////////////////////////////////////////////
        // GLOBAL MODULE PRIVATE VARIABLES
        //////////////////////////////////////////////////
        private static object m_Plugin;
        private static MainForm m_Mainform;
#if CONFIG_NETWORK
        private Cloud m_Cloud;
#endif
        private static POS.Controls.TouchKeyboard keyboard = new POS.Controls.TouchKeyboard();

        ////////////////////////////////
        //Devices variables
        ///////////////////////////////
#if CONFIG_POSNET
#if CONFIG_FISCAL
        public static FiscalPrinter defaultprinter;
#else
        public static PosPrinter defaultprinter;
#endif
        public static LineDisplay defaultdisplay;
        //scanner
        private static Scanner defaultscanner;
        private static string scanstring = "";
        public delegate void PosInputDelegate(string str);
#else
        public static IO.Ports.SerialPort defaultprinter = null;
        public static IO.Ports.SerialPort defaultdisplay = null;
        //scanner
        private static IO.Ports.SerialPort defaultscanner;
        string scanstring = "";
        public Delegate PosInputDelegate(string str);
#endif


        [STAThread]
        static void Main(){

            Splash splashform = new Splash();

            splashform.Show();

            Application.DoEvents();

            dirpath = AppDomain.CurrentDomain.BaseDirectory;

            datapath = dirpath + "data\\";

            AppDomain.CurrentDomain.SetData("DataDirectory", datapath);

            //Initialize authorization store
            try{

                System.Threading.Thread.CurrentPrincipal = new AzmanPrincipal(WindowsIdentity.GetCurrent(), dirpath + resources.GetString("authfilename"));

            }catch(Exception ex){
                MessageBox.Show(ex.Message);
                Application.Exit();
                return;
            }

            try
            {
                // Initialize Printer Header
                DataSet config_ds = ((POS.Interfaces.IDb)Engine.DbService).GetData("SELECT value FROM posxp_settings WHERE name='printer_header'", 0, 0);

                if (config_ds.Tables[0].Rows.Count > 0)
                {
                    printer_header = Convert.ToString(config_ds.Tables[0].Rows[0][0]);
                }
                else
                {
                    printer_header = "";
                }
                
            }
            catch (Exception ex)
            {
                // backward compatibility create table
                int ret = ((POS.Interfaces.IDb)Engine.DbService).SaveData("CREATE TABLE posxp_settings (name nvarchar (100) PRIMARY KEY, value nvarchar (100), type nvarchar (100))");
                
                printer_header = "";
            }
            

            // initialize mainform after authorization store
            // because some controls in mainform uses authorization
            m_Mainform = new MainForm();

            ////////////////////////////////////////////////
            // prova a connetterti al db altrimenti non caricare
            ////////////////////////////////////////////////
            try{

                //DbService = new POS.Db.DbService();

#if CONFIG_NETWORK
                POS.Db.NetworkServices.Start();
#endif
            }
            catch(Exception ex){
                MessageBox.Show(resources.GetString("msgdati") + "\r\n" + ex.Message);
                Application.Run(new MainForm());
                return;
            }

            /////////////////////////////////////
            //NETWORK INITIALIZATION
            //
            /////////////////////////////////////
#if CONFIG_NETWORK
            ''/////////////////////////////////////////////////
            ''PNRP Registration
            ''/////////////////////////////////////////////////
            Try
                Dim pName As PeerName = New PeerName("posxp", PeerNameType.Unsecured)
                Dim reg As PeerNameRegistration = New PeerNameRegistration(pName, 0)
                reg.Comment = Environment.MachineName

                ''Prendo la prima cloud locale
                For Each cl As Cloud In Cloud.GetAvailableClouds
                    If cl.Scope = PnrpScope.LinkLocal And cl.ScopeId = 5 Then
                        reg.Cloud = cl
                        m_Cloud = reg.Cloud
                    End If
                Next

                If m_Cloud Is Nothing Then
                    Throw New Exception
                End If

                reg.Start()
                Multicassa = reg.IsRegistered

            Catch ex As Exception
                ErrorLogger.WriteToErrorLog(ex.Message, ex.StackTrace)
                MsgBox(resources.GetString("msgmulticassa"))
            End Try

            ''///////////////////////////////////////
            ''sincronizzati con la clooud
            ''Direction = 1 DownloadUpload
            ''///////////////////////////////////////
            'Dim res As SyncOperationStatistics = SyncWithFirst(SyncDirectionOrder.Download)

            ''///////////////////////////////////////
            ''Se ho uploadato qualcosa SyncWithAll
            ''Direction Upload
            ''///////////////////////////////////////
            'If Not res Is Nothing Then
            '    If res.UploadChangesApplied > 0 Then

            '    End If
            'End If
#endif

            OpenDevices();

            if (m_Mainform.PosDevices.Items.Count > 0)
            {
                m_Mainform.PosDevices.SelectedIndex = 0;
            }

            LoadPlugins();

            if (m_Mainform.LstPlugins.Items.Count > 0)
            {
                m_Mainform.LstPlugins.SelectedIndex = 0;
            }

            splashform.Close();

            splashform = null;

            Application.Run(m_Mainform);

        }

        private static void LoadPlugins(){

            ///////////////////////////////////////
            //Verifica che il tabcontrol sia stato creato
            //altrimenti non posso inserire pagine
            ///////////////////////////////////////
            if(!m_Mainform.TabControl1.IsHandleCreated){
                IntPtr aa = m_Mainform.TabControl1.Handle;
            }

            DataSet ds = new DataSet();

            ds.ReadXml(dirpath + "plugins.xml");

            if (ds.Tables.Count > 0)
            {

                foreach (DataRow row in ds.Tables[0].Rows)
                {

                    if (Convert.ToInt32(row[0]) == 1)
                    {

                        System.Reflection.Assembly ass = System.Reflection.Assembly.LoadFile(dirpath + row["filename"]);

                        TabPage m_Page = new TabPage();

                        m_Page.BackColor = System.Drawing.SystemColors.Control;
                        m_Page.Location = new System.Drawing.Point(4, 64);
                        m_Page.Name = "Tab_" + row["class"];
                        m_Page.Padding = new System.Windows.Forms.Padding(3);
                        m_Page.Size = new System.Drawing.Size(1018, 700);
                        m_Page.UseVisualStyleBackColor = false;

                        try
                        {
                            m_Plugin = Activator.CreateInstance(ass.GetType((string)row["class"], true));

                            Plugins.Add(m_Plugin, m_Plugin.ToString(), null, null);

                            if (((POS.Interfaces.IPlugin)m_Plugin).Initialize(m_Page) == 0)
                            {

                                m_Mainform.TabControl1.TabPages.Insert(m_Mainform.TabControl1.TabPages.Count - 2, m_Page);

                                m_Mainform.LstPlugins.Items.Add(row["name"]);

                            }

                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message);
                        }

                    }

                }

            }

        }

        #region Devices Functions

        private static void OpenDevices()
        {

            /////////////////////////////////////
            //PERIPHERAL INITIALIZATION
            //
            /////////////////////////////////////
#if CONFIG_POSNET
            /////////////////////////////////////////
            //POS .NET interface
            /////////////////////////////////////////
            PosExplorer posexp = new PosExplorer();
            DeviceCollection devcoll = posexp.GetDevices();

            for (int i = 0; i <= devcoll.Count - 1; i++)
            {

                if (devcoll[i].IsDefault)
                {

#if CONFIG_FISCAL
                    if (devcoll[i].Type == "FiscalPrinter")
                    {
#else
                    if(devcoll[i].Type.Equals("PosPrinter")){
#endif
                        if (Engine.HasAccess(resources.GetString("Printer_Use")))
                        {

                            m_Mainform.PosDevices.Items.Add(devcoll[i].ServiceObjectName);

                            try
                            {
#if CONFIG_FISCAL
                                defaultprinter = (FiscalPrinter)posexp.CreateInstance(devcoll[i]);
#else
                                defaultprinter = (PosPrinter)posexp.CreateInstance(devcoll[i]);
#endif
                                defaultprinter.Open();

                                defaultprinter.Claim(5);
                                defaultprinter.DeviceEnabled = true;

                            }
                            catch (Exception ex)
                            {
                                defaultprinter = null;
                                m_Mainform.PosDevices.Items.Remove(devcoll[i].ServiceObjectName);
                                MessageBox.Show("Printer Error:" + ex.Message);
                            }

                        }

                    }

                    if (devcoll[i].Type.Equals("LineDisplay"))
                    {

                        if (Engine.HasAccess(resources.GetString("Display_Use")))
                        {

                            m_Mainform.PosDevices.Items.Add(devcoll[i].ServiceObjectName);

                            try
                            {

                                defaultdisplay = (LineDisplay)posexp.CreateInstance(devcoll[i]);
                                defaultdisplay.Open();

                                defaultdisplay.Claim(5);
                                defaultdisplay.DeviceEnabled = true;

                            }
                            catch (Exception ex)
                            {
                                defaultdisplay = null;
                                m_Mainform.PosDevices.Items.Remove(devcoll[i].ServiceObjectName);
                                MessageBox.Show("Display Error:" + ex.Message);
                            }

                        }
                    }

                    if (devcoll[i].Type.Equals("Scanner"))
                    {

                        if (Engine.HasAccess(resources.GetString("Scanner_Use")))
                        {

                            m_Mainform.PosDevices.Items.Add(devcoll[i].ServiceObjectName);

                            try
                            {

                                defaultscanner = (Scanner)posexp.CreateInstance(devcoll[i]);

                                defaultscanner.Open();

                                defaultscanner.Claim(5);
                                defaultscanner.DataEvent += scanner_DataReceived;
                                defaultscanner.ErrorEvent += scanner_ErrorEvent;
                                defaultscanner.DeviceEnabled = true;

                                defaultscanner.DataEventEnabled = true;

                            }
                            catch (Exception ex)
                            {
                                defaultscanner = null;
                                m_Mainform.PosDevices.Items.Remove(devcoll[i].ServiceObjectName);
                                MessageBox.Show("Scanner Error:" + ex.Message);
                            }

                        }

                    }

                }

            }
#else
                            ////////////////////////////
                            //Private Sub OpenSerials()
                            //this the old function that manages ports directly
                            /////////////////////////////
                            string[] arr = System.IO.Ports.SerialPort.GetPortNames();
                            Array.Sort(arr);

                            Collection ports = new Collection();

                            for(int i=0;i<=arr.Length - 1;i++){
                                ports.Add(arr(i).ToString, arr(i).ToString);
                            }

                            ////////////////////////////////////////////////////
                            // Assegna la prima porta alla stampante
                            ///////////////////////////////////////////////////

                        if(Engine.HasAccess(resources.GetString("Printer_Use"))){

                            if(ports.Count > 0){

                                defaultprinter = new IO.Ports.SerialPort(ports.Item(1), 9600);

                                //defaultprinter.Handshake = IO.Ports.Handshake.XOnXOff;

                                defaultprinter.WriteBufferSize = 10000;

                                try{

                                    defaultprinter.Open();

                                    m_Mainform.PosDevices.Items.Add(resources.GetString("printertxt") & ports.Item(1));

                                    ports.Remove(ports.Item(1));

                                }catch(Exception ex){
                                    MsgBox(ex.Message);
                                }

                            }

                        }

                            ////////////////////////////////////////////////////
                            // Assegna la seconda porta al display
                            ///////////////////////////////////////////////////

                        if(Engine.HasAccess(resources.GetString("Display_Use"))){

                            if(ports.Count > 0){

                                defaultdisplay = new IO.Ports.SerialPort(ports.Item(1), 9600);

                                defaultdisplay.WriteBufferSize = 10000;

                                try{
                                    defaultdisplay.Open();

                                    m_Mainform.PosDevices.Items.Add(resources.GetString("displaytxt") & ports.Item(1));

                                    ports.Remove(ports.Item(1));

                                }catch(Exception ex){
                                    MsgBox(ex.Message);
                                }

                            }

                        }

                            ////////////////////////////////////////////////////
                            // Assegna la terza porta allo scanner
                            ///////////////////////////////////////////////////

                        if(Engine.HasAccess(resources.GetString("Scanner_Use"))){

                            if(ports.Count > 0){

                                defaultscanner = new IO.Ports.SerialPort(ports.Item(1), 9600);

                                // open scanner
                                try{
                                    defaultscanner.Open();

                                    m_Mainform.PosDevices.Items.Add(resources.GetString("scannertxt") & ports.Item(1));

                                    ports.Remove(ports.Item(1));

                                }catch(Exception ex){
                                    MsgBox(ex.Message);
                                }

                            }


            }


#endif

        }

#if CONFIG_POSNET

        private static void scanner_DataReceived(object sender, Microsoft.PointOfService.DataEventArgs e)
        {

            char ch;
            PosInputDelegate d = PosInput;

            for (int i = 0; i <= (defaultscanner.ScanData.Length - 1); i++)
            {

                ch = (char)defaultscanner.ScanData[i];

                scanstring += ch;

            }

            m_Mainform.Invoke(d, scanstring);

            scanstring = "";

            defaultscanner.DataEventEnabled = true;

        }

        private static void scanner_ErrorEvent(object sender, Microsoft.PointOfService.DeviceErrorEventArgs e)
        {
            defaultscanner.DataEventEnabled = true;
        }

#else
        private void scanner_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        { //Handles defaultscanner.DataReceived

            string ch = "";
            //PosInputDelegate d = new PosInputDelegate(addressOf PosInput);

            while(defaultscanner.BytesToRead > 0){

                ch = Chr(defaultscanner.ReadChar);

                if(ch = vbCr){
                    m_Mainform.Invoke(d, scanstring);
                    scanstring = "";
                    defaultscanner.ReadExisting();
                    return;
                }

                scanstring += ch;

            }

        }
#endif

        // route scanner data
        private static void PosInput(string data)
        {
            if (!m_Mainform.TabControl1.SelectedTab.Name.Equals("TabImpo") && !m_Mainform.TabControl1.SelectedTab.Name.Equals("TabBrowser"))
            {
                POS.Interfaces.IPlugin idk = (POS.Interfaces.IPlugin)Plugins[m_Mainform.TabControl1.SelectedIndex + 1];
                idk.PosData(null, data);
            }
        }

#endregion

        #region Network Functions

#if CONFIG_NETWORK

        ''////////////////////////////////////////////////
        ''Ottiene la collezione dei peer attualmente connessi
        ''////////////////////////////////////////////////
        Public Function GetPeers() As PeerNameRecordCollection

            If Multicassa Then

                Dim resolver As PeerNameResolver = New PeerNameResolver()
                Dim peercoll As PeerNameRecordCollection = resolver.Resolve(New PeerName("0.posxp"), m_Cloud)

                Return peercoll

            End If

            Return Nothing

        End Function

        ''////////////////////////////////////////////////
        ''Ottiene il peer passando il nome
        ''////////////////////////////////////////////////
        Public Function GetPeer(ByVal name As String) As String

            If name <> Environment.MachineName Then

                Dim peercoll As PeerNameRecordCollection = GetPeers()
                For Each p As PeerNameRecord In peercoll
                    If p.Comment = name Then
                        Return p.EndPointCollection.Item(1).Address.ToString
                    End If
                Next
            End If

            Return Nothing

        End Function

        Public Function SyncWithFirst(ByVal direction As SyncDirectionOrder) As SyncOperationStatistics

            If Multicassa Then
                Dim resolver As PeerNameResolver = New PeerNameResolver()
                Dim peercoll As PeerNameRecordCollection = resolver.Resolve(New PeerName("0.posxp"), m_Cloud)
                If peercoll.Count > 0 Then
                    Try

                        'Return DbService.Synchronize("net.tcp://" & peercoll.Item(0).EndPointCollection.Item(1).Address.ToString & ":9000/sync", direction)

                    Catch ex As Exception

                    End Try
                End If
            End If

            Return Nothing

        End Function

        Public Function SyncWithAll(ByVal direction As SyncDirectionOrder)

            If Multicassa Then
                Dim resolver As PeerNameResolver = New PeerNameResolver()
                Dim peercoll As PeerNameRecordCollection = resolver.Resolve(New PeerName("0.posxp"), m_Cloud)
                If peercoll.Count > 0 Then
                    For Each peer As PeerNameRecord In peercoll
                        'DbService.Synchronize("net.tcp://" & peer.EndPointCollection.Item(1).Address.ToString & ":9000/sync", direction)
                    Next
                End If
            End If

            Return Nothing

        End Function

#endif

        #endregion


        /// <summary>
        /// Exposed Functions
        /// </summary>
        public static void ShowKeyboard()
        {

            keyboard.Show(0, 436);

        }

        public static void HideKeyboard()
        {

            keyboard.Hide();

        }


        public static bool HasAccess(string operation)
        {

            return ((AzmanPrincipal)System.Threading.Thread.CurrentPrincipal).HasAccess(operation);

        }


        public static object DbService
        {

            get
            {

                int threadId = System.Threading.Thread.CurrentThread.ManagedThreadId;
                POS.Db.DbService temp = null;

                if (threadDbMap.ContainsKey(threadId))
                {
                    temp = (POS.Db.DbService)threadDbMap[threadId];
                }
                else
                {
                    try
                    {

                        temp = new POS.Db.DbService();
                        threadDbMap.Add(threadId, temp);

                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(resources.GetString("msgdati") + "\r\n" + ex.Message);
                    }

                }

                return temp;

            }

        }

    }

    
    public class ErrorLogger{

        /*************************************************************
        'NAME:          WriteToErrorLog
        'PURPOSE:       Open or create an error log and submit error message
        'PARAMETERS:    msg - message to be written to error file
        '               stkTrace - stack trace from error message
        '               title - title of the error file entry
        'RETURNS:       Nothing
        *************************************************************/
        public static void WriteToErrorLog(string msg,string stkTrace){

            //check and make the directory if necessary; this is set to look in the application
            //folder, you may wish to place the error log in another location depending upon the
            //the user's role and write access to different areas of the file system
            if(!System.IO.Directory.Exists(System.Windows.Forms.Application.StartupPath + "\\Errors\\")){
                System.IO.Directory.CreateDirectory(System.Windows.Forms.Application.StartupPath + "\\Errors\\");
            }

            //check the file
            System.IO.FileStream fs = new System.IO.FileStream(System.Windows.Forms.Application.StartupPath + "\\Errors\\errlog.txt", System.IO.FileMode.OpenOrCreate, System.IO.FileAccess.ReadWrite);
            System.IO.StreamWriter s = new System.IO.StreamWriter(fs);
            s.Close();
            fs.Close();

            //log it
            System.IO.FileStream fs1 = new System.IO.FileStream(System.Windows.Forms.Application.StartupPath + "\\Errors\\errlog.txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
            System.IO.StreamWriter s1 = new System.IO.StreamWriter(fs1);
            s1.Write("Title: Error\r\n");
            s1.Write("Message: " + msg + "\r\n");
            s1.Write("StackTrace: " + stkTrace + "\r\n");
            s1.Write("Date/Time: " + DateTime.Now.ToString() + "\r\n");
            s1.Write("===========================================================================================\r\n");
            s1.Close();
            fs1.Close();

        }

    }



}

namespace POS.Objects {

    public class Message
    {

        public int func;
        public object pointer;

        public int MessageFiscalPlugin(){

            try{
                POS.Interfaces.IPlugin plug = (POS.Interfaces.IPlugin)POS.Framework.Engine.Plugins["POS.FiscalPlugin.FiscalPlugin"];
                return plug.Message(func, pointer);
            }catch(Exception ex){
                return -1;
            }

        }

        public int MessageCashPlugin(){

            try
            {
                POS.Interfaces.IPlugin plug = (POS.Interfaces.IPlugin)POS.Framework.Engine.Plugins["POS.CashPlugin.CashPlugin"];
                return plug.Message(func, pointer);
            }
            catch (Exception ex)
            {
                return -1;
            }

        }

        public int MessageProductPlugin(){

            try
            {
                POS.Interfaces.IPlugin plug = (POS.Interfaces.IPlugin)POS.Framework.Engine.Plugins["POS.ProductPlugin.ProductPlugin"];
                return plug.Message(func, pointer);
            }
            catch (Exception ex)
            {
                return -1;
            }

        }

    }

}