﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.DirectoryServices;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace POS.Framework
{
    public partial class SettingForm : Form
    {

        public SettingForm()
        {
            InitializeComponent();

        }

        private void SettingForm_Load(object sender, System.EventArgs e){

            DataTable cc = new DataTable();
            cc.Columns.Add("Group");
            cc.Columns.Add("Locale");
            cc.Rows.Add("Administrators", resources.GetString("admingroupname"));
            cc.Rows.Add("Power Users", resources.GetString("usersgroupname"));

            LstGroups.ValueMember = "Group";

            LstGroups.DisplayMember = "Locale";

            LstGroups.DataSource = cc;

            LstGroups.SelectedIndex = 0;

            LstAzRoles.DataSource = ((AzmanPrincipal)System.Threading.Thread.CurrentPrincipal).GetRoles;

            TabImpo.SelectedIndex = 0;

        }


        private void LstGroups_SelectedIndexChanged(object sender, System.EventArgs e){

            LstUsers.Items.Clear();

            DirectoryEntry objmember; 
            DirectoryEntry entry = new DirectoryEntry("WinNT://" + Environment.MachineName + "/" + (string)LstGroups.SelectedValue + ",group");
            object members = new object();

            try{
                members = entry.Invoke("members", null);
                
                foreach(object dd in (IEnumerable)members){
                    objmember = new DirectoryEntry(dd);
                    LstUsers.Items.Add(objmember.Name);
                }

            }catch(Exception ex){
                
            }

            if(LstUsers.Items.Count > 0){
                LstUsers.SelectedIndex = 0;
            }

        }

        private void BtnUserAdd_Click(object sender, System.EventArgs e){

            NewUser diag = new NewUser();

            diag.TxtUserGroup.Text = LstGroups.Text;

            if (diag.ShowDialog() == DialogResult.OK)
            {

                DirectoryEntry AD = new DirectoryEntry();
                DirectoryEntry User = new DirectoryEntry();

                try{

                    AD = new DirectoryEntry("WinNT://" + Environment.MachineName + ",computer");
                    User = AD.Children.Add(diag.TxtUserName.Text, "user");

                    User.Invoke("SetPassword", (object)diag.TxtUserPass.Text);
                    User.Invoke("Put", "Description", "PosXp User");
                    User.Invoke("Put", "Userflags", 0x10000);

                    User.CommitChanges();

                    DirectoryEntry grp;

                    grp = AD.Children.Find((string)LstGroups.SelectedValue, "group");
                    if(!grp.Name.Equals("")){
                        grp.Invoke("Add", (object)User.Path);
                    }

                    LstGroups_SelectedIndexChanged(new object(), new EventArgs());

                }catch(Exception ex){
                    if(System.Runtime.InteropServices.Marshal.GetHRForException(ex) == -2146232828){
                        AD.Children.Remove(User);
                        MessageBox.Show(resources.GetString("msgdenied"));
                        return;
                    }
                    MessageBox.Show(ex.Message);
                }

            }

        }

        private void BtnUserDel_Click(object sender, System.EventArgs e){

            DirectoryEntry objmember = new DirectoryEntry("WinNT://" + Environment.MachineName);


            try{
                DirectoryEntry user = objmember.Children.Find((string)LstUsers.SelectedItem);

                object adsGroups = user.Invoke("Groups");
                PopBox pop = new PopBox();

                foreach (object adsGroup in (IEnumerable)adsGroups)
                {
                    DirectoryEntry dd = new DirectoryEntry(adsGroup);
                    if(dd.Name.Equals("Administrators")){
                        
                        if(pop.ShowDialog(resources.GetString("msgadmindel"), new Point(370, 320))==DialogResult.Yes){
                            objmember.Children.Remove(user);
                            LstGroups_SelectedIndexChanged(new object(), new EventArgs());
                        }
                        return;
                    }
                }

                if (pop.ShowDialog(resources.GetString("msguserdel") + "\r\n" + user.Name, new Point(370, 320))==DialogResult.Yes){
                    objmember.Children.Remove(user);
                    LstGroups_SelectedIndexChanged(new object(), new EventArgs());
                    return;
                }

            }catch(Exception ex){
                MessageBox.Show(ex.Message);
            }

        }

        #region AuthorizationTab

        private void LstAzRoles_SelectedIndexChanged(object sender, System.EventArgs e){

            LstAzOperations.DataSource = ((AzmanPrincipal)System.Threading.Thread.CurrentPrincipal).GetOperations(LstAzRoles.SelectedItem.ToString());

        }

        private void BtnAddOp_Click(object sender, System.EventArgs e){

            Operations OpForm = new Operations();

            if(OpForm.ShowDialog()==DialogResult.OK){

                foreach (object op in OpForm.CheckedListBox.CheckedItems)
                {

                    try{
                        ((AzmanPrincipal)System.Threading.Thread.CurrentPrincipal).AddOperation(LstAzRoles.SelectedItem.ToString(), op.ToString());

                        LstAzRoles_SelectedIndexChanged(LstAzRoles, new EventArgs());

                    }catch(Exception ex){
                        MessageBox.Show(ex.Message);
                    }

                }

            }

        }

        private void BtnDelOp_Click(object sender, System.EventArgs e){

            PopBox pop = new PopBox();

            if(pop.ShowDialog(resources.GetString("msgdelop"), new Point(370, 320))==DialogResult.Yes){

                try{

                    ((AzmanPrincipal)System.Threading.Thread.CurrentPrincipal).RemoveOperation(LstAzRoles.SelectedItem.ToString(), LstAzOperations.SelectedItem.ToString());

                    LstAzRoles_SelectedIndexChanged(new object(), new EventArgs());

                }catch(Exception ex){
                    MessageBox.Show(ex.Message);
                }

            }

        }

        private void BtnAzRoleAdd_Click(object sender, System.EventArgs e)
        {

        }

        private void BtnAzRoleDel_Click(object sender, System.EventArgs e)
        {

        }

        #endregion

        private void TabMulti_Enter(object sender, System.EventArgs e){
#if CONFIG_NETWORK
            LstNetLocal.DataSource = null;
            LstNetLocal.DataSource = Engine.GetPeers;
            LstNetLocal.DisplayMember = "comment";
#endif
        }

        private void BtnSyncProd_Click(object sender, System.EventArgs e){
#if CONFIG_NETWORK
            PopBox pop = new PopBox();
            Engine.DbService.Synchronize("", "products", SyncDirectionOrder.Download);
            if(LstNetLocal.Items.Count > 0){

                If (pop.ShowDialog(resources.GetString("msgsyncprod"), new Point(400, 300))==DialogResult.Yes){
                    Application.UseWaitCursor=true;
                    int ret=Engine.DbService.Sync(LstNetLocal.Text);
                    Application.UseWaitCursor=false;
                    pop.ShowDialog(resources.GetString("msgsyncprodresult") + ret, new Point(400, 300), MsgBoxStyle.OkOnly);
                }

                if(pop.ShowDialog(resources.GetString("msgsyncspeed"), new Point(400, 300))==DialogResult.Yes){
                    Application.UseWaitCursor = true;
                    int ret=Engine.DbService.CloneSpeed(LstNetLocal.Text);
                    Application.UseWaitCursor = false;
                    pop.ShowDialog(resources.GetString("msgsyncspeedresult") + ret, new Point(400, 300), MsgBoxStyle.OkOnly);
                }

            }
#endif
                                                                         }
        
    }
}
