﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General public License for more details.
 * 
 * You should have received a copy of the GNU General public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Data;
using POS.Framework;
using POS.Interfaces;

namespace POS.ProductPlugin.Objects{

    /// <summary>
    /// Product Class to manage Products
    /// </summary>
    public class Product{

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ProductPlugin));
        private int int_id;
        private string str_name = "";
        private int cat;
        private double dbl_price;
        private double dbl_quantity;
        private DateTime data_last;
        private Unita unita_mis;
        private string str_ean = "";
        private double dbl_aliquota;
        private bool fiscal;
        private bool bool_new;
        private bool modified;

        public event EventHandler FiscaleChanged;

        /// <summary>
        /// Unit of measurement
        /// </summary>
        public enum Unita{
            pezzo = 1,
            peso = 2
        };

        public int Id{

            get{
                return int_id;
            }

        }

        public string Name{

            get{
                return str_name;
            }
            set{
                if(!str_name.Equals(value)){
                    str_name = value;
                    modified = true;
                }
            }

        }

        public int Categoria{

            get{
                return cat;
            }
            set{
                if(cat!=value){
                    cat = value;
                    modified = true;
                }
            }

        }

        public double Price{

            get{
                return dbl_price;
            }
            set{
                if(dbl_price!=value){
                    dbl_price = value;
                    modified = true;
                }
            }

        }

        public double Quantity{

            get{
                return dbl_quantity;
            }
            set{
                if(value!=0){
                    dbl_quantity += value;
                    modified = true;
                }
            }

        }

        public DateTime LastModified{

            get{
                return data_last;
            }
            set{
                data_last = value;
            }

        }

        public bool HasChanged{

            get{
                return modified;
            }
            set{
                modified = value;
            }

        }

        public Unita Misura{

            get{
                return unita_mis;
            }
            set{
                unita_mis = value;
            }

        }

        public string EAN{

            get{
                return str_ean;
            }
            set{
                if(!str_ean.Equals(value)){
                    str_ean = value;
                    modified = true;
                }
            }

        }

        public double Aliquota{

            get{
                return dbl_aliquota;
            }
            set{
                if(dbl_aliquota!=value){
                    dbl_aliquota = value;
                    modified = true;
                }
            }

        }

        public bool Fiscale{
            get{
                return fiscal;
            }
            set{
                if(fiscal!=value){
                    fiscal = value;
                    modified = true;
                    FiscaleChanged(this, new EventArgs());
                }
            }

        }

        public bool IsNew{

            get{
                return bool_new;
            }

        }

        public Product(){

            int_id = ((int)((IDb)Engine.DbService).GetData("SELECT COALESCE(max(products_id),0) FROM products", 0, 0).Tables[0].Rows[0][0]) + 1;

            bool_new = true;

            modified = false;

            fiscal = true;

            dbl_quantity = 0;

            dbl_aliquota = 20;

            data_last = DateTime.Today;

        }

        /// <summary>
        /// Product constructor from id
        /// </summary>
        /// <param name="idprod">pid</param>
        public Product(int idprod){

            DataTable m_ds = ((IProducts)Engine.DbService).GetProd(idprod);

            if (m_ds != null && m_ds.Rows.Count > 0)
            {
                int_id = idprod;
                str_name = Convert.ToString(m_ds.Rows[0]["products_model"]);
                str_ean = Convert.ToString(m_ds.Rows[0]["products_ean"]);
                dbl_price = Convert.ToDouble(m_ds.Rows[0]["products_price"]);
                cat = Convert.ToInt32(m_ds.Rows[0]["categories_id"]);
                fiscal = Convert.ToBoolean(m_ds.Rows[0]["fiscale"]);
                dbl_aliquota = Convert.ToDouble(m_ds.Rows[0]["aliquota"]);
                dbl_quantity = Convert.ToDouble(m_ds.Rows[0]["products_quantity"]);
                data_last = Convert.ToDateTime(m_ds.Rows[0]["last_modified"]);
                unita_mis = Unita.pezzo;
                bool_new = false;
                modified = false;
            }else{
                throw new ArgumentException("Product not found");
            }

        }

        /// <summary>
        /// Product constructor from ean code
        /// </summary>
        /// <param name="ean">ean code</param>
        public Product(string ean){

            DataTable m_ds = ((IProducts)Engine.DbService).GetProdEan(ean);

            if(m_ds!=null && m_ds.Rows.Count > 0){

                int_id = Convert.ToInt32(m_ds.Rows[0]["products_id"]);
                str_name = Convert.ToString (m_ds.Rows[0]["products_model"]);
                str_ean = ean;
                dbl_price = Convert.ToDouble(m_ds.Rows[0]["products_price"]);
                cat = Convert.ToInt32(m_ds.Rows[0]["categories_id"]);
                fiscal = Convert.ToBoolean(m_ds.Rows[0]["fiscale"]);
                dbl_aliquota = Convert.ToDouble(m_ds.Rows[0]["aliquota"]);
                dbl_quantity = Convert.ToDouble(m_ds.Rows[0]["products_quantity"]);
                data_last = Convert.ToDateTime(m_ds.Rows[0]["last_modified"]);
                unita_mis = Unita.pezzo;
                bool_new = false;
                modified = false;
            }else{
                throw new ArgumentException("Product not found");
            }

        }

        /// <summary>
        /// Save product in the db
        /// </summary>
        /// <returns></returns>
        public int Save(){

            if(modified){
                return ((IProducts)Engine.DbService).SaveProd(bool_new, int_id, str_name, str_ean, dbl_price, dbl_quantity, fiscal, cat, dbl_aliquota);
            }
            return 0;
        }

        public int Delete(){

            if(((IDb)Engine.DbService).GetData("select * from products where categories_id=" + int_id, 0, 0).Tables[0].Rows.Count > 0){
                throw new Exception(resources.GetString("msgdepterror"));
            }else{
                return ((IDb)Engine.DbService).SaveData("DELETE FROM products WHERE products_id = " + int_id);
            }

        }

#if CONFIG_NETWORK

        //public Shared Function Sync(ByVal peer As String) As Integer

        //    Dim rec As String = get{Peer(peer)

        //    If Not rec Is Nothing Then
        //        Dim ret As Microsoft.Synchronization.SyncOperationStatistics

        //        Try
        //            ret = Engine.DbService.Synchronize("net.tcp://" & rec & ":9000/sync", "products", Microsoft.Synchronization.SyncDirectionOrder.Download)
        //            return ret.DownloadChangesApplied
        //        Catch ex As Exception

        //        End Try

        //    End If

        //    return 0

        //End Function

        public static int CloneSpeed(string peer){

            string rec = GetPeer(peer);

            if(rec!=null){
                return ((IProducts)Engine.DbService).CloneSpeed("net.tcp://" + rec + ":9000/prod");
            }

            return 0;

        }

#endif

        public static DataTable GetProd(string ean){

            return ((IProducts)Engine.DbService).GetProdEan(ean);

        }

        public static DataTable GetAll(int page, int mode){

            return ((IProducts)Engine.DbService).GetProds(page, mode).Tables[0];

        }

        public static DataTable GetRep(){
            return ((IDb)Engine.DbService).GetData("SELECT products_id, products_model FROM products WHERE categories_id=0 ORDER BY products_model", 0, 0).Tables[0];
        }

        public static DataTable GetCombo(){
            return ((IDb)Engine.DbService).GetData("SELECT products_id as id, products_model + '  " + System.Globalization.NumberFormatInfo.CurrentInfo.CurrencySymbol + " ' + CAST(products_price/100 as nvarchar) as prodotto FROM products", 0, 0).Tables[0];
        }

        public static int CheckName(string name){
            return ((IDb)Engine.DbService).GetData("SELECT * from products where products_model like '" + name + "'", 0, 0).Tables[0].Rows.Count;
        }

        public static int CheckCode(string ean){
            return ((IDb)Engine.DbService).GetData("SELECT * from products where products_ean like '" + ean + "'", 0, 0).Tables[0].Rows.Count;
        }

        public override string ToString(){
            return this.str_name;
        }

    }

}
