﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Data;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using POS.Framework;
using POS.Interfaces;
using POS.ProductPlugin.Objects;

namespace POS.ProductPlugin {

    public class ProductPlugin : IPlugin{

        internal class PanelNoScrollOnFocus : Panel
        {

            protected override Point ScrollToControl(Control activeControl)
            {
                return base.DisplayRectangle.Location;
            }

        }

        System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ProductPlugin));
        internal Label lblModal;
        internal RadioButton ModalMod;
        internal RadioButton ModalQty;
        internal RadioButton ModalSearch;
        internal Label lblSearch;
        internal TextBox txtSearch;
        internal Button btnClearSearch;
        internal Button btnProdDel;
        internal Button btnProdMod;
        internal Button btnProdAdd;
        internal Panel pnlFake;
        internal PanelNoScrollOnFocus pnlProducts;
        internal Button btnHeaderProd;
        internal Button btnHeaderEan;
        internal Button btnHeaderPrice;
        internal Button btnHeaderVat;
        internal Button btnHeaderFis;
        internal Button btnHeaderQty;
        internal DataGridView dgdProducts;
        internal DataGridViewTextBoxColumn ColId;
        internal DataGridViewTextBoxColumn ColEan;
        internal DataGridViewCheckBoxColumn ColFiscal;
        internal DataGridViewTextBoxColumn ColVat;
        internal DataGridViewTextBoxColumn ColQty;
        internal DataGridViewTextBoxColumn ColProd;
        internal DataGridViewTextBoxColumn ColPrice;
        internal DataGridViewTextBoxColumn ColDep;

        [DllImport("user32.dll")]
        static extern bool CreateCaret(IntPtr hWnd, IntPtr hBitmap, int nWidth, int nHeight);
        [DllImport("user32.dll")]
        static extern bool ShowCaret(IntPtr hWnd);

        private void ShowCustomCaret(IntPtr h, int width, int height){
            CreateCaret(h,(IntPtr)0, width, height);
            ShowCaret(h);
        }

        //
        //local variables
        //
        private ProductWindow wizard = new ProductWindow();
        private int scanmode = 0;

        public enum ListMode
        {
            Departments=1,
            Alphabetic=2,
            TextSearch=3,
            EanSearch=4
        }
        private ListMode currentmode=ListMode.Departments;
        private int currentpage=0;
        internal DataTable data1;
        

        // Not implemented
        public int Message(int func, object data){

            return -1;

        }

        public void Install(){
    
            if(Engine.HasAccess(resources.GetString("Stock_Install"))){
    
            }

        }

        public void Uninstall(){
    
            if(Engine.HasAccess(resources.GetString("Stock_Uninstall"))){
    
            }
    
        }

        public int Initialize(TabPage container){

            if (Engine.HasAccess(resources.GetString("Stock_Load")))
            {

            this.lblModal = new Label();
            this.ModalMod = new RadioButton();
            this.ModalQty = new RadioButton();
            this.ModalSearch = new RadioButton();
            this.lblSearch = new Label();
            this.txtSearch = new TextBox();
            this.btnClearSearch = new Button();
            this.btnProdDel = new Button();
            this.btnProdMod = new Button();
            this.btnProdAdd = new Button();
            this.pnlFake = new Panel();
            this.pnlProducts = new PanelNoScrollOnFocus();
            this.btnHeaderProd = new Button();
            this.btnHeaderEan = new Button();
            this.btnHeaderPrice = new Button();
            this.btnHeaderVat = new Button();
            this.btnHeaderFis = new Button();
            this.btnHeaderQty = new Button();
            this.dgdProducts = new DataGridView();
            this.ColEan = new DataGridViewTextBoxColumn();
            this.ColFiscal = new DataGridViewCheckBoxColumn();
            this.ColId = new DataGridViewTextBoxColumn();
            this.ColVat = new DataGridViewTextBoxColumn();
            this.ColQty = new DataGridViewTextBoxColumn();
            this.ColProd = new DataGridViewTextBoxColumn();
            this.ColPrice = new DataGridViewTextBoxColumn();
            this.ColDep = new DataGridViewTextBoxColumn();
            //
            //TabProdotti
            //
            container.Controls.Add(this.lblModal);
            container.Controls.Add(this.ModalMod);
            container.Controls.Add(this.ModalQty);
            container.Controls.Add(this.ModalSearch);
            container.Controls.Add(this.lblSearch);
            container.Controls.Add(this.txtSearch);
            container.Controls.Add(this.btnClearSearch);
            container.Controls.Add(this.btnProdDel);
            container.Controls.Add(this.btnProdMod);
            container.Controls.Add(this.btnProdAdd);
            container.Controls.Add(this.pnlFake);
            container.Location = new Point(4, 64);
            container.Text = resources.GetString("container.Text");
            container.MouseClick+=new MouseEventHandler(container_MouseClick);
            //
            //btnProdAdd
            //
            this.btnProdAdd.BackColor = Color.White;
            this.btnProdAdd.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnProdAdd.Location = new Point(21, 48);
            this.btnProdAdd.Name = "btnProdAdd";
            this.btnProdAdd.Size = new Size(116, 54);
            this.btnProdAdd.TabIndex = 37;
            this.btnProdAdd.Text = resources.GetString("btnProdAdd.Text");
            this.btnProdAdd.UseVisualStyleBackColor = false;
            this.btnProdAdd.Click += new EventHandler(btnProdAdd_Click);
            //
            //btnProdMod
            //
            this.btnProdMod.BackColor = Color.White;
            this.btnProdMod.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnProdMod.Location = new Point(143, 48);
            this.btnProdMod.Name = "btnProdMod";
            this.btnProdMod.Size = new System.Drawing.Size(116, 54);
            this.btnProdMod.TabIndex = 38;
            this.btnProdMod.Text = resources.GetString("btnProdMod.Text");
            this.btnProdMod.UseVisualStyleBackColor = false;
            this.btnProdMod.Click += new EventHandler(btnProdMod_Click);
            //
            //btnProdDel
            //
            this.btnProdDel.BackColor = Color.IndianRed;
            this.btnProdDel.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnProdDel.Location = new Point(265, 48);
            this.btnProdDel.Name = "btnProdDel";
            this.btnProdDel.Size = new Size(116, 54);
            this.btnProdDel.TabIndex = 41;
            this.btnProdDel.Text = resources.GetString("btnProdDel.Text");
            this.btnProdDel.UseVisualStyleBackColor = false;
            this.btnProdDel.Click += new EventHandler(btnProdDel_Click);
            //
            //lblSearch
            //
            this.lblSearch.AutoSize = true;
            this.lblSearch.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.lblSearch.Location = new Point(840, 44);
            this.lblSearch.Name = "lblSearch";
            this.lblSearch.Text = resources.GetString("lblSearch.Text");
            //
            //txtSearch
            //
            this.txtSearch.CharacterCasing = CharacterCasing.Upper;
            this.txtSearch.Font = new Font("Microsoft Sans Serif", 20.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.txtSearch.Location = new Point(823, 72);
            this.txtSearch.Multiline = false;
            this.txtSearch.Name = "txtSearch";
            this.txtSearch.Size = new Size(150, 35);
            this.txtSearch.WordWrap = false;
            this.txtSearch.Click +=new EventHandler(txtSearch_Click);
            this.txtSearch.GotFocus += new EventHandler(txtSearch_GotFocus);
            this.txtSearch.KeyPress+=new KeyPressEventHandler(txtSearch_KeyPress);
            this.txtSearch.Leave += new EventHandler(txtSearch_Leave);
            this.txtSearch.TextChanged += new EventHandler(txtSearch_TextChanged);
            //
            //btnClearSearch
            //
            this.btnClearSearch.BackColor = Color.White;
            this.btnClearSearch.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnClearSearch.Location = new Point(974, 72);
            this.btnClearSearch.Name = "btnClearSearch";
            this.btnClearSearch.Size = new System.Drawing.Size(30, 30);
            this.btnClearSearch.Text = "X";
            this.btnClearSearch.UseVisualStyleBackColor = false;
            this.btnClearSearch.Click += new EventHandler(btnClearSearch_Click);
            //
            //lblModal
            //
            this.lblModal.AutoSize = true;
            this.lblModal.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.lblModal.Location = new Point(840, 137);
            this.lblModal.Name = "lblModal";
            this.lblModal.TabIndex = 7;
            this.lblModal.Text = resources.GetString("lblModal.Text");
            //
            //ModalMod
            //
            this.ModalMod.Appearance = Appearance.Button;
            this.ModalMod.AutoSize = false;
            this.ModalMod.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.ModalMod.Image = (Image)resources.GetObject("boxmod");
            this.ModalMod.ImageAlign = ContentAlignment.TopLeft;
            this.ModalMod.Location = new Point(850, 165);
            this.ModalMod.Name = "ModalMod";
            this.ModalMod.Size = new Size(121, 30);
            this.ModalMod.TabIndex = 1;
            this.ModalMod.Text = resources.GetString("ModalMod.Text");
            this.ModalMod.TextAlign = ContentAlignment.MiddleLeft;
            this.ModalMod.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageBeforeText;
            this.ModalMod.UseVisualStyleBackColor = false;
            this.ModalMod.CheckedChanged += new EventHandler(ModalMod_CheckedChanged);
            //
            //ModalQty
            //
            this.ModalQty.Appearance = Appearance.Button;
            this.ModalQty.AutoSize = false;
            this.ModalQty.Font = new System.Drawing.Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.ModalQty.Image = (Image)resources.GetObject("boxcar");
            this.ModalQty.ImageAlign = ContentAlignment.TopLeft;
            this.ModalQty.Location = new Point(850, 201);
            this.ModalQty.Name = "ModalQty";
            this.ModalQty.Size = new Size(121, 30);
            this.ModalQty.TabIndex = 2;
            this.ModalQty.Text = resources.GetString("ModalQty.Text");
            this.ModalQty.TextAlign = ContentAlignment.MiddleLeft;
            this.ModalQty.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.ModalQty.UseVisualStyleBackColor = false;
            this.ModalQty.CheckedChanged += new EventHandler(ModalQty_CheckedChanged);
            //
            //ModalSearch
            //
            this.ModalSearch.Appearance = Appearance.Button;
            this.ModalSearch.AutoSize = false;
            this.ModalSearch.Font = new Font("Microsoft Sans Serif", 16.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.ModalSearch.Image = (Image)resources.GetObject("boxsearch");
            this.ModalSearch.ImageAlign = ContentAlignment.TopLeft;
            this.ModalSearch.Location = new Point(850, 237);
            this.ModalSearch.Name = "ModalSearch";
            this.ModalSearch.Size = new Size(121, 30);
            this.ModalSearch.TabIndex = 3;
            this.ModalSearch.Text = resources.GetString("ModalSearch.Text");
            this.ModalSearch.TextAlign = ContentAlignment.MiddleLeft;
            this.ModalSearch.TextImageRelation = TextImageRelation.ImageBeforeText;
            this.ModalSearch.UseVisualStyleBackColor = false;
            this.ModalSearch.CheckedChanged += new EventHandler(ModalSearch_CheckedChanged);
            //
            //pnlFake
            //
            this.pnlFake.AutoScroll = false;
            this.pnlFake.AutoScrollMargin = new Size(0, 0);
            this.pnlFake.BorderStyle = BorderStyle.FixedSingle;
            this.pnlFake.Controls.Add(this.btnHeaderQty);
            this.pnlFake.Controls.Add(this.btnHeaderFis);
            this.pnlFake.Controls.Add(this.btnHeaderVat);
            this.pnlFake.Controls.Add(this.btnHeaderPrice);
            this.pnlFake.Controls.Add(this.btnHeaderEan);
            this.pnlFake.Controls.Add(this.btnHeaderProd);
            this.pnlFake.Controls.Add(this.pnlProducts);
            this.pnlFake.Name = "pnlFake";
            this.pnlFake.Size = new Size(777, 572);
            this.pnlFake.Location = new Point(21, 108);
            //
            //btnHeaderProd
            //
            this.btnHeaderProd.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderProd.Name = "btnHeaderRep";
            this.btnHeaderProd.BackColor = SystemColors.Control;
            this.btnHeaderProd.UseVisualStyleBackColor = false;
            this.btnHeaderProd.Location = new Point(0, 0);
            this.btnHeaderProd.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderProd.Text = resources.GetString("btnHeaderProd2.Text");
            this.btnHeaderProd.Size = new Size(294, 30);
            this.btnHeaderProd.Click += new EventHandler(btnHeaderProd_Click);
            //
            //btnHeaderEan
            //
            this.btnHeaderEan.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderEan.Name = "btnHeaderEan";
            this.btnHeaderEan.BackColor = SystemColors.Control;
            this.btnHeaderEan.UseVisualStyleBackColor = false;
            this.btnHeaderEan.Location = new Point(293, 0);
            this.btnHeaderEan.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderEan.Text = resources.GetString("btnHeaderEan.Text");
            this.btnHeaderEan.Size = new Size(171, 30);
            //
            //btnHeaderPrice
            //
            this.btnHeaderPrice.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderPrice.Name = "btnHeaderPrice";
            this.btnHeaderPrice.BackColor = System.Drawing.SystemColors.Control;
            this.btnHeaderPrice.UseVisualStyleBackColor = false;
            this.btnHeaderPrice.Location = new Point(463, 0);
            this.btnHeaderPrice.Font = new System.Drawing.Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderPrice.Text = resources.GetString("btnHeaderPrice.Text");
            this.btnHeaderPrice.Size = new Size(121, 30);
            //
            //btnHeaderVat
            //
            this.btnHeaderVat.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderVat.Name = "btnHeaderVat";
            this.btnHeaderVat.BackColor = SystemColors.Control;
            this.btnHeaderVat.UseVisualStyleBackColor = false;
            this.btnHeaderVat.Location = new Point(583, 0);
            this.btnHeaderVat.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderVat.Text = resources.GetString("btnHeaderVat.Text");
            this.btnHeaderVat.Size = new Size(43, 30);
            //
            //btnHeaderFis
            //
            this.btnHeaderFis.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderFis.Name = "btnHeaderFis";
            this.btnHeaderFis.BackColor = SystemColors.Control;
            this.btnHeaderFis.UseVisualStyleBackColor = false;
            this.btnHeaderFis.Location = new Point(625, 0);
            this.btnHeaderFis.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderFis.Text = resources.GetString("btnHeaderFis.Text");
            this.btnHeaderFis.Size = new Size(51, 30);
            //
            //btnHeaderQty
            //
            this.btnHeaderQty.TextAlign = ContentAlignment.MiddleCenter;
            this.btnHeaderQty.Name = "btnHeaderQty";
            this.btnHeaderQty.BackColor = SystemColors.Control;
            this.btnHeaderQty.UseVisualStyleBackColor = false;
            this.btnHeaderQty.Location = new Point(675, 0);
            this.btnHeaderQty.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.btnHeaderQty.Text = resources.GetString("btnHeaderQty.Text");
            this.btnHeaderQty.Size = new Size(101, 30);
            //
            //pnlProducts
            //
            this.pnlProducts.AutoScroll = true;
            this.pnlProducts.AutoScrollMargin = new Size(0, 0);
            this.pnlProducts.BorderStyle = BorderStyle.None;
            this.pnlProducts.Controls.Add(dgdProducts);
            this.pnlProducts.Name = "pnlProducts";
            this.pnlProducts.Size = new Size(795, 541); //2 pixels for visual styles
            this.pnlProducts.Location = new Point(0, 29);
            //
            //dgdProducts
            //
            this.dgdProducts.AllowUserToAddRows = false;
            this.dgdProducts.AllowUserToResizeColumns = false;
            this.dgdProducts.AllowUserToResizeRows = false;
            this.dgdProducts.AllowUserToOrderColumns = false;
            this.dgdProducts.AutoGenerateColumns = false;
            this.dgdProducts.BorderStyle = BorderStyle.None;
            this.dgdProducts.ColumnHeadersVisible = false;
            this.dgdProducts.Columns.AddRange(new DataGridViewColumn[] {this.ColId, this.ColProd, this.ColEan, this.ColPrice, this.ColVat, this.ColFiscal, this.ColQty, this.ColDep});
            //////////////////////////////////////////////////////
            //DefaultCellStyle
            /////////////////////////////////////////////////////
            this.dgdProducts.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            this.dgdProducts.DefaultCellStyle.BackColor = SystemColors.Window;
            this.dgdProducts.DefaultCellStyle.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.dgdProducts.DefaultCellStyle.ForeColor = SystemColors.ControlText;
            this.dgdProducts.DefaultCellStyle.SelectionBackColor = Color.Lime;
            this.dgdProducts.DefaultCellStyle.SelectionForeColor = Color.Black;
            this.dgdProducts.DefaultCellStyle.WrapMode = DataGridViewTriState.False;
            //////////////////////////////////////////////////////
            //EndDefaultCellStyle
            /////////////////////////////////////////////////////
            this.dgdProducts.EnableHeadersVisualStyles = false;
            this.dgdProducts.Location = new Point(0, 0);
            this.dgdProducts.MultiSelect = false;
            this.dgdProducts.Name = "dgdProducts";
            this.dgdProducts.ReadOnly = true;
            this.dgdProducts.RowHeadersVisible = false;
            this.dgdProducts.RowHeadersWidthSizeMode = DataGridViewRowHeadersWidthSizeMode.DisableResizing;
            this.dgdProducts.RowTemplate.Height = 30;
            this.dgdProducts.RowTemplate.Resizable = DataGridViewTriState.False;
            this.dgdProducts.ScrollBars = ScrollBars.None;
            this.dgdProducts.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            this.dgdProducts.Size = new Size(777, 1620);
            this.dgdProducts.TabStop = false;
            this.dgdProducts.CellFormatting+=new DataGridViewCellFormattingEventHandler(dgdProducts_CellFormatting);
            this.dgdProducts.MouseDown+=new MouseEventHandler(dgdProducts_MouseDown);
            this.dgdProducts.MouseMove+=new MouseEventHandler(dgdProducts_MouseMove);
            this.dgdProducts.MouseUp+=new MouseEventHandler(dgdProducts_MouseUp);
            //
            //ColEan
            //
            this.ColEan.DataPropertyName = "Ean";
            this.ColEan.Name = "ColEan";
            this.ColEan.Resizable = DataGridViewTriState.False;
            this.ColEan.Visible = true;
            this.ColEan.Width = 170;
            this.ColEan.SortMode = DataGridViewColumnSortMode.NotSortable;
            //
            //ColFiscal
            //
            this.ColFiscal.DataPropertyName = "Fiscale";
            this.ColFiscal.Name = "ColFiscal";
            this.ColFiscal.Resizable = DataGridViewTriState.False;
            this.ColFiscal.Visible = true;
            this.ColFiscal.Width = 50;
            this.ColFiscal.SortMode = DataGridViewColumnSortMode.NotSortable;
            //
            //ColId
            //
            this.ColId.DataPropertyName = "Id";
            this.ColId.Name = "ColId";
            this.ColId.Resizable = DataGridViewTriState.False;
            this.ColId.Visible = false;
            this.ColId.SortMode = DataGridViewColumnSortMode.NotSortable;
            //
            //ColQty
            //
            this.ColQty.DataPropertyName = "Qta";
            this.ColQty.Name = "ColQty";
            this.ColQty.Resizable = DataGridViewTriState.False;
            this.ColQty.Width = 100;
            this.ColQty.SortMode = DataGridViewColumnSortMode.NotSortable;
            //
            //ColVat
            //
            this.ColVat.DataPropertyName = "Iva";
            this.ColVat.Name = "ColVat";
            this.ColVat.Resizable = DataGridViewTriState.False;
            this.ColVat.Width = 42;
            this.ColVat.SortMode = DataGridViewColumnSortMode.NotSortable;
            //
            //ColProd
            //
            this.ColProd.DataPropertyName = "Prodotto";
            this.ColProd.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
            this.ColProd.DefaultCellStyle.Font = new Font("Microsoft Sans Serif", 18.0F, FontStyle.Bold, GraphicsUnit.Pixel, ((Byte)(0)));
            this.ColProd.Name = "ColProd";
            this.ColProd.Resizable = DataGridViewTriState.False;
            this.ColProd.Width = 292;
            this.ColProd.SortMode = DataGridViewColumnSortMode.Programmatic;
            //
            //ColPrice
            //
            this.ColPrice.DataPropertyName = "Prezzo";
            this.ColPrice.DefaultCellStyle.Format = "c";
            this.ColPrice.Name = "ColPrice";
            this.ColPrice.Resizable = DataGridViewTriState.False;
            this.ColPrice.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColPrice.Width = 120;
            //
            //ColDep
            //
            this.ColDep.DataPropertyName = "Rep";
            this.ColDep.HeaderText = "RepOrder";
            this.ColDep.Name = "ColDep";
            this.ColDep.Resizable = DataGridViewTriState.False;
            this.ColDep.SortMode = DataGridViewColumnSortMode.NotSortable;
            this.ColDep.Visible = false;
            this.ColDep.Width = 53;

            return 0;

            }

            return 1;

        }

        public void SetProperties(){

            if (Engine.HasAccess(resources.GetString("Stock_Settings")))
            {
    
            }

        }

        //FillDataGrid in the initialize function creates problems.
        //this way as a side effect the page is requeried and the selected row is reset
        //every time the tab is entered
        public void TabEnter(){

            FillDataGrid(currentpage, currentmode, null);

            this.ModalMod.Checked = true;

        }

        public void PosData(object dev, string data)
        {

            if (Engine.HasAccess(resources.GetString("Stock_AddProduct")) && !wizard.Visible)
            {

            switch (scanmode)
            {

                case 0:
                    /////////////////////////////////////
                    //Product Mode
                    /////////////////////////////////////
                    try
                    {
                        wizard.prod = new Product(data);
                    }
                    catch (ArgumentException ex)
                    {
                        //barcode not found, create new product
                        wizard.prod = new Product();
                        wizard.prod.EAN = data;
                    }

                    if (wizard.ShowDialog() == DialogResult.OK)
                    {
                        FillDataGrid(currentpage, currentmode, null);
                    }
                    break;
                case 1:
                    /////////////////////////////////////
                    //Stock Mode
                    /////////////////////////////////////

                    try
                    {
                        wizard.prod = new Product(data);
                    }
                    catch (ArgumentException ex)
                    {
                        //barcode not found
                        return;
                    }

                    wizard.tabProdotto.SelectedIndex = 5;

                    if (wizard.ShowDialog() == DialogResult.OK)
                    {
                        FillDataGrid(currentpage, currentmode, null);
                    }
                    break;
                case 2:
                    /////////////////////////////////////
                    //Search product mode
                    /////////////////////////////////////
                    FillDataGrid(0, ListMode.EanSearch, data);

                    break;
            }

            }

        }

        /// <summary>
        /// Fills DataGridView with products and pages data
        /// </summary>
        public void FillDataGrid(int page, ListMode mode, object param){

            if (Engine.HasAccess(resources.GetString("Stock_View")))
            {
    
            DataTable temp=null, temp2=null;
            object[] r=null;
            
            if (mode == ListMode.Departments)
            {
                r = new Object[] { null, null, null, null, null, null, null, null, null };
                temp = Product.GetAll(page == 0 ? 0 : page - 1, -1);

                btnHeaderProd.Text = resources.GetString("btnHeaderProd2.Text");

                // reset text search
                txtSearch.Text = "";
            }
            else if(mode == ListMode.Alphabetic)
            {
                r = new Object[] { null, null, null, null, null, null, null, null };
                temp = Product.GetAll(page == 0 ? 0 : page - 1, 1);

                btnHeaderProd.Text = resources.GetString("btnHeaderProd.Text");

                // reset text search
                txtSearch.Text = "";
            }
            else if (mode == ListMode.TextSearch)
            {
                if (param == null) return;

                r = new Object[] { null, null, null, null, null, null, null, null };
                temp = ((IDb)Engine.DbService).GetData("SELECT products_id as id, products_model as prodotto, products_price/100 as prezzo, aliquota as iva, products_quantity as qta, categories_id as rep, products_ean as ean, fiscale FROM products WHERE products_model like '%" + (string)param + "%' ORDER BY prodotto", (page == 0 ? 0 : page - 1) * 18, 54).Tables[0];

                // clearly signal that there are no results
                if (temp.Rows.Count == 0) temp.Rows.Add(new Object[] { null, resources.GetString("NoResultstxt"), null, null, null, null, null, null });

                btnHeaderProd.Text = resources.GetString("btnHeaderSearch.Text");

            }
            else if (mode == ListMode.EanSearch)
            {
                if (param == null) return;

                r = new Object[] { null, null, null, null, null, null, null, null };
                temp = ((IDb)Engine.DbService).GetData("SELECT products_id as id, products_model as prodotto, products_price/100 as prezzo, aliquota as iva, products_quantity as qta, categories_id as rep, products_ean as ean, fiscale FROM products WHERE products_ean='" + (string)param + "' ORDER BY prodotto", (page == 0 ? 0 : page - 1) * 18, 54).Tables[0];

                // clearly signal that there are no results
                if (temp.Rows.Count == 0) temp.Rows.Add(new Object[] { null, resources.GetString("NoResultstxt"), null, null, null, null, null, null });

                btnHeaderProd.Text = resources.GetString("btnHeaderSearch.Text");

            }
            
            // add empty buffer lines for page -1
            if(page == 0){
                
                temp2 = temp.Clone();

                for(int i=1;i<=18;i++) temp2.Rows.Add(r);

                temp2.Merge(temp, true, MissingSchemaAction.Add);

                temp = temp2;

            }
            
            temp.ExtendedProperties.Add("mode", (int)mode);
            temp.ExtendedProperties.Add("page", (int)page);
            temp.ExtendedProperties.Add("param", param);

            //save current mode and page
            currentmode = mode;
            currentpage = page;
            
            dgdProducts.DataSource = temp;

            // store product dataset to restore
            if (mode == ListMode.Departments || mode == ListMode.Alphabetic) data1 = temp;

            if (dgdProducts.Rows.Count > 18)
            {
                dgdProducts.Rows[18].Selected = true;
            }

            pnlProducts.AutoScrollPosition = new Point(0, 540);

            }

        }

        /// <summary>
        /// Switch dataset with all products back
        /// </summary>
        private void RestoreTable(){

            // reload also the table to avoid bug after product modified or deleted
            FillDataGrid((int)data1.ExtendedProperties["page"], (ListMode)data1.ExtendedProperties["mode"], null);

        }


        private void btnProdAdd_Click(object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Stock_AddProduct")))
            {

            if(wizard.ShowDialog() == DialogResult.OK){
                //reset everything
                FillDataGrid(0, ListMode.Departments, null);
            }

            }

        }

        private void btnProdMod_Click(Object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Stock_ModifyProduct")))
            {

            //seems like currentrow is different from selected row
            //don't know why it has worked so far.
            //wizard.prod = new Product((int)(dgdProducts.CurrentRow.Cells[0].Value));
            try
            {
                // catch no row selected exception
                wizard.prod = new Product((int)(dgdProducts.SelectedRows[0].Cells[0].Value));
            }catch(Exception ex){
                return;
            }
            
            if(wizard.ShowDialog() == DialogResult.OK){
                //reload current page don't go to page 0
                FillDataGrid(currentpage, currentmode, ((DataTable)dgdProducts.DataSource).ExtendedProperties["param"]);
            }

            }

        }

        private void btnProdDel_Click(object sender, EventArgs e){

            if (Engine.HasAccess(resources.GetString("Stock_DeleteProduct")))
            {

            PopBox pop = new PopBox();
            Product pr;

            //seems like currentrow is different from selected row
            //don't know why it has worked so far.
            //Product pr = new Product((int)(dgdProducts.CurrentRow.Cells[0].Value));
            try
            {
                // catch no row selected exception
                pr = new Product((int)(dgdProducts.SelectedRows[0].Cells[0].Value));
            }
            catch (Exception ex)
            {
                return;
            } 

            if(pop.ShowDialog(resources.GetString("msgproductremtxt"), new Point(465, 300)) == DialogResult.Yes){

                try{
                    pr.Delete();
                    //reload current page don't go to page 0
                    FillDataGrid(currentpage, currentmode, ((DataTable)dgdProducts.DataSource).ExtendedProperties["param"]);
                }catch(Exception ex){
                    pop.ShowDialog(ex.Message, new Point(465, 300), MessageBoxButtons.OK);
                }

            }

            }

        }

        private void ModalMod_CheckedChanged(object sender, EventArgs e){
            if(ModalMod.Checked) scanmode = 0;
        }

        private void ModalQty_CheckedChanged(object sender, EventArgs e){
            if(ModalQty.Checked) scanmode = 1;
        }

        private void ModalSearch_CheckedChanged(object sender, EventArgs e){
            if (ModalSearch.Checked) scanmode = 2;
            else {
                if (currentmode == ListMode.EanSearch)
                {
                    // switch dataset with all products back
                    RestoreTable();
                }
            }
        }

        #region Text Search
        /// <summary>
        /// Products text search
        /// </summary>
        private void txtSearch_Click(object sender, EventArgs e)
        {
            Engine.ShowKeyboard();
            txtSearch.Focus();
        }

        private void txtSearch_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtSearch.Handle, 4, 29);
        }

        // keyboard can be hidden by clicking outside control
        // or by pressing enter, even if search is performed as you type.
        private void txtSearch_Leave(object sender, EventArgs e)
        {
            Engine.HideKeyboard();
        }

        private void container_MouseClick(object sender, MouseEventArgs e)
        {
            txtSearch.FindForm().ActiveControl = null;
        }

        private void txtSearch_KeyPress(object sender, KeyPressEventArgs e)
        {
            // use same character set allowed for products name
            if (Char.IsPunctuation(e.KeyChar) || (Char.IsWhiteSpace(e.KeyChar) && txtSearch.TextLength == 0))
            {
                e.Handled = true;
            }
            if (e.KeyChar==(char)13)
            {
                Engine.HideKeyboard();
            }
        }

        private void txtSearch_TextChanged(object sender, EventArgs e)
        {
            // switch dataset with all products back
            if (txtSearch.TextLength == 0)
            {
                if(currentmode == ListMode.TextSearch){
                    // switch dataset with all products back
                    RestoreTable();
                }
                
                return;
            }

            FillDataGrid(0, ListMode.TextSearch, txtSearch.Text);
            
        }

        private void btnClearSearch_Click(object sender, EventArgs e)
        {
            if (txtSearch.TextLength > 0 && currentmode==ListMode.TextSearch)
            {
                // switch dataset with all products back
                RestoreTable();
            }

            txtSearch.Text = "";

        }

        #endregion

        /// <summary>
        /// Switch between alphabetical view and department view
        /// </summary>
        private void btnHeaderProd_Click(object sender, EventArgs e)
        {
            if (currentmode == ListMode.Alphabetic)
            {
                //fills the product list with the first page but ordered by departments
                FillDataGrid(0, ListMode.Departments, null);
            }
            else if (currentmode == ListMode.Departments)
            {
                //fills the product list with the first page but ordered alphabetically
                FillDataGrid(0, ListMode.Alphabetic, null);
            }
            else
            {
                //search modes do nothing
            }
            
        }

        /// <summary>
        /// Color department rows
        /// </summary>
        private void dgdProducts_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            if (e.ColumnIndex == 3)
            {
                if (!((dgdProducts.Rows[e.RowIndex].Cells["ColDep"].Value) == DBNull.Value))
                {
                    if ((int)dgdProducts.Rows[e.RowIndex].Cells["ColDep"].Value == 0) dgdProducts.Rows[e.RowIndex].DefaultCellStyle.BackColor = Color.LightBlue;
                }
                e.FormattingApplied = false;
            }
        }

        #region Scroll

        private Point mouse_down;
        private Rectangle dragBoxFromMouseDown;
        private bool switched;
        private Point temp;

        
        private void dgdProducts_MouseDown(object sender, MouseEventArgs e){
            mouse_down = new Point(e.X, e.Y);
            temp = pnlProducts.AutoScrollPosition;
            switched = false;
            Size dragSize = new Size(100, 250);
            dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
        }

        private void dgdProducts_MouseMove(object sender, MouseEventArgs e){

            if(e.Button == MouseButtons.Left && !switched){

                if(!(Math.Abs(temp.Y - pnlProducts.AutoScrollPosition.Y) > dragBoxFromMouseDown.Height)){
                    int dX = (mouse_down.X - e.X);
                    int dY = (mouse_down.Y - e.Y);
                    pnlProducts.AutoScrollPosition = new Point((dX - pnlProducts.AutoScrollPosition.X), (dY - pnlProducts.AutoScrollPosition.Y));
                }else{

                    switched = true;

                    if((temp.Y - pnlProducts.AutoScrollPosition.Y) < 0){
                        if(currentpage > 0){
                            FillDataGrid(currentpage - 1, currentmode, ((DataTable)dgdProducts.DataSource).ExtendedProperties["param"]);
                        }else{
                            //put the scroll back, there is no -1 page
                            pnlProducts.AutoScrollPosition = new Point(0, 540);
                        }
                    }else{
                        if(dgdProducts.Rows.Count > 35){
                            FillDataGrid(currentpage + 1, currentmode, ((DataTable)dgdProducts.DataSource).ExtendedProperties["param"]);
                        }else{
                            //put the scroll back there are no more pages
                            pnlProducts.AutoScrollPosition = new Point(0, 540);
                        }
                    }

                }

            }

        }

        private void dgdProducts_MouseUp(object sender, MouseEventArgs e){

            //put the scroll back
            if(!switched){
                pnlProducts.AutoScrollPosition = new Point(0, 540);
            }

        }

        #endregion

    }

}
