﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */

using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using POS.Framework;
using POS.ProductPlugin.Objects;


namespace POS.ProductPlugin
{

    //
    //PRODUCT WIZARD FORM
    //
    internal partial class ProductWindow : Form
    {

        public Product prod = new Product();

        [DllImport("user32.dll")]
        static extern IntPtr GetFocus();
        [DllImport("user32.dll")]
        static extern bool CreateCaret(IntPtr hWnd, IntPtr hBitmap, int nWidth, int nHeight);
        [DllImport("user32.dll")]
        static extern bool ShowCaret(IntPtr hWnd);

        private void ShowCustomCaret(IntPtr h, int width, int height)
        {
            CreateCaret(h, (IntPtr)0, width, height);
            ShowCaret(h);
        }

        internal ProductWindow()
        {

            InitializeComponent();

        }

        private void All_Labels_Paint(object sender, EventArgs e)
        {
            ((Label)sender).Left = (this.ClientSize.Width / 2) - (((Label)sender).Width / 2);
        }

        private void ProductWindow_FormClosed(object sender, FormClosedEventArgs e)
        {

            tabProdotto.SelectedIndex = 0;
            btnProdNext.Text = resources.GetString("btnProdNext.Text");
        }

        private void ProductWindow_Shown(object sender, EventArgs e)
        {

            if (tabProdotto.SelectedIndex == 5)
            {
                btnProdNext.BackColor = Color.LightGreen;
                btnProdNext.Text = resources.GetString("btnProdNextSavetxt");
                txtProdQty.Text = "1";
                txtProdQty.Focus();
                txtProdQty.SelectionLength = 10;
            }
            else
            {
                txtProdName.Focus();
                txtProdName.SelectionLength = 0;
            }

        }

        private void ProductWindow_Load(object sender, EventArgs e)
        {

            this.cmbCat.DataSource = Product.GetRep();
            this.cmbCat.DisplayMember = "products_model";
            this.cmbCat.ValueMember = "products_id";

            Engine.ShowKeyboard();

            txtProdName.DataBindings.Clear();
            txtProdName.DataBindings.Add("text", prod, "Name");
            txtProdEan.DataBindings.Clear();
            txtProdEan.DataBindings.Add("text", prod, "EAN");
            txtProdPrice.DataBindings.Clear();
            txtProdPrice.DataBindings.Add("text", prod, "Price");
            txtProdAli.DataBindings.Clear();
            txtProdAli.DataBindings.Add("text", prod, "Aliquota");
            chkProdFis.DataBindings.Clear();
            chkProdFis.DataBindings.Add("checked", prod, "Fiscale");


            if (!String.IsNullOrEmpty(this.prod.Name) && this.prod.Categoria == 0)
            {
                cmbCat.Enabled = false;
            }
            else
            {
                //////////////////////////////////////
                //riabilito i comandi disabilitati dal combo reparti
                //////////////////////////////////////
                cmbCat.Enabled = true;
            }

        }

        private void txtProdName_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtProdName.Handle, 4, 29);
        }

        private void txtProdEan_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtProdEan.Handle, 4, 29);
        }

        private void txtProdPrice_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtProdPrice.Handle, 4, 29);
        }

        private void txtProdAli_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtProdAli.Handle, 4, 29);
        }

        private void txtProdQty_GotFocus(object sender, EventArgs e)
        {
            ShowCustomCaret(txtProdQty.Handle, 4, 36);
        }

        private void txtProdPrice_KeyPress(object sender, KeyPressEventArgs e)
        {
            //Handles txtProdPrice.KeyPress
            //Handles txtProdAli.KeyPress
            //Handles txtProdQty.KeyPress
            //Handles txtProdEan.KeyPress
            if (Char.IsDigit(e.KeyChar) == true || e.KeyChar == (char)8)
            {
                e.Handled = false;
            }
            else
            {
                e.Handled = true;
            }

        }

        private void txtProdName_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Char.IsPunctuation(e.KeyChar) || (Char.IsWhiteSpace(e.KeyChar) && txtProdName.TextLength == 0))
            {
                e.Handled = true;
            }
        }

        private void txtProdName_TextChanged(object sender, EventArgs e)
        {
            if (txtProdName.Text.Length > 22)
            {
                txtProdName.Text = txtProdName.Text.Substring(0, 22);
                txtProdName.SelectionStart = 22;
            }
        }

        private void btnProdNext_Click(object sender, EventArgs e)
        {

            if (tabProdotto.SelectedIndex == 5)
            {
                prod.Quantity = double.Parse(this.txtProdQty.Text);
                this.prod.Save();
                this.prod = new Product();
                Engine.HideKeyboard();
                btnProdNext.BackColor = SystemColors.Control;
                btnProdNext.Text = resources.GetString("btnProdNext.Text");
                this.Hide();
                this.DialogResult = DialogResult.OK;
                return;
            }
            else if (tabProdotto.SelectedIndex == 3 && cmbCat.SelectedValue == (object)prod.Id)
            {
                MessageBox.Show(resources.GetString("reperrortxt"));
                return;
            }
            else if (tabProdotto.SelectedIndex == 0)
            {

                if (txtProdName.Text.Equals("")) return;

                if (Product.CheckName(txtProdName.Text) > 0 && this.prod.IsNew)
                {
                    MessageBox.Show(resources.GetString("nameerrortxt"));
                    return;
                }

            }
            else if (tabProdotto.SelectedIndex == 1 && !txtProdEan.Text.Equals("") && this.prod.IsNew)
            {
                if (Product.CheckCode(txtProdEan.Text) > 0)
                {
                    MessageBox.Show(resources.GetString("codeerrortxt"));
                    return;
                }
            }

            tabProdotto.SelectedIndex += 1;

        }

        private void btnProdBack_Click(object sender, EventArgs e)
        {
            if (tabProdotto.SelectedIndex > 0)
            {
                tabProdotto.SelectedIndex -= 1;
            }
        }

        private void btnProdCancel_Click(Object sender, EventArgs e)
        {
            this.prod = new Product();
            Engine.HideKeyboard();
            this.Hide();
        }

        private void tabProdotto_SelectedIndexChanged(object sender, EventArgs e)
        {

            switch (tabProdotto.SelectedIndex)
            {
                case 0:
                    txtProdName.Focus();
                    txtProdName.SelectionLength = 0;
                    btnProdNext.BackColor = SystemColors.Control;
                    break;
                case 1:
                    txtProdEan.Focus();
                    txtProdEan.SelectionLength = 0;
                    break;
                case 2:
                    txtProdPrice.Focus();
                    txtProdPrice.SelectionLength = 0;
                    break;
                case 3:
                    cmbCat.SelectedValue = prod.Categoria;
                    break;
                case 4:
                    prod.Categoria = Convert.ToInt32(cmbCat.SelectedValue);
                    if (prod.Categoria > 0 && prod.IsNew)
                    {
                        Product temp = new Product(prod.Categoria);
                        txtProdAli.Text = temp.Aliquota.ToString();
                        prod.Fiscale = temp.Fiscale;
                        chkProdFis.Focus();
                        //chkProdFis.Enabled = false;
                    }
                    txtProdAli.Focus();
                    txtProdAli.SelectionLength = 0;
                    btnProdNext.BackColor = SystemColors.Control;
                    btnProdNext.Text = resources.GetString("btnProdNext.Text");
                    break;
                case 5:
                    txtProdQty.Text = "0";
                    txtProdQty.SelectionLength = 10;
                    btnProdNext.BackColor = Color.LightGreen;
                    btnProdNext.Text = resources.GetString("btnProdNextSavetxt");
                    break;
            }
        }

    }

}
